@@ -48,9 +48,17 @@ public:
 	void			setData(Fatmap * image) { m_image = image; }
    	Fatmap *			getData(void) const { return m_image; }
 
+   	virtual GRType		getType() const { return m_grtype; }
+   	virtual bool		render(GR_Graphics *pGR, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 private:
-
 	Fatmap * m_image;
+
+	GRType m_grtype;
+
+	UT_sint32 m_iDisplayWidth;
+	UT_sint32 m_iDisplayHeight;
+
+	bool _convertPNGFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 };
 
 #endif /* GR_UNIXIMAGE_H */
