@@ -47,6 +47,8 @@ XAP_PrefsScheme *GR_Graphics::m_pPrefsScheme = 0;
 UT_uint32 GR_Graphics::m_uTick = 0;
 
 UT_uint32 GR_Graphics::m_instanceCount = 0;
+UT_uint32 GR_Graphics::s_iScreenResolution = 100;
+
 
 GR_Font::GR_Font() 
 {
@@ -82,12 +84,23 @@ GR_Graphics::~GR_Graphics()
 
 	if(m_instanceCount == 0)
 	{
-		delete m_pRemapGlyphsTableSrc;
-		delete m_pRemapGlyphsTableDst;
+		delete[] m_pRemapGlyphsTableSrc;
+		delete[] m_pRemapGlyphsTableDst;
 	}
 
 }
 
+UT_uint32 GR_Graphics::s_getScreenResolution() 
+{
+  return s_iScreenResolution ;
+}
+
+void GR_Graphics::setStaticScreenResolution(UT_uint32 iRes)
+{
+  s_iScreenResolution = iRes;
+}
+
+
 void GR_Graphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 {
 }
@@ -106,7 +119,7 @@ UT_uint32 GR_Graphics::getMaxCharacterWidth(const UT_UCSChar*s, UT_uint32 Length
 			MaxWidth = pWidths[i];
 	}
 
-	delete pWidths;
+	DELETEPV(pWidths);
 
 	return MaxWidth;
 
@@ -226,10 +239,11 @@ UT_UCSChar GR_Graphics::remapGlyph(const UT_UCSChar actual_, bool noMatterWhat)
 		UT_uint32 doublelength;
 		doublelength = gb.getLength();
 		m_iRemapGlyphsTableLen = doublelength / 2;
-		delete m_pRemapGlyphsTableSrc;
-		delete m_pRemapGlyphsTableDst;
-		m_pRemapGlyphsTableSrc = 0;
-		m_pRemapGlyphsTableDst = 0;
+
+		// free these resources
+		DELETEPV(m_pRemapGlyphsTableSrc);
+		DELETEPV(m_pRemapGlyphsTableDst);
+
 		if (m_iRemapGlyphsTableLen)
 		{
 			m_pRemapGlyphsTableSrc = new UT_UCSChar[m_iRemapGlyphsTableLen];
@@ -243,7 +257,13 @@ UT_UCSChar GR_Graphics::remapGlyph(const UT_UCSChar actual_, bool noMatterWhat)
 			}
 		}
 	}
-	if (!m_bRemapGlyphsMasterSwitch) return (actual);
+	if (!m_bRemapGlyphsMasterSwitch
+		|| actual == 0x200B  // zero width space
+		|| actual == 0xFEFF) // zero width non-breaking space
+	{
+		return (actual);
+	}
+	
 	UT_uint32 width = 0xFFFF;
 	if (noMatterWhat  ||  m_bRemapGlyphsNoMatterWhat  ||  !(width = measureUnRemappedChar(actual)))
 	{
@@ -304,11 +324,10 @@ UT_uint32 GR_Graphics::getAppropriateFontSizeFromString(const char * pszFontSize
 		m_bLayoutResolutionModeEnabled = false;
 		iSize = convertDimension(pszFontSize);
 		m_bLayoutResolutionModeEnabled = curRes;
-		dSize = iSize;
-		double rat = (double) UT_LAYOUT_UNITS / (double) getResolution();
-		iSizeLayout = (UT_uint32) (dSize * rat + 0.5);
 		if( m_bLayoutResolutionModeEnabled)
 		{
+			iSizeLayout = (UT_uint32) ((double ) UT_LAYOUT_UNITS * UT_convertToInches(pszFontSize) +0.05);
+			xxx_UT_DEBUGMSG(("SEVIOR: iSizeLayout in gr_graphics = %d \n",iSizeLayout));
 			return iSizeLayout;
 		} 
 		return iSize;
@@ -319,17 +338,11 @@ UT_uint32 GR_Graphics::getAppropriateFontSizeFromString(const char * pszFontSize
 //
 	else
 	{
-		double dScreenSize = UT_convertToInches(pszFontSize) * (double) getScreenResolution();
-		UT_uint32 iScreenSize = (UT_uint32) (dScreenSize + 0.05);
-		dScreenSize = (double) iScreenSize;
-
-		double ratToLayout = (double) UT_LAYOUT_UNITS / (double) getScreenResolution();
-		UT_sint32 iSizeLayout = (UT_sint32) (dScreenSize * ratToLayout + 0.5);
-
-		double dPaperSize = dScreenSize * (double) getResolution() / (double) getScreenResolution();
+		double dPaperSize = UT_convertToInches(pszFontSize) * (double) getResolution();
 		iSize = (UT_sint32) (dPaperSize + 0.05);
 		if( m_bLayoutResolutionModeEnabled)
 		{
+			iSizeLayout = (UT_uint32) ((double ) UT_LAYOUT_UNITS * UT_convertToInches(pszFontSize) +0.05);
 			return iSizeLayout;
 		} 
 		return iSize;
@@ -346,13 +359,13 @@ UT_sint32 GR_Graphics::convertDimension(const char * s) const
 	double dInches = UT_convertToInches(s);
 	double dResolution;
 	if(m_bLayoutResolutionModeEnabled)
-		{
+	{
 		dResolution = UT_LAYOUT_UNITS;
-		}
+	}
 	else
-		{
+	{
 		dResolution = getResolution();		// NOTE: assumes square pixels/dpi/etc.
-		}
+	}
 
 	/* 
 	 We do this because the convert Dimension code is actually
@@ -660,6 +673,7 @@ void GR_Graphics::doRepaint( UT_Rect * rClip)
 //
 // Look if we have a pending expose left over.
 //
+	xxx_UT_DEBUGMSG(("SEVIOR: Starting doRepaint \n"));
 	while(isSpawnedRedraw())
 	{
 		UT_usleep(100);
@@ -669,7 +683,7 @@ void GR_Graphics::doRepaint( UT_Rect * rClip)
 //
 	setDontRedraw(true);
 //
-// Gte a lock on the expose rectangle
+// Get a lock on the expose rectangle
 //
 	while(isExposedAreaAccessed())
 	{
@@ -707,6 +721,7 @@ void GR_Graphics::doRepaint( UT_Rect * rClip)
 // Allow the repainter to paint
 //
 	setDontRedraw(false);
+	xxx_UT_DEBUGMSG(("SEVIOR: Finished doRepaint \n"));
 //
 // OK this event is handled.
 //
