@@ -1,3 +1,22 @@
+/* AbiSource Program Utilities
+ * Copyright (C) 2001 Tomas Frydrych
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 /*
  !!! PLEASE, USE ONLY C-STYLE COMMENTS IN THIS FILE !!!
 */
@@ -55,6 +74,13 @@
 	#elif BYTE_ORDER == BIG_ENDIAN
 		#define UT_BIG_ENDIAN
 	#endif
+#elif defined(_AIX)
+        #include <sys/machine.h>
+        #if BYTE_ORDER == LITTLE_ENDIAN
+                #define UT_LITTLE_ENDIAN
+        #elif BYTE_ORDER == BIG_ENDIAN
+                #define UT_BIG_ENDIAN
+        #endif
 #else /* this is for Linux */
 	#include <endian.h>
 	#if __BYTE_ORDER == __LITTLE_ENDIAN		
