@@ -44,6 +44,8 @@
 #include "fg_Graphic.h"
 #include "fv_View.h"
 
+#include "libgnomevfs/gnome-vfs.h"
+
 /*****************************************************************/
 
 #define ENSUREP(p)		do { UT_ASSERT(p); if (!p) goto Cleanup; } while (0)
@@ -53,6 +55,7 @@
 static int
 s_mapMimeToUriType (const char * mime)
 {
+	UT_DEBUGMSG(("SEVIOR: mime %s dropped into AbiWord \n",mime));
 	if (!UT_strcmp (mime, "application/rtf")) 
 		return XAP_UnixGnomeFrame::TARGET_URI_LIST;
 	if (!UT_strcmp (mime, "application/msword")) 
@@ -65,8 +68,20 @@ s_mapMimeToUriType (const char * mime)
 		return XAP_UnixGnomeFrame::TARGET_URI_LIST;
 	if (!UT_strcmp (mime, "image/png"))
 		return XAP_UnixGnomeFrame::TARGET_PNG;		
+	if (!UT_strcmp (mime, "image/jpeg"))
+		return XAP_UnixGnomeFrame::TARGET_JPG;		
+	if (!UT_strcmp (mime, "image/gif"))
+		return XAP_UnixGnomeFrame::TARGET_GIF;		
+	if (!UT_strcmp (mime, "image/x-xpixmap"))
+		return XAP_UnixGnomeFrame::TARGET_XPM;		
 	if (!UT_strcmp (mime, "image/bmp")) 
 		return XAP_UnixGnomeFrame::TARGET_BMP;
+	if (!UT_strcmp (mime, "image/x-bmp")) 
+		return XAP_UnixGnomeFrame::TARGET_BMP;
+	if (!UT_strcmp (mime, "image/x-cmu-raster")) 
+		return XAP_UnixGnomeFrame::TARGET_RAS;
+	if (!UT_strcmp (mime, "image/tiff")) 
+		return XAP_UnixGnomeFrame::TARGET_TIF;
 	if (!UT_strcmp (mime, "image/svg-xml")) 
 		return XAP_UnixGnomeFrame::TARGET_SVG;
 	if (!UT_strcmp (mime, "text/plain")) 
@@ -74,7 +89,7 @@ s_mapMimeToUriType (const char * mime)
 	if (!UT_strcmp (mime, "text/abiword")) 
 		return XAP_UnixGnomeFrame::TARGET_URI_LIST;
 	if (!UT_strcmp (mime, "text/html")) 
-		return XAP_UnixGnomeFrame::TARGET_URI_LIST;
+		return XAP_UnixGnomeFrame::TARGET_URL;
 	if (!UT_strcmp (mime, "text/xml")) 
 		return XAP_UnixGnomeFrame::TARGET_URI_LIST;
 	if (!UT_strcmp (mime, "text/vnd.wap.wml")) 
@@ -110,18 +125,125 @@ void XAP_UnixGnomeFrame::_dnd_drop_event(GtkWidget        *widget,
 	FG_Graphic * pFG;
 	IEGraphicFileType iegft;
 	UT_Error error;
-	
+	UT_DEBUGMSG(("SEVIOR: _dnd_drop_event being handled \n"));
 	pUnixFrame = (XAP_UnixFrame *) gtk_object_get_user_data(GTK_OBJECT(widget));
 	pApp = pUnixFrame->getApp ();
 
-	names = gnome_uri_list_extract_filenames ((char *) selection_data->data);
-
+	char * rawChar = (char *) selection_data->data;
+	UT_DEBUGMSG(("SEVIOR: text in selection = %s \n", rawChar));
+	names = gnome_uri_list_extract_filenames (rawChar);
+    bool bDropHTML = (strstr(rawChar,"html") !=0) || (strstr(rawChar,"htm") != 0);
 	if (!names)
-		return;
-	
-	for (; names; names = names->next) {
-		filename = (char *) names->data;
+	{
+		UT_DEBUGMSG(("SEVIOR: No filename found in drop event \n"));
+//
+// Drag an image into AbiWord. Use our magic auto -detection of image types
+//
+		UT_DEBUGMSG(("SEVIOR: Load image into Abiword \n"));
+		GnomeVFSResult    result;
+		GnomeVFSHandle   *handle;
+		gchar             buffer[1024];
+		GnomeVFSFileSize  bytes_read;
+		GnomeVFSURI 	 *uri;
+		UT_String stripped;
+		UT_uint32 i =0;
+		for(i=0; i < strlen(rawChar); i++)
+		{
+			if(rawChar[i] != '[' && rawChar[i] != ']')
+			{
+				stripped += rawChar[i];
+			}
+		}
+		uri = gnome_vfs_uri_new (stripped.c_str());
+		if (uri == NULL) 
+		{
+			UT_DEBUGMSG(("SEVIOR: Invalid uri text was %s \n",stripped.c_str()));
+			return;
+		}
+		result = gnome_vfs_open_uri (&handle, uri, GNOME_VFS_OPEN_READ);
+//
+// Read a chunk to determine if this is an image.
+//
+		if(result == GNOME_VFS_OK)
+		{
+			UT_DEBUGMSG(("SEVIOR: uri %s openned OK \n",stripped.c_str()));
+			result = gnome_vfs_read (handle, buffer, sizeof buffer - 1,
+									 &bytes_read);
+		}
+		if(result==GNOME_VFS_OK && !bDropHTML)
+		{
+			UT_DEBUGMSG(("SEVIOR:  uri %s read OK \n",stripped.c_str()));
+			UT_ByteBuf * Bytes = new UT_ByteBuf();
+			Bytes->append((UT_Byte *)buffer,(UT_uint32) bytes_read);
+			iegft = IEGFT_Unknown;
+			error = IE_ImpGraphic::constructImporter(Bytes, iegft, &pIEG);
+			if(!error)
+			{
+//
+// We have an image! Load it into a byte buffer and import it into Abiword
+//
+				UT_DEBUGMSG(("SEVIOR: uri gave valid image data \n"));
+				while( result==GNOME_VFS_OK ) 
+				{
+					result = gnome_vfs_read (handle, buffer, sizeof buffer - 1,
+										 &bytes_read);
+					if(!bytes_read) break;
+					Bytes->append((UT_Byte *) buffer,(UT_uint32) bytes_read);
+				}
+			
+				error = pIEG->importGraphic(Bytes, &pFG);
+				if(error)
+				{
+					UT_DEBUGMSG(("Sevior: could not import graphic (%d)\n", error));
+					DELETEP(pIEG);
+					return;
+				}
+			
+				DELETEP(pIEG);
+			
+				FV_View * pView = static_cast<FV_View*>(pFrame->getCurrentView());
+				error = pView->cmdInsertGraphic(pFG, rawChar);
+				if (error)
+				{
+					UT_DEBUGMSG(("Sevior: could not insert graphic (%d)\n", error));
+					DELETEP(pFG);
+					return;
+				}
+				DELETEP(pFG);
+				return;
+			}
+			else
+			{
+				delete Bytes;
+			}
+			result = gnome_vfs_close (handle);
+			g_free (uri);
+		}
+			
+		int type = s_mapMimeToUriType (gnome_mime_type_or_default (rawChar, "foobar"));
+		if(type== TARGET_URL)
+		{
+			UT_DEBUGMSG(("DOM: target url\n"));
+			FV_View * pView = (FV_View *) pUnixFrame->getCurrentView();
+			pView->cmdInsertHyperlink(rawChar);
+		}
+		else if (strstr(rawChar,"http:") !=0)
+		{
+			UT_DEBUGMSG(("Sevior: Overiding dumb gnome parser Inserting URL \n"));
+			FV_View * pView = (FV_View *) pUnixFrame->getCurrentView();
+			pView->cmdInsertHyperlink(rawChar);
+			return;
+		}
+		else
+		{
+			return;
+		}
+	}
 
+	for (; names; names = names->next) 
+	{
+		filename = (char *) names->data;
+		UT_DEBUGMSG(("SEVIOR: Selection %s found \n",filename));
 		int type = s_mapMimeToUriType (gnome_mime_type_or_default (filename, "foobar"));
 		switch (type) {
 
@@ -152,6 +274,8 @@ void XAP_UnixGnomeFrame::_dnd_drop_event(GtkWidget        *widget,
 			// TODO
 
 			UT_DEBUGMSG(("DOM: target url\n"));
+			FV_View * pView = (FV_View *) pUnixFrame->getCurrentView();
+			pView->cmdInsertHyperlink(filename);
 #if 0
 			error = pNewUnixFrame->loadDocument(filename, IEFT_Unknown);
 			
@@ -165,6 +289,11 @@ void XAP_UnixGnomeFrame::_dnd_drop_event(GtkWidget        *widget,
 		case TARGET_PNG:
 		case TARGET_SVG:
 		case TARGET_BMP:
+		case TARGET_JPG:
+		case TARGET_GIF:
+		case TARGET_XPM:
+		case TARGET_RAS:
+		case TARGET_TIF:
 		{
 			if (type == TARGET_PNG)
 			{
@@ -174,10 +303,14 @@ void XAP_UnixGnomeFrame::_dnd_drop_event(GtkWidget        *widget,
 			{
 				iegft = IEGFT_BMP;
 			}
-			else
+			else if (type == TARGET_SVG)
 			{
 				iegft = IEGFT_SVG;
 			}
+			else
+			{
+				iegft = IEGFT_Unknown;
+			}
 			
 			error = IE_ImpGraphic::constructImporter(filename, iegft, &pIEG);
 			if(error)
@@ -217,6 +350,18 @@ void XAP_UnixGnomeFrame::_dnd_drop_event(GtkWidget        *widget,
 	gnome_uri_list_free_strings (names);
 }
 
+void XAP_UnixGnomeFrame::_dnd_real_drop_event (GtkWidget *widget, GdkDragContext * context, gint x, gint y, guint time, gpointer ppFrame)
+{
+	UT_DEBUGMSG(("SEVIOR: drop event on Gnome frame. \n"));
+	GdkAtom selection = gdk_drag_get_selection(context);
+	gtk_drag_get_data (widget,context,selection,time);
+}
+
+void XAP_UnixGnomeFrame::_dnd_drag_end(GtkWidget  *widget, GdkDragContext *context, gpointer ppFrame)
+{
+	UT_DEBUGMSG(("SEVIOR: drop end event on Gnome frame. \n"));
+}
+
 /*****************************************************************/
 
 XAP_UnixGnomeFrame::XAP_UnixGnomeFrame(XAP_UnixGnomeApp * app)
@@ -249,7 +394,7 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 	bool bResult;
 	static GtkTargetEntry drag_types[] =
 	{
-#if 0
+#if 1
 		// i'm concerned that DND doesn't work for mime-types other
 		// than text/uri-list. i can't get this to behave well with
         // nautilus or gmc
@@ -268,28 +413,49 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 		{"text/vnd.wap.wml", 0, TARGET_URI_LIST},
 		{"text/richtext", 0, TARGET_URI_LIST},
 		{"text/rtf", 0, TARGET_URI_LIST},
-		{"_NETSCAPE_URL", 0, TARGET_URL}
+		{"_NETSCAPE_URL", 0, TARGET_URL},
+		{"text/uri-list", 0, TARGET_URI_LIST}
 #else
 		{"text/uri-list", 0, TARGET_URI_LIST}
 #endif
 	};
 	static gint n_drag_types = sizeof(drag_types)/sizeof(drag_types[0]);
 
-	m_wTopLevelWindow = gnome_app_new((gchar *)(m_pUnixApp->getApplicationName()),
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		m_wTopLevelWindow = gnome_app_new((gchar *)(m_pUnixApp->getApplicationName()),
 									  (gchar *)(m_pUnixApp->getApplicationTitleForTitleBar()));
+		gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic_attr", NULL);
+		gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "ic", NULL);
+		gtk_window_set_policy(GTK_WINDOW(m_wTopLevelWindow), TRUE, TRUE, FALSE);
+		gtk_window_set_wmclass(GTK_WINDOW(m_wTopLevelWindow),
+						   m_pUnixApp->getApplicationName(),
+						   m_pUnixApp->getApplicationName());
+	} 
 	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "toplevelWindow",
 						m_wTopLevelWindow);
 	gtk_object_set_user_data(GTK_OBJECT(m_wTopLevelWindow),this);
-	gtk_window_set_policy(GTK_WINDOW(m_wTopLevelWindow), TRUE, TRUE, FALSE);
-	gtk_window_set_wmclass(GTK_WINDOW(m_wTopLevelWindow),
-						   m_pUnixApp->getApplicationName(),
-						   m_pUnixApp->getApplicationName());
 
 	gtk_drag_dest_set (m_wTopLevelWindow,
 					   GTK_DEST_DEFAULT_ALL,
 					   drag_types, 
 					   n_drag_types, 
 					   GDK_ACTION_COPY);
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+
+		gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "realize",
+ 					   GTK_SIGNAL_FUNC(_fe::realize), NULL);
+		gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "unrealize",
+ 					   GTK_SIGNAL_FUNC(_fe::unrealize), NULL);
+	}
+	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "size_allocate",
+ 					   GTK_SIGNAL_FUNC(_fe::sizeAllocate), NULL);
+  
+ 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "focus_in_event",
+ 					   GTK_SIGNAL_FUNC(_fe::focusIn), NULL);
+ 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "focus_out_event",
+ 					   GTK_SIGNAL_FUNC(_fe::focusOut), NULL);
 
 	gtk_signal_connect (GTK_OBJECT (m_wTopLevelWindow),
 						"drag_data_get",
@@ -300,6 +466,16 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
                        GTK_SIGNAL_FUNC(_dnd_drop_event), 
 					   (gpointer)this);
 
+  	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), 
+  					   "drag_drop",
+                         GTK_SIGNAL_FUNC(_dnd_real_drop_event), 
+					   (gpointer)this);
+
+  	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), 
+  					   "drag_end",
+                         GTK_SIGNAL_FUNC(_dnd_drag_end), 
+					   (gpointer)this);
+
 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "delete_event",
 					   GTK_SIGNAL_FUNC(_fe::delete_event), NULL);
 	// here we connect the "destroy" event to a signal handler.  
@@ -313,13 +489,20 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 					   GTK_SIGNAL_FUNC(_fe::focus_out_event), NULL);
 
 	// create a VBox inside it.
-	
+
 	m_wVBox = gtk_vbox_new(FALSE,0);
 	gtk_object_set_data(GTK_OBJECT(m_wTopLevelWindow), "vbox", m_wVBox);
 	gtk_object_set_user_data(GTK_OBJECT(m_wVBox),this);
-	gnome_app_set_contents(GNOME_APP(m_wTopLevelWindow), m_wVBox);
-
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		gnome_app_set_contents(GNOME_APP(m_wTopLevelWindow), m_wVBox);
+	}
+	else
+	{
+		gtk_container_add(GTK_CONTAINER(m_wTopLevelWindow), m_wVBox);
+	}
 	// synthesize a menu from the info in our base class.
+
 	m_pUnixMenu = new EV_UnixGnomeMenuBar(m_pUnixApp,this,
 										  m_szMenuLayoutName,
 										  m_szMenuLabelSetName);
@@ -333,11 +516,18 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 	// create a toolbar instance for each toolbar listed in our base class.
 	// TODO for some reason, the toolbar functions require the TLW to be
 	// TODO realized (they reference m_wTopLevelWindow->window) before we call them.
-	gtk_widget_realize(m_wTopLevelWindow);
-
+	if(m_iFrameMode == XAP_NormalFrame)
+	{
+		gtk_widget_realize(m_wTopLevelWindow);
+	}
+	else
+	{
+		gtk_widget_realize(m_wVBox);
+	}
 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "key_press_event",
 					   GTK_SIGNAL_FUNC(_fe::key_press_event), NULL);
 
+
 	_createToolbars();
 
 	// Let the app-specific frame code create the contents of
@@ -352,14 +542,19 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 	// if it wants to.  we will put it below the document
 	// window (a peer with toolbars and the overall sunkenbox)
 	// so that it will appear outside of the scrollbars.
-
+	m_wStatusBar = NULL;
 	m_wStatusBar = _createStatusBarWindow();
-	if (m_wStatusBar)
+
+	if (m_iFrameMode == XAP_NormalFrame)
 	{
 		gnome_app_set_statusbar(GNOME_APP(m_wTopLevelWindow), m_wStatusBar);
-		gtk_widget_show(m_wStatusBar);
 	}
-	
+	else
+	{
+		gtk_container_add(GTK_CONTAINER(m_wTopLevelWindow), m_wVBox);
+	}
+	gtk_widget_show(m_wStatusBar);
+
 	gtk_widget_show(m_wVBox);
 	
 	// set the icon
@@ -397,15 +592,95 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 
 	// we let our caller decide when to show m_wTopLevelWindow.
 
-
 	UT_ASSERT(m_pUnixMenu);
 
-
 	return;
 }
 
+void XAP_UnixGnomeFrame::rebuildToolbar(UT_uint32 ibar)
+{
+//
+// Destroy the old toolbar
+//
+	GtkOrientation bOrient;
+	const char* szGtkName = NULL;
+	EV_Toolbar * pToolbar = (EV_Toolbar *) m_vecToolbars.getNthItem(ibar);
+	const char * szTBName = (const char *) m_vecToolbarLayoutNames.getNthItem(ibar);
+	EV_UnixGnomeToolbar * pUTB = static_cast<EV_UnixGnomeToolbar *>( pToolbar);
+	GnomeDockItem * wDockItem = pUTB->destroy(&bOrient,&szGtkName);
+//
+// Delete the old class
+//
+	delete pToolbar;
+//
+// Build a new one.
+//
+	pToolbar = _newToolbar(m_app, (XAP_Frame *) this,szTBName,
+						   (const char *) m_szToolbarLabelSetName);
+	static_cast<EV_UnixGnomeToolbar *>(pToolbar)->rebuildToolbar(wDockItem, 
+																 bOrient, 
+																 szGtkName);
+	delete [] szGtkName;
+	m_vecToolbars.setNthItem(ibar, (void *) pToolbar, NULL);
+//
+// Refill the framedata pointers
+//
+	refillToolbarsInFrameData();
+	repopulateCombos();
+}
+
+void  XAP_UnixGnomeFrame::rebuildMenus(void)
+{
+	((EV_UnixGnomeMenuBar *) m_pUnixMenu)->destroy();
+//
+// Delete the old class
+//
+	DELETEP(m_pUnixMenu);
+//
+// Build a new one.
+//
+	m_pUnixMenu = new EV_UnixGnomeMenuBar(m_pUnixApp,this,
+									 m_szMenuLayoutName,
+									 m_szMenuLabelSetName);
+	UT_ASSERT(m_pUnixMenu);
+
+	((EV_UnixGnomeMenuBar *) m_pUnixMenu)->synthesizeMenuBar();
+}
+
 bool XAP_UnixGnomeFrame::openURL(const char * szURL)
 {
+#if 0
+	GnomeVFSResult    result;
+	GnomeVFSHandle   *handle;
+	gchar             buffer[1024];
+	GnomeVFSFileSize  bytes_read;
+	GnomeVFSURI 	 *uri;
+
+	uri = gnome_vfs_uri_new (szURL);
+	if (uri == NULL) 
+	{
+		gnome_url_show(szURL);
+		return false;
+	}
+	result = gnome_vfs_open_uri (&handle, uri, GNOME_VFS_OPEN_READ);
+//
+// Read a chunk to determine if this is an image.
+//
+	if(result == GNOME_VFS_OK)
+	{
+		result = gnome_vfs_read (handle, buffer, sizeof buffer - 1,
+									 &bytes_read);
+		if(result==GNOME_VFS_OK)
+		{
+			UT_ByteBuf * Bytes = new UT_ByteBuf();
+			Bytes->append((UT_Byte *)buffer,(UT_uint32) bytes_read);
+//
+// TODO load document via gnome_vfs
+//
+		}
+	}
+
+#endif
 	gnome_url_show(szURL);
 	return false;
 }
@@ -444,7 +719,11 @@ bool XAP_UnixGnomeFrame::runModalContextMenu(AV_View *  pView, const char * szMe
 		gnome_popup_menu_do_popup_modal (GTK_WIDGET (pUnixPopup->getMenuHandle ()),
 										 NULL, NULL,bevent,(void *) bevent);
  	}
-
+	XAP_Frame * pFrame = (XAP_Frame *) this;
+	if (pFrame->getCurrentView())
+	{
+		pFrame->getCurrentView()->focusChange( AV_FOCUS_HERE);
+	}
  	DELETEP(pUnixPopup);
  	return bResult;
 }
