@@ -188,17 +188,6 @@ GnomeFont * XAP_UnixGnomePrintGraphics::_allocGnomeFont(PSFont* pFont)
 	
 		// ok, this is the ugliest hack of the year, so I'll take it one step
 		// at a time
-#if 0 // This code gratututously removed by Sevior!! GnomePrint works fine at all scales!
-		// add 0.0001 so 11.9999 gets rounded up to 12
-		double size         = (double)pFont->getSize() * _scale_factor_get () + 0.0001;
-	
-		// test for oddness, if odd, subtract 1
-		// why? abi allows odd point fonts for at least 
-		// 9 and 11 points. gnome print does not, so we
-		// scale it down. *ugly*
-		if((int)size % 2 != 0)
-				size -= 1.0;
-#endif
 		double size         = (double)pFont->getSize() * _scale_factor_get ();
 
 		// first try to directly allocate abi's name
@@ -483,8 +472,7 @@ UT_uint32 XAP_UnixGnomePrintGraphics::measureUnRemappedChar(const UT_UCSChar c)
 	UT_uint32 width = 0;
 	
         UT_ASSERT(m_pCurrentFont);
-	if (c >= 256)
-		return 0;
+
 	UT_UCSChar cc = c; 
 	//
 	// If the font is in cache we're doing layout calculations so use exactly
@@ -531,39 +519,45 @@ void XAP_UnixGnomePrintGraphics::drawChars(const UT_UCSChar* pChars,
 
 	xxx_UT_DEBUGMSG(("DOM: drawChars (x: %d) (y: %d)\n", xoff, yoff));
 
+	// push a graphics state & save it. then set the font
+	gnome_print_gsave ( m_gpc ) ;
+	gnome_print_setfont (m_gpc, m_pCurrentFont);
+
 	gnome_print_moveto(m_gpc, _scale_x_dir(xoff), _scale_y_dir(yoff));
 
 	pEnd = pChars + iCharOffset + iLength;
 	
 	for (pS = pChars + iCharOffset; pS < pEnd; pS += OUR_LINE_LIMIT) {
-			const UT_UCSChar * pB;
-			UT_UCSChar currentChar;
+			const UT_UCSChar * pB = 0;
    
 			pD = buf;
 			for (pB = pS; (pB < pS + OUR_LINE_LIMIT) && (pB < pEnd); pB++) {
-					currentChar = remapGlyph(*pB, *pB >= 256 ? true : false);
-					if(!m_bisSymbol && !m_bisDingbats)
+					UT_UCSChar currentChar = *pB; // normal, default case
+
+					if(m_bisSymbol)
 					{
-							currentChar = currentChar <= 0xff ? currentChar : XAP_EncodingManager::get_instance()->UToNative(currentChar);
-					}
-					else if(m_bisSymbol)
-					{
-//
-// Convert to Unicode..
-//
+							//
+							// Convert to Unicode..
+							//
+							//currentChar = remapGlyph(*pB, *pB >= 256 ? true : false);
 							currentChar = (UT_UCSChar) getUnicodeForSymbol((UT_uint32) currentChar);
 					}
 					else if(m_bisDingbats)
 					{
-//
-// Convert to Unicode..
-//
+							//
+							// Convert to Unicode..
+							//
+
+							//currentChar = remapGlyph(*pB, *pB >= 256 ? true : false);
 							currentChar = (UT_UCSChar) getUnicodeForDingbats((UT_uint32) currentChar);
 					}
 					pD += unichar_to_utf8 (currentChar, pD);
 			}
 			gnome_print_show_sized (m_gpc, (gchar *) buf, pD - buf);
 	}
+
+	// pop the graphics state
+	gnome_print_grestore ( m_gpc ) ;
 }
 
 void XAP_UnixGnomePrintGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
@@ -617,17 +611,9 @@ void XAP_UnixGnomePrintGraphics::setFont(GR_Font* pFont)
 
 	m_pCurrentFont = _allocGnomeFont(psFont);
 
-#if 0
-	XAP_UnixFont *uf          = static_cast<PSFont*>(pFont)->getUnixFont();
-	UT_DEBUGMSG(("Dom: setting font:\n"
-				 "\tsize returned: %f (requested %f)\n"
-				 "\tname returned: %s (requested %s)\n", 
-				 gnome_font_get_size(m_pCurrentFont),
-				 (double)static_cast<PSFont*>(pFont)->getSize() * _scale_factor_get(), 
-				 gnome_font_get_name(m_pCurrentFont), uf->getName()));
-#endif
-
-	gnome_print_setfont (m_gpc, m_pCurrentFont);
+	// we delay gnome_print_setfont until we draw characters. this fixes
+	// a bug with regard to font sizes < 10 pts in gnome-print version
+	// <= 0.35 at the cost of a slightly larger PS output file
 }
 
 bool XAP_UnixGnomePrintGraphics::queryProperties(GR_Graphics::Properties gp) const
@@ -635,6 +621,7 @@ bool XAP_UnixGnomePrintGraphics::queryProperties(GR_Graphics::Properties gp) con
 	switch (gp)
 	{
 	case DGP_SCREEN:
+	case DGP_OPAQUEOVERLAY:
 		return false;
 	case DGP_PAPER:
 		return true;
@@ -849,30 +836,16 @@ UT_uint32 XAP_UnixGnomePrintGraphics::_getResolution(void) const
         return GPG_RESOLUTION;
 }
 
-void XAP_UnixGnomePrintGraphics::fillRect(UT_RGBColor& c, UT_sint32 x, 
-										  UT_sint32 y, UT_sint32 w, 
-										  UT_sint32 h)
+void XAP_UnixGnomePrintGraphics::fillRect(UT_RGBColor& c, 
+										  UT_sint32 x, UT_sint32 y, 
+										  UT_sint32 w, UT_sint32 h)
 {
 		// draw background color
-		if(c.m_red == 255 && c.m_grn == 255 && c.m_blu == 255)
-		{
-				return;
-		}
 		gnome_print_setrgbcolor(m_gpc,
 								((double) c.m_red) / 255.0,
 								((double) c.m_grn) / 255.0,
 								((double) c.m_blu) / 255.0);
 
-#if 0
-		// adjust for the text's height
-		y += getFontDescent () + getFontHeight();
-		
-		/* Mirror gdk which excludes the far point */
-
-		w -= (int)_scale_x_dir (1);
-		h -= (int)_scale_y_dir (1);
-#endif
-
 		gnome_print_newpath (m_gpc);
 		gnome_print_moveto (m_gpc, _scale_x_dir(x),   _scale_y_dir(y));		
 		gnome_print_lineto (m_gpc, _scale_x_dir(x+w), _scale_y_dir(y));
@@ -1106,13 +1079,6 @@ UT_uint32 XAP_UnixGnomePrintGraphics::getFontHeight()
 UT_uint32 XAP_UnixGnomePrintGraphics::getFontHeight(GR_Font *fnt)
 {
 	UT_sint32 height = getFontAscent(fnt) + getFontDescent(fnt);
-#if 0
-	UT_DEBUGMSG(("Font height in gnome-print = %d \n",height));
-	PSFont * pFont = static_cast<PSFont *>(fnt);
-	XAP_UnixFont *uf          = pFont->getUnixFont();
-	char *abi_name            = (char*)uf->getName();
-	UT_DEBUGMSG(("Font size in Gnome-print = %d Font Name %s \n",pFont->getSize(),abi_name));
-#endif
 	return height;
 }
 
