@@ -70,7 +70,7 @@ class XAP_UnixGnomePrintGraphics : public GR_Graphics
 
 	virtual UT_uint32 measureUnRemappedChar(const UT_UCSChar c);
 	
-	virtual void setColor(UT_RGBColor& clr);
+	virtual void setColor(const UT_RGBColor& clr);
 	virtual GR_Font* getGUIFont();
 	virtual GR_Font* findFont(const char* pszFontFamily, 
 				  const char* pszFontStyle, 
