@@ -30,7 +30,7 @@
 #include <sys/stat.h>
 
 #include "ut_debugmsg.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #include "ut_string.h"
 
 #include "xap_Strings.h"
@@ -44,6 +44,7 @@
 #include "xap_Prefs.h"
 #include "xap_UnixEncodingManager.h"
 #include "xap_UnixFontManager.h"
+#include "xap_UnixNullGraphics.h"
 
 /*****************************************************************/
 
@@ -64,6 +65,13 @@ XAP_UnixApp::XAP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 	// specify a --geometry argument, we only want to obey the
 	// size (which is set above), not a position.
 	m_geometry.flags = GEOMETRY_FLAG_SIZE;
+
+	// hack to ensure UnixNull_Graphics is linked properly
+	// this condition should be false always
+	if (this == 0)
+	{
+		UnixNull_Graphics * pUN_G = new UnixNull_Graphics(0,0);
+	}
 }
 
 XAP_UnixApp::~XAP_UnixApp()
@@ -226,13 +234,13 @@ bool XAP_UnixApp::_loadFonts()
  		if ( ((i == 0) || (szPrefFontPathPtr[i-1] == ';')) && (szPrefFontPathPtr[i] != '/'))
  		{
 		        // if relative path in prefs, prepend library directory.
- 			szTemp = (char *)calloc(suiteLenCache + strlen(szPrefFontPathPtr) + 10, sizeof(char));
+ 			szTemp = (char *)UT_calloc(suiteLenCache + strlen(szPrefFontPathPtr) + 10, sizeof(char));
  			strcpy(szTemp, szPrefFontPathPtr);
  			sprintf(szTemp + i, "%s/%s", suiteDirCache, szPrefFontPathPtr + i);
  			
  			// if relativePathsSoFar > 1 then szPrefFontPathPtr
  			// was really szTemp and was allocated by our call to
- 			// calloc so we must free it. also, we are sure not to
+ 			// UT_calloc so we must free it. also, we are sure not to
 			// free the memory pointed to by getPrefsValue()
  			if (relativePathsSoFar && szPrefFontPathPtr)
  				free(szPrefFontPathPtr);
