@@ -28,7 +28,7 @@
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #include "xap_UnixFont.h"
 #include "xap_UnixFontXLFD.h"
 #include "xap_EncodingManager.h"
@@ -188,10 +188,9 @@ XAP_UnixFont::~XAP_UnixFont(void)
 {
 	
 	FREEP(m_name);
-	
 	FREEP(m_fontfile);
 	FREEP(m_metricfile);
-
+	FREEP(m_xlfd);
 	FREEP(m_fontKey);
 
 	//	UT_VECTOR_PURGEALL(allocFont *, m_allocFonts);
@@ -208,13 +207,26 @@ XAP_UnixFont::~XAP_UnixFont(void)
 	}
 	
 	_deleteEncodingTable();
+
 	// leave GdkFont * alone
+	// but not ABIFontInfo *!
+
+	if (m_metricsData != NULL)
+	{
+		FREEP(m_metricsData->cmi);
+		FREEP(m_metricsData->cwi);
+		FREEP(m_metricsData->gfi);
+		FREEP(m_metricsData->tkd);
+		FREEP(m_metricsData->pkd);
+		FREEP(m_metricsData->ccd);
+		FREEP(m_metricsData);
+	}
 }
 
 bool XAP_UnixFont::openFileAs(const char * fontfile,
-								const char * metricfile,
-								const char * xlfd,
-								XAP_UnixFont::style s)
+			      const char * metricfile,
+			      const char * xlfd,
+			      XAP_UnixFont::style s)
 {
 	// test all our data to make sure we can continue
 	if (!fontfile)
@@ -224,8 +236,6 @@ bool XAP_UnixFont::openFileAs(const char * fontfile,
 	if (!xlfd)
 		return false;
 
-	//UT_DEBUGMSG(("XAP_UnixFont::openFileAs [fontfile `%s']\n", fontfile));
-
 	struct stat buf;
 	
 	if (!m_is_cjk) //HJ's patch had this logic
@@ -253,7 +263,7 @@ bool XAP_UnixFont::openFileAs(const char * fontfile,
 	FREEP(m_name);
 	UT_cloneString(m_name, strtok(NULL, "-"));
 	
-	free(newxlfd);
+	FREEP(newxlfd);
 	
 	// save to memebers
 	FREEP(m_fontfile);
@@ -277,7 +287,9 @@ bool XAP_UnixFont::openFileAs(const char * fontfile,
 	{
 		m_fontType = FONT_TYPE_UNKNOWN;
 		if(!m_is_cjk)	/* cjk has different fonts.dir format. */
+		  {
 		    return false;
+		  }
 	}
 
 	return true;
@@ -355,7 +367,7 @@ void XAP_UnixFont::_deleteEncodingTable()
 	if(m_pEncodingTable)
 	{
 		for(UT_uint32 i = 0; i < m_iEncodingTableSize; i++)
-			delete[] m_pEncodingTable[i].adb;
+			FREEP(m_pEncodingTable[i].adb);
 		delete[] m_pEncodingTable;
 		m_pEncodingTable = 0;
 		m_iEncodingTableSize = 0;
@@ -367,7 +379,11 @@ const encoding_pair * XAP_UnixFont::loadEncodingFile()
 	char * encfile = new char[strlen(m_fontfile)+3]; //3 to be on the safe side, in case extension is < 3
 	strcpy(encfile, m_fontfile);
 	char * dot = strrchr(encfile, '.');
-	if(!dot) return NULL;
+	if(!dot)
+	  {
+	    delete [] encfile;
+	    return NULL;
+	  }
 	*(dot+1) = 'u';
 	*(dot+2) = '2';
 	*(dot+3) = 'g';
@@ -432,9 +448,11 @@ const encoding_pair * XAP_UnixFont::loadEncodingFile(char * encfile)
 				char msg[300];
 				sprintf(msg, "AbiWord could not find required encoding file %s.", full_name);
 				messageBoxOK(msg);
+				delete[] full_name;
 				return 0;
 			}
 		}
+		delete[] full_name;
 	}
 
 	char buff[128];
@@ -460,7 +478,7 @@ const encoding_pair * XAP_UnixFont::loadEncodingFile(char * encfile)
 		if(!fgets(buff,128,ef))
 		{
 			for(UT_uint32 j = 0; j < i; j++)
-				delete [] m_pEncodingTable[j].adb;
+				FREEP(m_pEncodingTable[j].adb);
 			delete[] m_pEncodingTable;
 			m_pEncodingTable = 0;
 			m_iEncodingTableSize = 0;
@@ -476,7 +494,7 @@ const encoding_pair * XAP_UnixFont::loadEncodingFile(char * encfile)
 			if(!comma)
 			{
 				for(UT_uint32 j = 0; j < i; j++)
-					delete [] m_pEncodingTable[j].adb;
+					FREEP(m_pEncodingTable[j].adb);
 				delete[] m_pEncodingTable;
 				m_pEncodingTable = 0;
 				m_iEncodingTableSize = 0;
@@ -638,7 +656,7 @@ ABIFontInfo * XAP_UnixFont::getMetricsData(void)
 		int result = abi_parseFile(fp, &m_metricsData, P_GM);
 		switch (result)
 		{
-			case parseError:
+			case parseFileRes_parseError:
 				g_snprintf(message, 1024,
 				   "AbiWord encountered errors parsing the font metrics file\n"
 				   "[%s].\n"
@@ -646,20 +664,22 @@ ABIFontInfo * XAP_UnixFont::getMetricsData(void)
 				   m_metricfile);
 				messageBoxOK(message);
 				break;
-			case earlyEOF:
+			case parseFileRes_earlyEOF:
 				g_snprintf(message, 1024,
 				   "AbiWord encountered a premature End of File (EOF) while parsing\n"
 				   "the font metrics file [%s].\n"
 				   "Printing cannot continue.",
 				   m_metricfile);
 				messageBoxOK(message);
+				free (m_metricsData);
 				m_metricsData = NULL;
 				break;
-			case storageProblem:
+			case parseFileRes_storageProblem:
 				// if we got here, either the metrics file is broken (like it's
 				// saying it has 209384098278942398743982 kerning lines coming, and
 				// we know we can't allocate that), or we really did run out of memory.
 				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				free (m_metricsData);
 				m_metricsData = NULL;
 				break;
 			default:
@@ -694,7 +714,22 @@ ABIFontInfo * XAP_UnixFont::getMetricsData(void)
 					m_uniWidths[numfound].width = m_metricsData->cmi[i].wx;
 					++numfound;
 				}
+
+				// AbiWord does the adobe glyph -> unicode mapping
+				// with a tiny bit of bogosity.  It assumes that the
+				// relation is one-to-one.
+				// This isn't necessarily so; read 4c (double mappings) at
+			// http://partners.adobe.com/asn/developer/type/unicodegn.html#4
+				// Fortunately, we can insert a gross hack!
+
+				if (unicode == 0x20) // gross hack: space
+				{
+					m_uniWidths[numfound].ucs = 0xa0;
+					m_uniWidths[numfound].width = m_metricsData->cmi[i].wx;
+					++numfound;
+				}
 			}
+
 			// we want to set m_metricsData->numOfChars to the number of chars which we found;
 			// this is typically smaller than the afm file indicated (often the font contains
 			// alternative glyphs, names of which use vendor-defined names; we are not able to
@@ -756,7 +791,7 @@ bool XAP_UnixFont::openPFA(void)
 	bool  delete_pfa = false;
 	if(is_TTF_font())
 	{
-		pfafile = new char[strlen(m_fontfile)];
+		pfafile = new char[strlen(m_fontfile)+1];
 		strcpy(pfafile, m_fontfile);
 		delete_pfa = true;
 		char* dot = strrchr(pfafile, '.');
@@ -930,11 +965,13 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 {
 	// this might return NULL, but that means a font at a certain
 	// size couldn't be found
-	UT_uint32 l = 0;
+    UT_uint32 l = 0;
 	UT_uint32 count = m_allocFonts.getItemCount();
 	xxx_UT_DEBUGMSG(("There are %d allocated fonts for %s \n",count,m_name));
 	allocFont * entry;
         char buf[1000];
+
+ 	bool bFontNotFound = false;
 		
 	while (l < count)
 	{
@@ -945,6 +982,7 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 			l++;
 	}
 
+
 	GdkFont * gdkfont = NULL;
 	
 	// If the font is really, really small (an EXTREMELY low Zoom can trigger this) some
@@ -971,7 +1009,7 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 
 	if(!is_CJK_font())
 	{
-		xxx_UT_DEBUGMSG(("Loading gdkfont [%s]\n", newxlfd));
+		UT_DEBUGMSG(("Loading gdkfont [%s]\n", newxlfd));
 		gdkfont = gdk_font_load(newxlfd);
 
 	 	if (!gdkfont)
@@ -980,6 +1018,7 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 			newxlfd = myXLFD.getFallbackXLFD();
 			requested_lfd  = newxlfd;
 			gdkfont = gdk_font_load(newxlfd);
+			UT_ASSERT(0);
    		}
    		
 	}
@@ -1018,11 +1057,10 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 
 	if (!gdkfont)
 	{
-		char message[1024];
-		g_snprintf(message, 1024,
-			"AbiWord could not load the following font or fontset from the X Window System display server:\n"
-			"[%s]\n"
-			"\n"
+		UT_String message("AbiWord could not load the following font or "
+						  "fontset from the X Window System display server:\n[");
+		message += requested_lfd;
+		message += "]\n\n"
 			"This error could be the result of an incomplete AbiSuite installation,\n"
 			"an incompatibility with your X Window System display server,\n"
 			"or a problem communicating with a remote font server.\n"
@@ -1031,14 +1069,26 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 			"its wrapper shell script.  The script dynamically adds the AbiSuite font directory\n"
 			"to your X Window System display server font path before running the executable.\n"
 			"\n"
-			"Please visit http://www.abisource.com/ for more information.",
-			requested_lfd);
-		messageBoxOK(message);
+			"IMPORTANT: If you're a XFree86 4.x user, try adding:\n"
+			"Load  \"type1\"\n"
+			"to the \"Module\" section of the XF86Config-4 file (usually located at /etc/X11)\n"
+			"\n"
+			"Please visit http://www.abisource.com/ for more information.";
+		messageBoxOK(message.c_str());
+		
+		// we do not have to quit here, just continue ...
+#if 0		
 		exit(1);
+#else
+		bFontNotFound = true;
+#endif
 	}
 
 	free(newxlfd);
 	
+	if(bFontNotFound)
+		return NULL;
+		
 	allocFont * item = new allocFont;
 	item->pixelSize = pixelsize;
 	item->gdkFont = gdkfont;
@@ -1048,7 +1098,7 @@ GdkFont * XAP_UnixFont::getGdkFont(UT_uint32 pixelsize)
 	return gdkfont;
 }
 
-void XAP_UnixFont::_makeFontKey(void)
+void XAP_UnixFont::_makeFontKey()
 {
 	ASSERT_MEMBERS;
 
