@@ -26,26 +26,174 @@
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 
-#include "xap_UnixApp.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFontXLFD.h"
 #include "xap_EncodingManager.h"
 #include "ut_string_class.h"
-
+#include <sys/stat.h>
+#include "xap_Strings.h"
+#include "xap_Prefs.h"
 // TODO get this from some higher-level place
 #define FONTS_DIR_FILE	"/fonts.dir"
 
-XAP_UnixFontManager::XAP_UnixFontManager(void) : m_fontHash(256)
+static char **	s_oldFontPath = NULL;
+static int		s_oldFontPathCount = 0;
+static int		(*s_oldErrorHandler)(Display *dsp, XErrorEvent *e);
+static int		s_errorDepth = 0;
+
+static int s_xerror_handler(Display *dsp, XErrorEvent *e)
+{
+	
+	if(++s_errorDepth > 1)
+	{
+		// we have been caught in a loop; this happens when the old font path
+		// we got from XGetFontPath is invalid, so that the attempt of this
+		// error handler to restore it fails. There is nothing we can do
+		// but treat this a unhandled error
+		// (we do some tests in the main code to avoid this situation,
+		// this is a last resort)
+		goto unhandled;
+	}
+	
+	if(e->error_code == 86)
+	{
+		// BadFontPath, we handle this ourselves
+		UT_DEBUGMSG(("Caught BadFontPath Error, reverting to old path\n"));
+		
+		if(s_oldFontPath)
+		{
+			XSetFontPath(dsp, s_oldFontPath, s_oldFontPathCount);
+			
+			bool bShowWarning = true;
+			XAP_App * pApp = XAP_App::getApp();
+			UT_ASSERT(pApp);
+			pApp->getPrefsValueBool(XAP_PREF_KEY_ShowUnixFontWarning, &bShowWarning);
+
+#if 0 // sevior FIXME switch this back.
+			
+			if(bShowWarning)
+			{
+				if(pApp->getDisplayStatus())
+				{
+					const XML_Char * msg = pApp->getStringSet()->getValue(XAP_STRING_ID_MSG_ShowUnixFontWarning);
+					UT_ASSERT(msg);
+					messageBoxOK(msg);
+				}
+       			else
+       				fprintf(stderr, "AbiWord WARNING: unable to modify font path\n");
+				
+			}
+			
+			
+			
+#else			
+			if(bShowWarning)			
+				messageBoxOK("WARNING: AbiWord could not add its fonts to the X font path.\n"
+				 " See \"Unix Font Warning\" in the FAQ section of AbiWord help.");
+#endif
+				
+			//s_errorDepth--;
+			return (0);
+		}
+		
+		UT_DEBUGMSG(("!!! No Old Path available !!!\n"));
+		
+	}
+
+unhandled:
+	// on everything else we will call the previous handler.
+	UT_DEBUGMSG(("Unhandled X error: %d\n"
+				 "          display %s, serial %lu, request %d:%d, XID %lu\n"
+				 "          AbiWord will terminate\n",
+				 (int) e->error_code, DisplayString (e->display), e->serial, (int) e->request_code, (int) e->minor_code, (unsigned long) e->resourceid));
+	
+	if(!s_oldErrorHandler)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		// we cannot handle this ourselves, nor is there any handler available
+		// what can we do but exit?
+		exit(1);
+	}
+
+	s_errorDepth--;	
+	return s_oldErrorHandler(dsp,e);
+}
+
+XAP_UnixFontManager::XAP_UnixFontManager(void) : m_fontHash(256), m_pExtraXFontPath(0),
+m_iExtraXFontPathCount(0)
 {
 }
 
 XAP_UnixFontManager::~XAP_UnixFontManager(void)
 {
-	UT_VECTOR_PURGEALL(char *, m_searchPaths);
+	UT_VECTOR_FREEALL(char *, m_searchPaths);
 
 	UT_HASH_PURGEDATA(XAP_UnixFont *, &m_fontHash, delete);
+
+	// we have to remove what we have added to the X font path
+	// the code below is not perfect, because if some other
+	// program adds one of our paths while we are running,
+	// we will remove it as well.	
+	if(m_pExtraXFontPath)
+	{
+		Display *dsp  = XOpenDisplay(gdk_get_display());
+		UT_sint32 pathSize;
+		char ** oldPath = XGetFontPath(dsp, &pathSize);
+		
+		char ** newPath = new char *[pathSize];
+		UT_ASSERT(newPath);
+		
+		char ** newPath_ptr = newPath;
+		
+		xxx_UT_DEBUGMSG(("current path size %d, m_iExtraXFontPathCount %d\n", pathSize,m_iExtraXFontPathCount));
+		
+		UT_sint32 i;
+		UT_sint32 j;
+		
+		for(i = 0; i < pathSize; i++)
+		{
+			bool bFound = false;
+			for(j = 0; j < m_iExtraXFontPathCount; j++)
+				if(!UT_strcmp(m_pExtraXFontPath[j], oldPath[i]))
+				{
+					bFound = true;
+					break;
+				}
+				
+			if(bFound)
+				continue;
+				
+			*newPath_ptr++ = oldPath[i];
+		}
+		
+		pathSize = newPath_ptr - newPath;
+
+#ifdef DEBUG		
+		UT_DEBUGMSG(("--- new path (size %d) ---\n", pathSize));
+		for(j = 0; j < pathSize; j++)
+			UT_DEBUGMSG(("\tfontpath[%d]: \"%s\"\n", j, newPath[j]));
+		UT_DEBUGMSG(("-------------------------------------------------\n\n"));
+#endif
+		
+		XSetFontPath(dsp, newPath, pathSize);
+		for(j = 0; j < m_iExtraXFontPathCount; j++)
+		{
+		    xxx_UT_DEBUGMSG(("freeing indx %d\n", j));
+			FREEP(m_pExtraXFontPath[j]);
+		}
+		xxx_UT_DEBUGMSG(("deleting m_pExtraXFontPath\n"));
+		delete [] m_pExtraXFontPath;
+
+		xxx_UT_DEBUGMSG(("freeing old X font path\n"));
+		XFreeFontPath(oldPath);
+		xxx_UT_DEBUGMSG(("closing X display\n"));
+		XCloseDisplay(dsp);
+		xxx_UT_DEBUGMSG(("done.\n"));
+
+		delete[] newPath ;
+	}
 }
 
 bool XAP_UnixFontManager::setFontPath(const char * searchpath)
@@ -70,7 +218,200 @@ bool XAP_UnixFontManager::scavengeFonts(void)
 	UT_uint32 totalfonts = 0;
 	
 	const char** subdirs = localeinfo_combinations("","","");
-	/* If j is even, we open fonts.dir in encoding-specific directory */
+
+#if 1
+	bool bModifyPath = true;
+	XAP_App * pApp = XAP_App::getApp();
+	UT_ASSERT(pApp);
+	pApp->getPrefsValueBool(XAP_PREF_KEY_ModifyUnixFontPath, &bModifyPath);
+
+	if(bModifyPath)
+	{
+	    const char** subdir = subdirs;
+    	UT_uint32 subdircount  = 0;
+	
+		while(*subdir++)
+			subdircount++;
+	
+		Display *dsp  = XOpenDisplay(gdk_get_display());
+		UT_uint32 fontPathDirCount, realFontPathDirCount = 0;
+	
+		if(m_pExtraXFontPath)
+		{
+			// this should not happen, if it does, we will not be able to restore
+			// font path when we exit
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT(m_iExtraXFontPathCount >= 0);
+		
+		    for(i = 0; i < (UT_uint32)m_iExtraXFontPathCount; i++)
+		    	FREEP(m_pExtraXFontPath[i]);
+	    	delete [] m_pExtraXFontPath;
+		}
+		
+		char ** oldFontPath = XGetFontPath(dsp,(int*)&fontPathDirCount);
+	
+		// set stuff for our error handler
+		s_oldFontPath = oldFontPath;
+		s_oldFontPathCount = fontPathDirCount;
+	
+		realFontPathDirCount = fontPathDirCount;
+		m_iExtraXFontPathCount = -fontPathDirCount;
+	
+		fontPathDirCount += count*subdircount;
+		UT_DEBUGMSG(("XDisplay [%s], current X font path count %d, AW path count %d, subdircount %d\n",
+				gdk_get_display(),realFontPathDirCount,count, subdircount));
+				
+		char ** newFontPath = new char*[fontPathDirCount];
+		UT_ASSERT(newFontPath);
+		char ** newFontPath_ptr = newFontPath;
+		char ** oldFontPath_ptr	= oldFontPath;
+		for(i = 0; i < realFontPathDirCount; i++)
+		{
+			// do some elementary tests on validity of the old path
+			// basically, if it contains anything else than directories
+			// we will abort our attempt to modify the path.
+			
+			// first of all, take care of the path elements that have size
+			// specification;
+			
+			char * temp_str = UT_strdup(*oldFontPath_ptr);
+			UT_ASSERT(temp_str);
+			
+			char * colon = strrchr(temp_str, ':');
+			
+			if(colon)
+				*colon = 0;
+				
+	    	struct stat stat_data;
+	    	if(!stat(temp_str,&stat_data) && S_ISDIR(stat_data.st_mode))
+			{
+    			xxx_UT_DEBUGMSG(("orig font path element: %s\n", *oldFontPath_ptr));
+    			*newFontPath_ptr++ = UT_strdup(*oldFontPath_ptr++);
+    			FREEP(temp_str);
+    		}
+    		else
+    		{
+    			// remember how many elements of the font path we have
+    			// duplicated
+    			realFontPathDirCount = i;
+    			m_iExtraXFontPathCount = 0;
+				bool bShowWarning = true;
+				XAP_App * pApp = XAP_App::getApp();
+				UT_ASSERT(pApp);
+				pApp->getPrefsValueBool(XAP_PREF_KEY_ShowUnixFontWarning, &bShowWarning);
+
+				UT_DEBUGMSG(("found non-directory entry in existing fontpath [%s]\n", *oldFontPath_ptr));
+				if(bShowWarning)			
+				{
+					if(pApp->getDisplayStatus())
+					{
+#if 1
+						const XML_Char * msg = pApp->getStringSet()->getValue(XAP_STRING_ID_MSG_ShowUnixFontWarning);
+						UT_ASSERT(msg);
+						messageBoxOK(msg);
+
+#else					
+						messageBoxOK("WARNING: Your current font path contains non-directory entries.\n"
+							"AbiWord will not attempt to modify this path. You should see \"Unix Font Warning\"\n"
+							"in the FAQ section of AbiWord help for further information and instructions on how to\n"
+                        	"turn this warning off");
+#endif
+       				}
+       				else
+       					fprintf(stderr, "AbiWord WARNING: unable to modify font path\n");
+				}
+    			
+    			FREEP(temp_str);
+    			
+    			goto invalid_old_path;
+    		}
+		}
+		
+		for(i = 0; i < count; i++)
+		{
+			UT_String str;
+		    UT_ASSERT(m_searchPaths.getNthItem(i));	
+		    char* basedirname = (char *) m_searchPaths.getNthItem(i);
+	
+	    	for(subdir = subdirs;*subdir;++subdir)
+		    {
+		    	str = basedirname;
+	    		str += "/";
+	    		str += *subdir;
+		    	struct stat stat_data;
+		    	if(!stat(str.c_str(),&stat_data))
+	    		{
+	    			if(S_ISDIR(stat_data.st_mode))
+	    			{
+	    				bool bFound = false;
+		    			// only add this if it is not already in the path
+		    			for(UT_uint32 j = 0; j < realFontPathDirCount; j++)
+	    				{
+	    					if(!UT_strcmp(newFontPath[j], str.c_str()))
+	    					{
+	    						UT_DEBUGMSG(("directory [%s] already in the X font path\n", str.c_str()));
+	    						bFound = true;
+	    						break;
+		    				}
+		    			}
+	    			
+	    				if(!bFound)
+	    				{
+	    					//str+= "rubbish"; // to force error while testing
+	    					*newFontPath_ptr++ = UT_strdup(str.c_str());
+		    				UT_DEBUGMSG(("adding \"%s\" to font path\n", str.c_str()));
+			    			realFontPathDirCount++;
+			    		}
+	    			}
+	    		}
+			}
+		}
+	
+		m_iExtraXFontPathCount += realFontPathDirCount;
+		UT_ASSERT(m_iExtraXFontPathCount >= 0);
+	    xxx_UT_DEBUGMSG(("m_iExtraXFontPathCount %d\n", m_iExtraXFontPathCount));
+		
+		m_pExtraXFontPath = new char *[m_iExtraXFontPathCount];
+		UT_ASSERT(m_pExtraXFontPath);
+
+		{	// the block is here because of the jump to invalid_old_path
+			char ** pExtraXFontPath = m_pExtraXFontPath;
+	
+			for(i = realFontPathDirCount - m_iExtraXFontPathCount; i < realFontPathDirCount; i++)
+				*pExtraXFontPath++ = newFontPath[i];
+	    }
+#ifdef DEBUG		
+		UT_DEBUGMSG(("--- setting X font path (%d items)\n", realFontPathDirCount));
+		for(UT_uint32 j = 0; j < realFontPathDirCount; j++)
+			UT_DEBUGMSG(("\tfontpath[%d]: \"%s\"\n", j, newFontPath[j]));
+		UT_DEBUGMSG(("-------------------------------------------------\n\n"));
+#endif
+
+		s_oldErrorHandler = XSetErrorHandler(s_xerror_handler);
+		// force synchronic behaviour, so that any error is caught
+		// immediately
+		XSynchronize(dsp,1);
+		XSetFontPath(dsp, newFontPath, (int)realFontPathDirCount);
+	
+		// now if we failed, our handler should have been called and if we are
+		// here, we have things under control, co clean up
+		XSynchronize(dsp,0);
+		XSetErrorHandler(s_oldErrorHandler);
+		s_oldErrorHandler = NULL;
+		s_oldFontPath = 0;
+
+invalid_old_path:	
+		// now free what we do not need	
+		XFreeFontPath(oldFontPath);
+		XCloseDisplay(dsp);
+	
+		// free the directories that we got from the X server, not the bit
+		// we appended
+		for(i = 0; i < realFontPathDirCount - m_iExtraXFontPathCount; i++)
+			FREEP(newFontPath[i]);
+		delete [] newFontPath;
+	}
+#endif
 	for (i = 0; i < count; i++)
 	{
 	    UT_ASSERT(m_searchPaths.getNthItem(i));	
@@ -80,7 +421,7 @@ bool XAP_UnixFontManager::scavengeFonts(void)
 	    
 	    for(const char** subdir = subdirs;*subdir;++subdir)
 	    {	    	
-		filename = (char *) calloc(basedirname_len + 1 + strlen(*subdir) + strlen((char *) FONTS_DIR_FILE) + 1, sizeof(char));
+		filename = (char *) UT_calloc(basedirname_len + 1 + strlen(*subdir) + strlen((char *) FONTS_DIR_FILE) + 1, sizeof(char));
 		sprintf(filename, "%s/%s%s", basedirname, *subdir, FONTS_DIR_FILE);
 
 		FILE * file;
@@ -252,14 +593,40 @@ XAP_UnixFont * XAP_UnixFontManager::getDefaultFont(void)
 	// X11R6 I can think of)
 	UT_DEBUGMSG(("XAP_UnixFontManager::getDefaultFont\n"));
 
-	XAP_UnixFont * f = new XAP_UnixFont();
+	static bool fontInitted = false ;
+	static XAP_UnixFont m_f ;
 
-	// do some manual behind-the-back construction
-	f->setName("Default");
-	f->setStyle(XAP_UnixFont::STYLE_NORMAL);
-	f->setXLFD("-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*");
+	if ( !fontInitted )
+	  {
+	    // do some manual behind-the-back construction
+	    m_f.setName("Default");
+	    m_f.setStyle(XAP_UnixFont::STYLE_NORMAL);
+	    m_f.setXLFD("-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*");
+	    fontInitted = true ;
+	  }
 
-	return f;
+	return &m_f;
+}
+
+XAP_UnixFont * XAP_UnixFontManager::getDefaultFont16Bit(void)
+{
+	UT_DEBUGMSG(("XAP_UnixFontManager::getDefaultFont16Bit\n"));
+
+	XAP_UnixFont * f = NULL;
+	
+	if(XAP_EncodingManager::get_instance()->isUnicodeLocale())
+	{
+		XAP_UnixFont * f = new XAP_UnixFont();
+		UT_ASSERT(f);
+		// people runing utf-8 locale will probably have the
+		// MS Arial ttf font ...
+		f->setName("Default");
+		f->setStyle(XAP_UnixFont::STYLE_NORMAL);
+		f->setXLFD("-*-Arial-medium-r-*-*-*-*-*-*-*-*-iso10646-1");
+		return f;
+	}
+    else
+		return getDefaultFont();	
 }
 
 XAP_UnixFont * XAP_UnixFontManager::getFont(const char * fontname,
@@ -347,7 +714,7 @@ void XAP_UnixFontManager::_allocateThisFont(const char * line,
 	// allocate a real buffer for this
 	fontfile = UT_strdup(fontfile);
 	// tack on the working directory to get the full path to the font file
-	char * newstuff = (char *) calloc(strlen(workingdir) + 1 + strlen(fontfile) + 1,
+	char * newstuff = (char *) UT_calloc(strlen(workingdir) + 1 + strlen(fontfile) + 1,
 									  sizeof(char));
 	sprintf(newstuff, "%s/%s", workingdir, fontfile);
 	FREEP(fontfile);
@@ -421,7 +788,7 @@ void XAP_UnixFontManager::_allocateThisFont(const char * line,
 
 	// allocate a new buffer for the AFM name
 	size_t len = stemchars + 4 + 1;
-	char * metricfile = (char *) calloc(len, sizeof(char));
+	char * metricfile = (char *) UT_calloc(len, sizeof(char));
 	g_snprintf(metricfile, stemchars, "%s", fontfile);
 	strcat(metricfile, ".afm");
 
