@@ -27,7 +27,7 @@
 #include "ut_string.h"
 #include "ut_misc.h"
 #include "ut_wctomb.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #include "ut_endian.h"
 
 #include "xap_UnixPSGenerate.h"
@@ -50,6 +50,14 @@
 // the resolution that we report to the application (pixels per inch).
 #define PS_RESOLUTION		7200
 
+/*
+	How long line in the PS output we will allow;
+	DSC3.0 limits to 256, but we may need to print a few extra
+	characters after we reach the end of our buffer
+*/
+#define OUR_LINE_LIMIT		220			
+#define LINE_BUFFER_SIZE   OUR_LINE_LIMIT + 30
+
 #if 1
 #include <gdk/gdkprivate.h>
 static bool isFontUnicode(GdkFont *font)
@@ -100,6 +108,7 @@ PS_Graphics::PS_Graphics(const char * szFilename,
 	m_currentColor.m_red = 0;
 	m_currentColor.m_grn = 0;
 	m_currentColor.m_blu = 0;
+	m_iPageCount = 0;
 }
 
 PS_Graphics::~PS_Graphics()
@@ -113,6 +122,7 @@ bool PS_Graphics::queryProperties(GR_Graphics::Properties gp) const
 	switch (gp)
 	{
 	case DGP_SCREEN:
+	case DGP_OPAQUEOVERLAY:
 		return false;
 	case DGP_PAPER:
 		return true;
@@ -446,8 +456,6 @@ GR_Font* PS_Graphics::findFont(const char* pszFontFamily,
 	return pFont;
 }
 
-#define OUR_LINE_LIMIT		70			/* DSC3.0 limits to 256 */
-
 void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 							int iLength, UT_sint32 xoff, UT_sint32 yoff)
 {
@@ -528,7 +536,7 @@ void PS_Graphics::_drawCharsOverstriking(const UT_UCSChar* pChars, UT_uint32 iCh
 	yoff += getFontAscent();
 
 	// unsigned buffer holds Latin-1 data to character code 255
-	char buf[OUR_LINE_LIMIT*2];
+	char buf[LINE_BUFFER_SIZE]; //some extra space at the end
 	char * pD = buf;
 
 	const UT_UCSChar * pS = pChars+iCharOffset;
@@ -569,7 +577,9 @@ void PS_Graphics::_drawCharsOverstriking(const UT_UCSChar* pChars, UT_uint32 iCh
 				pD = buf + strlen(buf);
 				using_names = true;
 			}
-				
+
+			// we don't properly handle double mappings:
+			// http://partners.adobe.com/asn/developer/type/unicodegn.html#4
 			const char * glyph = ae->ucsToAdobe(currentChar);
 			// ' /glyph GS '
 			if(pD - buf + strlen(glyph) + 6 > OUR_LINE_LIMIT)
@@ -648,7 +658,7 @@ void PS_Graphics::_drawCharsCJK(const UT_UCSChar* pChars, UT_uint32 iCharOffset,
 	yoff += getFontAscent();
 
 	// unsigned buffer holds Latin-1 data to character code 255
-	char buf[OUR_LINE_LIMIT*2];
+	char buf[LINE_BUFFER_SIZE];
 	char * pD = buf;
 
 	const UT_UCSChar * pS = pChars+iCharOffset;
@@ -734,7 +744,7 @@ void PS_Graphics::_drawCharsNonCJK(const UT_UCSChar* pChars, UT_uint32 iCharOffs
 	yoff += getFontAscent();
 
 	// unsigned buffer holds Latin-1 data to character code 255
-	char buf[OUR_LINE_LIMIT*2];
+	char buf[LINE_BUFFER_SIZE];
 	char * pD = buf;
 
 	const UT_UCSChar * pS = pChars+iCharOffset;
@@ -821,7 +831,7 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 	yoff += getFontAscent();
 
 	// unsigned buffer holds Latin-1 data to character code 255
-	char buf[OUR_LINE_LIMIT*2];
+	char buf[LINE_BUFFER_SIZE];
 	char * pD = buf;
 
 	const UT_UCSChar * pS = pChars+iCharOffset;
@@ -862,6 +872,8 @@ void PS_Graphics::_drawCharsUTF8(const UT_UCSChar* pChars, UT_uint32 iCharOffset
 				using_names = true;
 			}
 				
+			// we don't properly handle double mappings:
+			// http://partners.adobe.com/asn/developer/type/unicodegn.html#4
 			const char * glyph = ae->ucsToAdobe(currentChar);
 			// ' /glyph GS '
 			if(pD - buf + strlen(glyph) + 6 > OUR_LINE_LIMIT)
@@ -930,7 +942,7 @@ void PS_Graphics::drawLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2, UT_sint32 y
 	// emit a change in line width
 	_emit_SetLineWidth();
 	
-	char buf[OUR_LINE_LIMIT*2];
+	char buf[LINE_BUFFER_SIZE];
 //	UT_sint32 nA = getFontAscent();
 	g_snprintf(buf,sizeof (buf),"%d %d %d %d ML\n", x2, y2, x1, y1);
 	m_ps->writeBytes(buf);
@@ -1075,6 +1087,8 @@ bool PS_Graphics::_startDocument(void)
 		m_ps->formatComment("Title",&m_szTitle,1);
 	m_ps->formatComment("Orientation", isPortrait() ? "Portrait" : "Landscape");
 
+	m_ps->formatComment("Pages",m_iPageCount);
+	
 	//TODO: emit iWidth and iHeight in BoundingBox somehow (what's a 
 	//factor between them and PS units (that are 1/72 of inch IIRC)
 	m_ps->formatComment("DocumentPaperSizes",m_szPageSizeName);
@@ -1181,113 +1195,179 @@ bool PS_Graphics::_endDocument(void)
 void PS_Graphics::_emit_DocumentNeededResources(void)
 {
 	UT_Vector vec;
-	UT_uint32 k;
+	UT_uint32 k,n;
 	UT_uint32 kLimit = m_vecFontList.getItemCount();
 
+    bool bFontKeyword = true;
 	for (k=0; k<kLimit; k++)
 	{
-		vec.addItem((void *) "font");
 		PSFont * psf = (PSFont *)m_vecFontList.getNthItem(k);
 		if(!psf->getUnixFont()->is_CJK_font())
-		    vec.addItem(psf->getMetricsData()->gfi->fontName);
+        {
+            if(bFontKeyword)
+            {
+        		vec.addItem((void *) "font");
+                bFontKeyword = false;
+            }
+            
+            // only include each font name once
+            const char * pName = psf->getMetricsData()->gfi->fontName;
+            bool bFound = false;
+            for(n = 0; n < vec.getItemCount(); n++)
+            {
+                if(!UT_strcmp(pName, (const char *)vec.getNthItem(n)))
+                {
+                    bFound = true;
+                    break;
+                }
+            }
+            
+            if(!bFound)
+    		    vec.addItem((void*)pName);
+        }
 	}
 
 	// TODO add any other resources here
-	
-	m_ps->formatComment("DocumentNeededResources",&vec);
+	bool bEmbedFonts;
+	XAP_App::getApp()->getPrefsValueBool((const XML_Char *)XAP_PREF_KEY_EmbedFontsInPS, &bEmbedFonts);
+	if(bEmbedFonts)
+        m_ps->formatComment("DocumentSuppliedResources",&vec);
+    else
+    	m_ps->formatComment("DocumentNeededResources",&vec);
 }
 
 void PS_Graphics::_emit_IncludeResource(void)
 {
-	UT_uint32 k;
-	UT_uint32 kLimit = m_vecFontList.getItemCount();
-	typedef char* pchar_t;
-	char **fontKey= new pchar_t[kLimit];
-	int fontKeyCount=0;
+   	UT_Vector vec;
+   	UT_uint32 k,n;
+   	UT_uint32 kLimit = m_vecFontList.getItemCount();
 
-	for (k=0; k<kLimit; k++)
-	{
-		char buf[128];
+	// we want to have a checkbox in the Preferences that would allow
+	// to disable splating of the fonts into the output, since people who
+	// use Ghostscript my simply register their fonts with GS and do not
+	// need them in the document
+	bool bEmbedFonts;
+	XAP_App::getApp()->getPrefsValueBool((const XML_Char *)XAP_PREF_KEY_EmbedFontsInPS, &bEmbedFonts);
+	//UT_DEBUGMSG(("bEmbedFonts: %d\n",bEmbedFonts));
 
-		PSFont * psf = (PSFont *) m_vecFontList.getNthItem(k);
+    if(bEmbedFonts)
+    {
+    	for (k=0; k<kLimit; k++)
+    	{
+    		char buf[128];
+
+    		PSFont * psf = (PSFont *) m_vecFontList.getNthItem(k);
 		
-		// m_ps->formatComment("IncludeResource",psf->getMetricsData()->gfi->fontName);
+    		// m_ps->formatComment("IncludeResource",psf->getMetricsData()->gfi->fontName);
 
-		// Instead of including the resources, we actually splat the fonts
-		// into the document.  This looks really slow... perhaps buffer line
-		// by line or in larger chunks the font data.
-		XAP_UnixFont * unixfont = psf->getUnixFont();
+    		// Instead of including the resources, we actually splat the fonts
+    		// into the document.  This looks really slow... perhaps buffer line
+    		// by line or in larger chunks the font data.
+    		XAP_UnixFont * unixfont = psf->getUnixFont();
 		
-		// TODO we want to have a checkbox in the Preferences that would allow
-		// to disable splating of the fonts into the output, since people who
-		// use Ghostscript my simply register their fonts with GS and do not
-		// need them in the document
-		bool bEmbedFonts;
-		XAP_App::getApp()->getPrefsValueBool((const XML_Char *)XAP_PREF_KEY_EmbedFontsInPS, &bEmbedFonts);
-		//UT_DEBUGMSG(("bEmbedFonts: %d\n",bEmbedFonts));
-		if(unixfont->is_CJK_font() || !bEmbedFonts)
-		  continue;
-		int match=0;
-		for(int i=0;i<fontKeyCount;++i)
-		  if(!strcmp(unixfont->getFontKey(),fontKey[i]))
-			{
-			  match=1;
-			  break;
-			}
-		if(match)
-		  continue;
-		fontKey[fontKeyCount++]=UT_strdup(unixfont->getFontKey());
+    		if(unixfont->is_CJK_font())
+		      continue;
+    		bool match = false;
+            const char * pName = unixfont->getFontKey();
+    		for(int i=0;i<vec.getItemCount();++i)
+    		{
+				if(!strcmp(pName,(const char*)vec.getNthItem(i)))
+    			{
+			    	match=true;
+    				break;
+    			}
+    		}
+    		if(match)
+		    	continue;
+            
+            vec.addItem((void*)pName);
 
-		// Make sure the font file will open, maybe it disappeared...
-		if(!unixfont->openPFA())
-		{
-			char message[1024];
-			g_snprintf(message, sizeof (message),
+    		// Make sure the font file will open, maybe it disappeared...
+    		if(!unixfont->openPFA())
+    		{
+    			char message[1024];
+    			g_snprintf(message, sizeof (message),
 					   "Font data file [%s] cannot be opened for reading!\n"
 					   "Did it disappear on us?  AbiWord can't print without\n"
 					   "this file; your PostScript might be missing this resource.",
 					   unixfont->getFontfile());
 
-			// we don't have any frame info, so we use the non-parented dialog box
-			messageBoxOK(message);
-			return;
-		}
+    			// we don't have any frame info, so we use the non-parented dialog box
+    			messageBoxOK(message);
+    			return;
+    		}
 		
-		signed char ch = 0;
-		while ((ch = unixfont->getPFAChar()) != EOF)
-			m_ps->writeBytes((UT_Byte *) &ch, 1);
+    		signed char ch = 0;
+    		while ((ch = unixfont->getPFAChar()) != EOF)
+    			m_ps->writeBytes((UT_Byte *) &ch, 1);
 
-		unixfont->closePFA();
+    		unixfont->closePFA();
 
-		// NOTE : here's an internationalization process step.  If the font
-		// NOTE : encoding is NOT "iso8859", we do not emit this macro.
-		// NOTE : this keeps fonts like Standard Symbols, and really
-		// NOTE : any other encoding, from being mangled.  however, it's
-		// NOTE : not intended to guarantee that these other encodings
-		// NOTE : actually work.  that requires more design work.
+    		// NOTE : here's an internationalization process step.  If the font
+    		// NOTE : encoding is NOT "iso8859", we do not emit this macro.
+    		// NOTE : this keeps fonts like Standard Symbols, and really
+    		// NOTE : any other encoding, from being mangled.  however, it's
+    		// NOTE : not intended to guarantee that these other encodings
+    		// NOTE : actually work.  that requires more design work.
 
-		// Fetch an XLFD object from the font
-		XAP_UnixFontXLFD myXLFD(unixfont->getXLFD());
+    		// Fetch an XLFD object from the font
+    		XAP_UnixFontXLFD myXLFD(unixfont->getXLFD());
 
-		// write findfont
-		g_snprintf(buf, sizeof (buf), "/%s findfont\n", psf->getMetricsData()->gfi->fontName);
-		m_ps->writeBytes(buf);
+    		// write findfont
+    		g_snprintf(buf, sizeof (buf), "/%s findfont\n", psf->getMetricsData()->gfi->fontName);
+    		m_ps->writeBytes(buf);
 
-		// Compare with iso8859, and emit LAT for that font
-		if (!UT_stricmp(myXLFD.getRegistry(), "iso8859") && !UT_strcmp(myXLFD.getEncoding(), "1"))
+    		// Compare with iso8859, and emit LAT for that font
+    		if (!UT_stricmp(myXLFD.getRegistry(), "iso8859") && !UT_strcmp(myXLFD.getEncoding(), "1"))
+    		{
+    			g_snprintf(buf, sizeof (buf), "LAT\n");
+    			m_ps->writeBytes(buf);
+    		}
+
+    		// exec the swapper macro
+    		g_snprintf(buf, sizeof (buf), "/%s EXC\n", psf->getMetricsData()->gfi->fontName);
+    		m_ps->writeBytes(buf);
+
+    	}
+    }
+	else
+	{
+		// when not embeding fonts, we have to issue %%IncludeResource statement
+		bool bFontKeyword = true;
+		const char* pFResource[2] = {"font", NULL};
+		for (k=0; k<kLimit; k++)
 		{
-			g_snprintf(buf, sizeof (buf), "LAT\n");
-			m_ps->writeBytes(buf);
+			PSFont * psf = (PSFont *)m_vecFontList.getNthItem(k);
+			if(!psf->getUnixFont()->is_CJK_font())
+			{
+				if(bFontKeyword)
+				{
+					vec.addItem((void *) "font");
+					bFontKeyword = false;
+				}
+            
+				// only include each font name once
+				const char * pName = psf->getMetricsData()->gfi->fontName;
+				bool bFound = false;
+				for(n = 0; n < vec.getItemCount(); n++)
+				{
+					if(!UT_strcmp(pName, (const char *)vec.getNthItem(n)))
+					{
+						bFound = true;
+						break;
+					}
+				}
+            
+				if(!bFound)
+				{
+					vec.addItem((void*)pName);
+					pFResource[1] = pName;
+					m_ps->formatComment("IncludeResource", pFResource,2);
+				}
+			}
 		}
-
-		// exec the swapper macro
-		g_snprintf(buf, sizeof (buf), "/%s EXC\n", psf->getMetricsData()->gfi->fontName);
-		m_ps->writeBytes(buf);
-
 	}
-	for(int i=0;i<fontKeyCount;++i)
-	  free(fontKey[i]);
-	delete []fontKey;
+        
 	// TODO add any other IncludeResource's here
 }
 
@@ -1305,9 +1385,11 @@ void PS_Graphics::_emit_PrologMacros(void)
 		"/BPL {BP SZ 90 rotate} bind def",					// Begin Landscape Page. <w> <h> <res> BPP
 		"/EP  {grestore showpage} bind def",				// EP
 		"/ML  {neg moveto neg lineto stroke} bind def",		// Move and Line. <x2> <y2> <x1> <y1> ML
-		"/LAT {dup length dict begin",						// Change encoding vector for current font to Latin1
+		"/LAT {dup length dict copy dup begin",				// Change encoding vector for current font to Latin1
 		"       {1 index /FID ne {def} {pop pop} ifelse} forall",
-		"       /Encoding ISOLatin1Encoding def",
+		"       /Encoding ISOLatin1Encoding 256 array copy def",
+		"       Encoding 39 /quotesingle put",              // hack for 2313
+		"       Encoding 45 /hyphen put",
 		"       currentdict",
 		"       end} bind def",
 		"/EXC {exch definefont pop} bind def",				// Exchange font entry
