@@ -37,6 +37,7 @@
 #include "xap_ModuleManager.h"
 
 #include "ie_types.h"
+#include "ut_string_class.h"
 
 #include "xap_Win32Resources.rc2"
 
@@ -121,8 +122,14 @@ BOOL CALLBACK XAP_Win32Dialog_PluginManager::s_dlgProc(HWND hWnd,UINT msg,WPARAM
 
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
-
-#define _DSX(c,s)	SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,pSS->getValue(XAP_STRING_ID_##s))
+// As Tabbed Dialogs have problems with HotKeys, this macro have been replaced to remove &
+//#define _DSX(c,s)	SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,pSS->getValue(XAP_STRING_ID_##s))
+#define _DSX(c,s) { \
+                    XML_Char* p = NULL; \
+                    UT_XML_cloneNoAmpersands( p, pSS->getValue(XAP_STRING_ID_##s));\
+                    SetDlgItemText(hWnd,XAP_RID_DIALOG_##c,p); \
+					FREEP(p); \
+                  }
 #define _GVX(s)		(pSS->getValue(XAP_STRING_ID_##s))
 
 // the order of the tabs
@@ -148,7 +155,7 @@ BOOL XAP_Win32Dialog_PluginManager::_onInitDialog(HWND hWnd, WPARAM wParam, LPAR
 	SetWindowText(hWnd, pSS->getValue(XAP_STRING_ID_DLG_PLUGIN_MANAGER_TITLE));
 
 	// localize controls
-		// No controls except the CLOSE button???
+	_DSX(PLUGIN_MANAGER_BTN_CLOSE, DLG_Close);
 
 	// setup the tabs
 	{
@@ -482,15 +489,20 @@ void XAP_Win32Dialog_PluginManager::event_Load()
 		= (XAP_Dialog_FileOpenSaveAs *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_FILE_OPEN));
 	UT_ASSERT(pDialog);
 	
-	pDialog->setCurrentPathname(0);
+	// set the intial plugin directory to the user-local plugin directory
+	// could also set to: XAP_App::getApp()->getUserPrivateDirectory()\plugins
+	// could also set to: XAP_App::getApp()->getAbiSuiteLibDir()\plugins
+	UT_String pluginDir (XAP_App::getApp()->getAbiSuiteAppDir());
+	pluginDir += "\\plugins";
+	pDialog->setCurrentPathname (pluginDir.c_str());
 	pDialog->setSuggestFilename(false);
 	
 	UT_uint32 filterCount = 1;
-	const char ** szDescList = (const char **) calloc(filterCount + 1,
+	const char ** szDescList = (const char **) UT_calloc(filterCount + 1,
 													  sizeof(char *));
-	const char ** szSuffixList = (const char **) calloc(filterCount + 1,
+	const char ** szSuffixList = (const char **) UT_calloc(filterCount + 1,
 														sizeof(char *));
-	IEFileType * nTypeList = (IEFileType *) calloc(filterCount + 1,
+	IEFileType * nTypeList = (IEFileType *) UT_calloc(filterCount + 1,
 												   sizeof(IEFileType));
 	
 	// we probably shouldn't hardcode this
