@@ -67,10 +67,17 @@ typedef enum
 typedef enum 
 {       
 	FL_HDRFTR_HEADER,
-	FL_HDRFTR_FOOTER
+	FL_HDRFTR_HEADER_EVEN,
+	FL_HDRFTR_HEADER_FIRST,
+	FL_HDRFTR_HEADER_LAST,
+	FL_HDRFTR_FOOTER,
+	FL_HDRFTR_FOOTER_EVEN,
+	FL_HDRFTR_FOOTER_FIRST,
+	FL_HDRFTR_FOOTER_LAST,
+	FL_HDRFTR_NONE
 } HdrFtrType;
 
-class fl_SectionLayout : public fl_Layout
+class ABI_EXPORT fl_SectionLayout : public fl_Layout
 {
 	friend class fl_DocListener;
 
@@ -80,7 +87,7 @@ public:
 
 	SectionType     	getType(void) const { return m_iType; }
 
-	virtual bool		recalculateFields(void);
+	virtual bool		recalculateFields(UT_uint32 iUpdateCount);
 	
 	FL_DocLayout*		getDocLayout(void) const;
 
@@ -167,7 +174,7 @@ protected:
 	fl_HdrFtrSectionLayout * m_pHdrFtrSL;
 };
 
-class fl_DocSectionLayout : public fl_SectionLayout
+class ABI_EXPORT fl_DocSectionLayout : public fl_SectionLayout
 {
 	friend class fl_DocListener;
 
@@ -196,10 +203,12 @@ public:
 	inline UT_sint32			getLeftMarginInLayoutUnits(void) const { return m_iLeftMarginLayoutUnits; }
 	inline UT_sint32			getRightMargin(void) const { return m_iRightMargin; }
 	inline UT_sint32			getRightMarginInLayoutUnits(void) const { return m_iRightMarginLayoutUnits; }
-	inline UT_sint32			getTopMargin(void) const { return m_iTopMargin; }
-	inline UT_sint32			getTopMarginInLayoutUnits(void) const { return m_iTopMarginLayoutUnits; }
-	inline UT_sint32			getBottomMargin(void) const { return m_iBottomMargin; }
-	inline UT_sint32			getBottomMarginInLayoutUnits(void) const { return m_iBottomMarginLayoutUnits; }
+	bool                        arePageNumbersRestarted (void) const { return m_bRestart;}
+	UT_sint32                   getRestartedPageNumber(void) const { return m_iRestartValue;}
+	UT_sint32                   getTopMargin(void) const;
+	UT_sint32                   getTopMarginInLayoutUnits(void) const;
+    UT_sint32                   getBottomMargin(void) const;
+    UT_sint32                   getBottomMarginInLayoutUnits(void) const;
 	inline UT_sint32			getFooterMargin(void) const { return m_iFooterMargin; }
 	inline UT_sint32			getFooterMarginInLayoutUnits(void) const { return m_iFooterMarginLayoutUnits; }
 	inline UT_sint32			getHeaderMargin(void) const { return m_iHeaderMargin; }
@@ -225,13 +234,18 @@ public:
 
 	fl_HdrFtrSectionLayout*         getHeader(void);
 	fl_HdrFtrSectionLayout*         getFooter(void);
+	fl_HdrFtrSectionLayout*         getHeaderEven(void);
+	fl_HdrFtrSectionLayout*         getFooterEven(void);
+	fl_HdrFtrSectionLayout*         getHeaderFirst(void);
+	fl_HdrFtrSectionLayout*         getFooterFirst(void);
+	fl_HdrFtrSectionLayout*         getHeaderLast(void);
+	fl_HdrFtrSectionLayout*         getFooterLast(void);
 
 	void				setEndnote(fl_DocSectionLayout*);
 	fl_DocSectionLayout* getEndnote(void);
 
 	void				setEndnoteOwner(fl_DocSectionLayout*);
 	fl_DocSectionLayout* getEndnoteOwner(void);
-
 	void				addOwnedPage(fp_Page*);
 	void                            prependOwnedHeaderPage(fp_Page * p_Page);
 	void                            prependOwnedFooterPage(fp_Page * p_Page);
@@ -242,6 +256,12 @@ public:
 	void				checkAndAdjustColumnGap(UT_sint32 iLayoutWidth);
     void                markForReformat(void) { m_bNeedsFormat = true;}
     bool                needsReFormat(void) const { return m_bNeedsFormat;}
+	bool                isThisPageValid(HdrFtrType hfType, fp_Page * pThisPage);
+    void                getVecOfHdrFtrs(UT_Vector * vecHdrFtr);
+	void                formatAllHdrFtr(void);
+	void                checkAndRemovePages(void);
+	void                addValidPages(void);
+
 protected:
 	virtual void		_lookupProperties(void);
 
@@ -250,6 +270,12 @@ protected:
 	*/
 	fl_HdrFtrSectionLayout*		m_pHeaderSL;
 	fl_HdrFtrSectionLayout*		m_pFooterSL;
+	fl_HdrFtrSectionLayout*		m_pHeaderEvenSL;
+	fl_HdrFtrSectionLayout*		m_pFooterEvenSL;
+	fl_HdrFtrSectionLayout*		m_pHeaderFirstSL;
+	fl_HdrFtrSectionLayout*		m_pFooterFirstSL;
+	fl_HdrFtrSectionLayout*		m_pHeaderLastSL;
+	fl_HdrFtrSectionLayout*		m_pFooterLastSL;
 	
 	UT_uint32			m_iNumColumns;
 	UT_uint32			m_iColumnGap;
@@ -261,6 +287,8 @@ protected:
 
 	UT_sint32			m_iSpaceAfter;
 	UT_sint32			m_iSpaceAfterLayoutUnits;
+	bool                m_bRestart;
+	UT_sint32           m_iRestartValue;
 	UT_sint32			m_iLeftMargin;
 	UT_sint32			m_iLeftMarginLayoutUnits;
 	double				m_dLeftMarginUserUnits;
@@ -301,7 +329,7 @@ private:
 	bool                m_bNeedsRebuild;
 };
 
-class fl_HdrFtrSectionLayout : public fl_SectionLayout
+class ABI_EXPORT fl_HdrFtrSectionLayout : public fl_SectionLayout
 {
 	friend class fl_DocListener;
 
@@ -313,11 +341,15 @@ public:
 	HdrFtrType      			getHFType(void) const { return m_iHFType; }
 	void                        setDocSectionLayout(fl_DocSectionLayout * pDSL) { m_pDocSL = pDSL;}
 	void                        setHdrFtr(HdrFtrType iHFType) { 	m_iHFType = iHFType;}
-	virtual bool				recalculateFields(void);
+	virtual bool				recalculateFields(UT_uint32 iUpdateCount);
 	bool                        doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx);
 	void                        localFormat(void);
 	void                        localCollapse(void);
 	void                        markAllRunsDirty(void);
+	void                        checkAndRemovePages(void);
+	void                        addValidPages(void);
+	bool                        isPageHere( fp_Page *pPage);
+	bool                        isPointInHere(PT_DocPosition pos);
 	void                        collapseBlock(fl_BlockLayout * pBlock);
 	virtual void				format(void);
 	virtual void				updateLayout(void);
@@ -333,8 +365,8 @@ public:
 	fl_HdrFtrShadow *               findShadow( fp_Page * pPage);
 	virtual bool 			doclistener_changeStrux(const PX_ChangeRecord_StruxChange * pcrxc);
 	void                                    changeIntoHdrFtrSection( fl_DocSectionLayout * pSL);
-	void						addPage(fp_Page*);
-	void						deletePage(fp_Page*);
+	void				    addPage(fp_Page*);
+	void					deletePage(fp_Page*);
 	void                            clearScreen(void);
 	void                            collapse(void);
 
@@ -377,7 +409,8 @@ private:
 	fp_Container *              m_pHdrFtrContainer;
 };
 
-class fl_HdrFtrShadow : public fl_SectionLayout
+
+class ABI_EXPORT fl_HdrFtrShadow : public fl_SectionLayout
 {
 	friend class fl_DocListener;
 
@@ -407,7 +440,7 @@ protected:
 	fp_Page*					m_pPage;
 };
 
-class fl_ShadowListener : public PL_Listener
+class ABI_EXPORT fl_ShadowListener : public PL_Listener
 {
 public:
 	fl_ShadowListener(fl_HdrFtrSectionLayout* pHFSL, fl_HdrFtrShadow* pShadow);
