@@ -21,37 +21,23 @@
 ** Only one of these is created by the application.
 *****************************************************************/
 
+#define ABIWORD_INTERNAL
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <signal.h>
 
-// HACK to no collide with perl DEBUG
-#ifdef DEBUG
-#define ABI_DEBUG
-#undef DEBUG
-#endif
-
-#ifdef ABI_OPT_PERL
-#include <EXTERN.h>
-#include <perl.h>
-#endif
-
-#ifdef DEBUG
-#define PERL_DEBUG
-#undef DEBUG
-#endif
-
-#ifdef ABI_DEBUG
-#define DEBUG
-#endif
-
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "ut_misc.h"
+#include "ut_PerlBindings.h"
+#include "ut_Script.h"
+#include "ut_unixDirent.h"
 
 #include "xap_Args.h"
 #include "ap_Convert.h"
@@ -65,7 +51,9 @@
 #include "xap_Dlg_MessageBox.h"
 #include "xap_Dialog_Id.h"
 #include "xap_Menu_ActionSet.h"
+#include "xap_Menu_Layouts.h"
 #include "xap_Toolbar_ActionSet.h"
+#include "xap_Toolbar_Layouts.h"
 #include "xav_View.h"
 
 #include "gr_Graphics.h"
@@ -73,7 +61,7 @@
 #include "gr_Image.h"
 #include "ut_bytebuf.h"
 #include "ut_png.h"
-#include "ut_dialogHelper.h"
+#include "xap_UnixDialogHelper.h"
 #include "ut_debugmsg.h"
 
 #include "fv_View.h"
@@ -83,6 +71,10 @@
 #include "xap_EncodingManager.h"
 
 #include "ie_impexp_Register.h"
+#include "xap_EditMethods.h"
+#include "ev_EditMethod.h"
+#include "xap_ModuleManager.h"
+#include "xap_Module.h"
 
 #include "ie_exp.h"
 #include "ie_exp_RTF.h"
@@ -96,6 +88,10 @@
 #include "ap_Prefs_SchemeIds.h"
 #include "gr_Image.h"
 
+#include "xap_ModuleManager.h"
+#include "xap_UnixPSGraphics.h"
+#include "abiwidget.h"
+
 #ifdef GTK_WIN_POS_CENTER_ALWAYS
 #define WIN_POS GTK_WIN_POS_CENTER_ALWAYS
 #else
@@ -123,6 +119,13 @@ AP_UnixApp::AP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 	  m_cacheSelectionView(0),
 	  m_pFrameSelection(0)
 {
+//
+// hack to link abi_widget - thanks fjf
+//
+	if(this == 0)
+	{
+		GtkWidget * pUn = abi_widget_new_with_file("fred.abw");
+	}
 }
 
 /*!
@@ -134,8 +137,7 @@ AP_UnixApp::~AP_UnixApp(void)
     DELETEP(m_pStringSet);
     DELETEP(m_pClipboard);
 
-	IE_Exp::unregisterAllExporters ();
-	IE_Imp::unregisterAllImporters ();
+    IE_ImpExp_UnRegisterXP ();
 }
 
 /*!
@@ -168,69 +170,6 @@ static bool s_createDirectoryIfNecessary(const char * szDir)
     return false;
 }	
 
-#ifdef ABI_OPT_PERL
-
-#ifdef __cplusplus
-#  define EXTERN_C extern "C"
-#else
-#  define EXTERN_C extern
-#endif
-
-EXTERN_C void boot_DynaLoader (CV* cv);
-EXTERN_C void boot_abi (CV* cv);
-
-static void 
-xs_init () {
-	char *file = __FILE__;
-	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
-	/* we want to link to the module code, but until it's stable
-	it's better to have it dynamically loaded...
-	newXS("abi::boot_abi", boot_abi, file);*/
-}
-
-static PerlInterpreter &getPerlInterp()
-{
-	static PerlInterpreter *pPerlInt = 0;
-
-	if (pPerlInt == 0)
-	{
-		char *argv[] = { "", "-Mabi", "-e", "0" };
-#ifdef PERL_DEBUG
-#define DEBUG
-#endif
-		pPerlInt = perl_alloc();
-		perl_construct(pPerlInt);
-		perl_parse(pPerlInt, xs_init, sizeof(argv) / sizeof(char *), argv, (char **)NULL);
-#ifndef ABI_DEBUG
-#undef DEBUG
-#endif
-	}
-
-	return *pPerlInt;
-}
-
-void AP_UnixApp::perlEvalFile(const UT_String &filename)
-{
-#ifdef PERL_DEBUG
-#define DEBUG
-#endif
-	UT_String code("require \"");
-	code += filename;
-	code += '"';
-	getPerlInterp();
-	UT_DEBUGMSG(("executing script file: %s\n", filename.c_str()));
-	perl_eval_pv(code.c_str(), FALSE);
-
-	if (SvTRUE(ERRSV))
-	{
-#ifndef ABI_DEBUG
-#undef DEBUG
-#endif
-		printf("eval error: %s\n", SvPV(ERRSV, PL_na));
- 	}
-}
-#endif
-
 /*!
   Initialize the application.  This involves preferences, keybindings,
   toolbars, graphics, spelling and everything else.  
@@ -244,55 +183,33 @@ bool AP_UnixApp::initialize(void)
     const char * szUserPrivateDirectory = getUserPrivateDirectory();
     bool bVerified = s_createDirectoryIfNecessary(szUserPrivateDirectory);
     UT_ASSERT(bVerified);
-	
+
     // load the preferences.
     
     m_prefs = new AP_UnixPrefs(this);
     m_prefs->fullInit();
 
-    // now that preferences are established, let the xap init
-		   
-    m_pClipboard = new AP_UnixClipboard(this);
-    UT_ASSERT(m_pClipboard);
-    m_pClipboard->initialize();
-    
-    m_pEMC = AP_GetEditMethods();
-    UT_ASSERT(m_pEMC);
-    
-    m_pBindingSet = new AP_BindingSet(m_pEMC);
-    UT_ASSERT(m_pBindingSet);
-	
-    m_pMenuActionSet = AP_CreateMenuActionSet();
-    UT_ASSERT(m_pMenuActionSet);
-    
-    m_pToolbarActionSet = AP_CreateToolbarActionSet();
-    UT_ASSERT(m_pToolbarActionSet);
-    
-    if (! XAP_UNIXBASEAPP::initialize())
-		return false;
 
-	//////////////////////////////////////////////////////////////////
-	// Initialize the importers/exporters
-	//////////////////////////////////////////////////////////////////
-	IE_ImpExp_RegisterXP ();
-    
     //////////////////////////////////////////////////////////////////
     // load the dialog and message box strings
+    //
+    // (we want to do this as soon as possible so that any errors in
+    // the initialization could be properly localized before being
+    // reported to the user)
     //////////////////////////////////////////////////////////////////
 	
     {
 		// assume we will be using the builtin set (either as the main
 		// set or as the fallback set).
-	    
+	
 		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,(XML_Char*)AP_PREF_DEFAULT_StringSet);
 		UT_ASSERT(pBuiltinStringSet);
 		m_pStringSet = pBuiltinStringSet;
-	    
 		// see if we should load an alternative set from the disk
-	    
+	
 		const char * szDirectory = NULL;
 		const char * szStringSet = NULL;
-	    
+	
 		if (   (getPrefsValue(AP_PREF_KEY_StringSet,
 							  (const XML_Char**)&szStringSet))
 			   && (szStringSet)
@@ -326,6 +243,32 @@ bool AP_UnixApp::initialize(void)
 			}
 		}
     }
+
+    // now that preferences are established, let the xap init
+		   
+    m_pClipboard = new AP_UnixClipboard(this);
+    UT_ASSERT(m_pClipboard);
+    m_pClipboard->initialize();
+    
+    m_pEMC = AP_GetEditMethods();
+    UT_ASSERT(m_pEMC);
+    
+    m_pBindingSet = new AP_BindingSet(m_pEMC);
+    UT_ASSERT(m_pBindingSet);
+	
+    m_pMenuActionSet = AP_CreateMenuActionSet();
+    UT_ASSERT(m_pMenuActionSet);
+    
+    m_pToolbarActionSet = AP_CreateToolbarActionSet();
+    UT_ASSERT(m_pToolbarActionSet);
+    
+    if (! XAP_UNIXBASEAPP::initialize())
+		return false;
+
+	//////////////////////////////////////////////////////////////////
+	// Initialize the importers/exporters
+	//////////////////////////////////////////////////////////////////
+	IE_ImpExp_RegisterXP ();
 	
     // Now we have the strings loaded we can populate the field names correctly
     int i;
@@ -342,8 +285,34 @@ bool AP_UnixApp::initialize(void)
 		UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
     }
 
+    ///////////////////////////////////////////////////////////////////////
+    /// Build a labelset so the plugins can add themselves to something ///
+    ///////////////////////////////////////////////////////////////////////
+
+	const char * szMenuLabelSetName = NULL;
+	if (getPrefsValue( AP_PREF_KEY_MenuLabelSet, (const XML_Char**)&szMenuLabelSetName)
+		&& (szMenuLabelSetName) && (*szMenuLabelSetName))
+	{
+		;
+	}
+	else
+		szMenuLabelSetName = AP_PREF_DEFAULT_MenuLabelSet ;
+
+	getMenuFactory()->buildMenuLabelSet(szMenuLabelSetName);
+	bool bLoadPlugins = true;
+	bool bFound = getPrefsValueBool(XAP_PREF_KEY_AutoLoadPlugins,&bLoadPlugins);
+	if(bLoadPlugins || !bFound)
+	{
+		loadAllPlugins();
+	}
     //////////////////////////////////////////////////////////////////
 
+#ifdef ABI_OPT_PERL
+    // hack to keep the perl bindings working on unix
+    UT_ScriptLibrary& instance = UT_ScriptLibrary::instance(); 
+    instance.registerScript ( new UT_PerlScriptSniffer () );
+#endif
+
     return true;
 }
 
@@ -371,6 +340,13 @@ XAP_Frame * AP_UnixApp::newFrame(void)
 */
 bool AP_UnixApp::shutdown(void)
 {
+//
+// Save our toolbars if we're customizable
+//
+	if(areToolbarsCustomizable())
+	{
+		m_pToolbarFactory->saveToolbarsInCurrentScheme();
+	}
     if (m_prefs->getAutoSavePrefs())
 		m_prefs->savePrefsFile();
 
@@ -606,6 +582,144 @@ bool AP_UnixApp::canPasteFromClipboard(void)
 #endif
 }
 
+// return > 0 for directory entries ending in ".so"
+#if defined (__APPLE__) || defined (__FreeBSD__) || defined (__OpenBSD__) \
+	|| defined(_AIX)
+static int so_only (struct dirent *d)
+#else
+static int so_only (const struct dirent *d)
+#endif
+{
+  const char * name = d->d_name;
+
+  if ( name )
+    {
+      int len = strlen (name);
+
+      if (len >= 7)
+	{
+	  if(!strcmp(name+(len-7), ".bundle"))
+	    return 1;
+	}
+
+      if (len >= 3)
+	{
+	  if(!strcmp(name+(len-3), ".so"))
+	    return 1;
+	}
+    }
+  return 0;
+}
+
+void AP_UnixApp::loadAllPlugins ()
+{
+  struct dirent **namelist;
+  int n = 0;
+
+  UT_String pluginList[2];
+  UT_String pluginDir;
+
+  // the global plugin directory
+  pluginDir = getAbiSuiteAppDir();
+  pluginDir += "/plugins/";
+  pluginList[0] = pluginDir;
+
+  // the user-local plugin directory
+  pluginDir = getUserPrivateDirectory ();
+  pluginDir += "/AbiWord/plugins/";
+  pluginList[1] = pluginDir;
+
+  for(UT_uint32 i = 0; i < (sizeof(pluginList)/sizeof(pluginList[0])); i++)
+  {
+      pluginDir = pluginList[i];
+
+      n = scandir(pluginDir.c_str(), &namelist, so_only, alphasort);
+      UT_DEBUGMSG(("DOM: found %d plugins in %s\n", n, pluginDir.c_str()));
+
+      if (n > 0)
+	  {
+		  while(n--) 
+		  {
+			  UT_String plugin (pluginDir + namelist[n]->d_name);
+
+			  UT_DEBUGMSG(("DOM: loading plugin %s\n", plugin.c_str()));
+
+			  struct stat pluginInfo;
+			  if (stat (plugin.c_str(), &pluginInfo) != 0)
+			  {
+				  UT_DEBUGMSG(("FJF: stat failed... odd.\n"));
+				  free(namelist[n]);
+				  continue;
+			  }
+#ifdef S_ISDIR
+			  bool pluginIsBundle = (S_ISDIR (pluginInfo.st_mode) != 0);
+#else
+			  bool pluginIsBundle = ((pluginInfo.st_mode & S_IFDIR) != 0);
+#endif
+			  /* Bundles "*.bundle" are directories, plugins "*.so" are modules.
+			   * so_only checks only the suffix, so we need to confirm nature here:
+			   */
+			  if (pluginIsBundle)
+			  {
+				  int len = strlen (namelist[n]->d_name);
+				  if (len < 8)
+				  {
+					  UT_DEBUGMSG(("FJF: bad name for a bundle\n"));
+					  free(namelist[n]);
+					  continue;
+				  }
+				  if(strcmp (namelist[n]->d_name+(len-7), ".bundle") != 0)
+				  {
+					  UT_DEBUGMSG(("FJF: not really a bundle?\n"));
+					  free(namelist[n]);
+					  continue;
+				  }
+			  }
+			  else
+			  {
+				  int len = strlen (namelist[n]->d_name);
+				  if (len < 4)
+				  {
+					  UT_DEBUGMSG(("FJF: bad name for a plugin\n"));
+					  free(namelist[n]);
+					  continue;
+				  }
+				  if(strcmp (namelist[n]->d_name+(len-3), ".so") != 0)
+				  {
+					  UT_DEBUGMSG(("FJF: not really a plugin?\n"));
+					  free(namelist[n]);
+					  continue;
+				  }
+			  }
+			  if (pluginIsBundle)
+			  {
+				  if (XAP_ModuleManager::instance().loadBundle (plugin.c_str(), namelist[n]->d_name))
+				  {
+					  UT_DEBUGMSG(("FJF: loaded bundle: %s\n", namelist[n]->d_name));
+				  }
+				  else
+				  {
+					  UT_DEBUGMSG(("FJF: didn't load bundle: %s\n", namelist[n]->d_name));
+				  }
+				  free(namelist[n]);
+				  continue;
+			  }
+
+			  if (XAP_ModuleManager::instance().loadModule (plugin.c_str()))
+			  {
+				  UT_DEBUGMSG(("DOM: loaded plugin: %s\n", namelist[n]->d_name));
+			  }
+			  else
+			  {
+				  UT_DEBUGMSG(("DOM: didn't load plugin: %s\n", namelist[n]->d_name));
+			  }
+			  free(namelist[n]);
+		  }
+		  free(namelist);
+      }
+  }
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -1048,7 +1162,7 @@ rms:  I'm adding something here to get a localized splash screen
 			}
 		}
 		// create a centered window the size of our image
-		wSplash = gtk_window_new(GTK_WINDOW_POPUP);
+		wSplash = gtk_window_new(GTK_WINDOW_POPUP); //GTK_WINDOW_DIALOG
 		gtk_window_set_default_size (GTK_WINDOW (wSplash),
 									 iSplashWidth, iSplashHeight);
 		gtk_window_set_policy(GTK_WINDOW(wSplash), FALSE, FALSE, FALSE);
@@ -1089,7 +1203,7 @@ rms:  I'm adding something here to get a localized splash screen
 
     DELETEP(pBB);
 #ifdef DEBUG
-    DELETEP(buf);
+    free(buf);
 #endif
 
     return pSplashImage;
@@ -1118,6 +1232,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
     bool bShow = false;
     bool bNoSplash = false;
     bool bHelp = false;
+    bool bVersion = false;
     
     for (int k = 1; k < Args.m_argc; k++)
 		if (*Args.m_argv[k] == '-')
@@ -1126,6 +1241,16 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 			if ((strcmp(Args.m_argv[k],"-to") == 0)
 				|| (strcmp(Args.m_argv[k],"--to") == 0))
 				bTo = true;
+
+			// Do a quick and dirty find for "-to"
+			if ((strcmp(Args.m_argv[k],"-to-png") == 0)
+				|| (strcmp(Args.m_argv[k],"--to-png") == 0))
+				bTo = true;
+
+			// Do a quick and dirty find for "-to"
+			else if ((strcmp(Args.m_argv[k],"-p") == 0)
+				|| (strcmp(Args.m_argv[k],"--print") == 0))
+				bTo = true;
 		
 		// Do a quick and dirty find for "-show"
 			else if ((strcmp(Args.m_argv[k],"-show") == 0)
@@ -1136,15 +1261,26 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 			else if ((strcmp(Args.m_argv[k],"-nosplash") == 0)
 					 || (strcmp(Args.m_argv[k],"--nosplash") == 0))
 				bNoSplash = true;
-		
+
+ 	// Do a quick and dirty find for "--plugin"
+			else if ((strcmp(Args.m_argv[k],"-plugin") == 0)
+					 || (strcmp(Args.m_argv[k],"--plugin") == 0))
+				bNoSplash = true;
+
 		// Do a quick and dirty find for "-help",
 		// "--help" or "-h"
 			else if (strncmp(Args.m_argv[k],"-h",2) == 0 ||
 					 strncmp(Args.m_argv[k],"--h",3) == 0)
 				bHelp = true;
+
+			else if ((strcmp(Args.m_argv[k],"-version") == 0)
+				 || (strcmp(Args.m_argv[k],"--version") == 0))
+			  {
+			    bVersion = true;
+			  }
 		}
     
-    if((bTo && !bShow) || bHelp)
+    if((bTo && !bShow) || bHelp || bVersion)
     {
 		bShowSplash = false;
 		bShowApp = false;
@@ -1157,24 +1293,19 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
     // HACK : but need to be here to throw the splash screen as
     // HACK : soon as possible.
 
-    if (bShowSplash || bShowApp)
-    {
-		gtk_set_locale();
-		gtk_init(&Args.m_argc,&Args.m_argv);
-    }
+    gtk_set_locale();
+    gtk_init(&Args.m_argc,&Args.m_argv);
     
     AP_UnixApp * pMyUnixApp = new AP_UnixApp(&Args, szAppName);
+
+    pMyUnixApp->setDisplayStatus(bShowApp);
     
-    if (bShowApp)
-    {
-		// if the initialize fails, we don't have icons, fonts, etc.
-		if (!pMyUnixApp->initialize())
-		{
-			delete pMyUnixApp;
-			return -1;	// make this something standard?
-		}
-    }
-	
+    // if the initialize fails, we don't have icons, fonts, etc.
+    if (!pMyUnixApp->initialize())
+      {
+	delete pMyUnixApp;
+	return -1;	// make this something standard?
+      }
 
     const XAP_Prefs * pPrefs = pMyUnixApp->getPrefs();
 	UT_ASSERT(pPrefs);
@@ -1195,6 +1326,12 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 		delete pMyUnixApp;
 		return 0;
     }
+
+    if(bVersion)
+      {
+	printf( "%s\n", XAP_App::s_szBuild_Version );
+	return 0;
+      }
     
     // Setup signal handlers, primarily for segfault
     // If we segfaulted before here, we *really* blew it
@@ -1205,8 +1342,8 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
     
     sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
     sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
-
-#ifndef AIX
+/* #ifndef AIX - I presume these are always #define not extern... -fjf */
+#if defined (SA_NODEFER) && defined (SA_RESETHAND)
     sa.sa_flags = SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
 #else
     sa.sa_flags = 0;
@@ -1226,10 +1363,14 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
     {
 		// turn over control to gtk
 		gtk_main();
-		pMyUnixApp->shutdown();
     }
+    else
+      {
+	UT_DEBUGMSG(("DOM: not parsing command line or showing app\n"));
+      }
     
     // destroy the App.  It should take care of deleting all frames.
+    pMyUnixApp->shutdown();
     delete pMyUnixApp;
     
     return 0;
@@ -1255,28 +1396,21 @@ bool AP_UnixApp::parseCommandLine()
     int k;
     int kWindowsOpened = 0;
     char *to = NULL;
+    bool topng = false ;
     int verbose = 1;
     bool show = false;
+
+    char *printto = NULL;
+	char * plugin = NULL;
+
 #ifdef ABI_OPT_PERL
     bool script = false;
 #endif
 
     for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
     {
-		if (*m_pArgs->m_argv[k] == '-')
+        if (*m_pArgs->m_argv[k] == '-')
 		{
-#if 0
-			// This code is currently unused.
-			// We don't load libraries
-			if ((strcmp(m_pArgs->m_argv[k],"-lib") == 0)
-					 || (strcmp(m_pArgs->m_argv[k],"--lib") == 0))
-			{
-				// [-lib <AbiSuiteLibDirectory>]
-				// we've already processed this when we initialized the App class
-				k++;
-			}
-#endif
-	    
 			if ((strcmp(m_pArgs->m_argv[k],"-dumpstrings") == 0)
 				|| (strcmp(m_pArgs->m_argv[k],"--dumpstrings") == 0))
 			{
@@ -1331,6 +1465,28 @@ bool AP_UnixApp::parseCommandLine()
 			{
 				k++;
 				to = m_pArgs->m_argv[k];
+				UT_DEBUGMSG(("DOM: got --to: %s\n", to));
+			}
+			else if ((strcmp (m_pArgs->m_argv[k],"-to-png") == 0)
+					 || (strcmp (m_pArgs->m_argv[k],"--to-png") == 0))
+			{
+				k++;
+				to = m_pArgs->m_argv[k];
+				UT_DEBUGMSG(("DOM: got --to-png: %s\n", to));
+			}
+			else if ((strcmp (m_pArgs->m_argv[k],"-plugin") == 0)
+					 || (strcmp (m_pArgs->m_argv[k],"--plugin") == 0))
+			{
+				k++;
+				plugin = m_pArgs->m_argv[k];
+				UT_DEBUGMSG(("DOM: got --plugin: %s\n", to));
+			}
+			else if ((strcmp (m_pArgs->m_argv[k],"-print") == 0)
+					 || (strcmp (m_pArgs->m_argv[k],"--print") == 0))
+			{
+				k++;
+				printto = m_pArgs->m_argv[k];
+				UT_DEBUGMSG(("DOM: got --print: %s\n", printto));
 			}
 			else if ((strcmp (m_pArgs->m_argv[k], "-show") == 0)
 					 || (strcmp (m_pArgs->m_argv[k], "--show") == 0))
@@ -1359,16 +1515,20 @@ bool AP_UnixApp::parseCommandLine()
 				// [-script]
 				if (k < m_pArgs->m_argc)
 				{
-					perlEvalFile(m_pArgs->m_argv[k]);
+					UT_PerlBindings& pb(UT_PerlBindings::getInstance());
+
+					if (!pb.evalFile(m_pArgs->m_argv[k]))
+						printf("%s\n", pb.errmsg().c_str());
+					
 					script = true;
 				}
 				else
-					UT_DEBUGMSG(("No script file to execute in --script option\n"));
+					printf("No script file to execute in --script option\n");
 			}
 #endif
 			else  
 			{
-				UT_DEBUGMSG(("Unknown command line option [%s]\n",m_pArgs->m_argv[k]));
+				printf("Unknown command line option [%s]\n", m_pArgs->m_argv[k]);
 				// TODO don't know if it has a following argument or not -- assume not
 				_printUsage();
 				return false;
@@ -1380,10 +1540,42 @@ bool AP_UnixApp::parseCommandLine()
 			if (to) 
 			{
 				AP_Convert * conv = new AP_Convert(getApp());
+				UT_DEBUGMSG(("DOM: inside other --to: %s\n", to));
 				conv->setVerbose(verbose);
 				conv->convertTo(m_pArgs->m_argv[k], to);
 				delete conv;
 			}
+			else if ( topng )
+			  {
+				AP_Convert * conv = new AP_Convert(getApp());
+				UT_DEBUGMSG(("DOM: inside --to-png\n"));
+				conv->setVerbose(verbose);
+				conv->convertToPNG(m_pArgs->m_argv[k]);
+				delete conv;
+			  }
+			else if (printto)
+			  {
+			    const char * file = m_pArgs->m_argv[k];
+			    UT_DEBUGMSG(("DOM: inside other --print %s %s\n", printto, file));
+			    if (file)
+			      {
+				AP_Convert * conv = new AP_Convert(this);
+				conv->setVerbose(verbose);
+				
+				PS_Graphics * pG = new PS_Graphics ((printto[0] == '|' ? printto+1 : printto), file,
+								    getApplicationName(), getFontManager(),
+								    (printto[0] != '|'), this);
+				conv->print (file, pG);			       
+
+				delete pG;
+				delete conv;
+                  }
+			    else
+			      {
+				// no filename
+				printf("Error: --print specified but no file to print was given!\n");
+			      }
+			  }
 			else
 			{
 				AP_UnixFrame * pFirstUnixFrame = new AP_UnixFrame(this);
@@ -1420,10 +1612,53 @@ bool AP_UnixApp::parseCommandLine()
 		}
     }
 	
-    // command-line conversion may not open any windows at all
-    if (to && !show)
-		return true;
-    
+    // command-line conversion or printing may not open any windows at all
+    if ((to || printto) && !show)
+		return false;
+    if(plugin)
+    {
+//
+// Start a plugin rather than the main abiword application.
+//
+	    const char * szName = NULL;
+		XAP_Module * pModule = NULL;
+
+		const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+		bool bFound = false;
+		for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
+		{
+			pModule = (XAP_Module *)pVec->getNthItem (i);
+			szName = pModule->getModuleInfo()->name;
+			if(UT_strcmp(szName,plugin) == 0)
+			{
+				bFound = true;
+			}
+		}
+		if(!bFound)
+		{
+			printf("Plugin %s not found or loaded \n",plugin);
+			return false;
+		}
+//
+// You must put the name of the ev_EditMethod in the usage field
+// of the plugin registered information.
+//
+		const char * evExecute = pModule->getModuleInfo()->usage;
+		EV_EditMethodContainer* pEMC = getEditMethodContainer();
+		const EV_EditMethod * pInvoke = pEMC->findEditMethodByName(evExecute);
+		if(!pInvoke)
+		{
+			printf("Plugin %s invoke method %s not found \n",plugin,evExecute);
+			return false;
+		}
+//
+// Execute the plugin, then quit
+//
+		ev_EditMethod_invoke(pInvoke, "Called From UnixGnomeApp");
+		return false;
+	}
+
+
     if (kWindowsOpened == 0)
     {
 		// no documents specified or were able to be opened,
