@@ -150,7 +150,8 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 		AP_RID_DIALOG_LIST_BUTTON_DEFAULT		, AP_STRING_ID_DLG_Lists_SetDefault,
 		AP_RID_DIALOG_LIST_STATIC_FORMAT		, AP_STRING_ID_DLG_Lists_Format,
 		AP_RID_DIALOG_LIST_STATIC_FONT			, AP_STRING_ID_DLG_Lists_Font,
-		AP_RID_DIALOG_LIST_BTN_FONT				, AP_STRING_ID_DLG_Lists_Font, // FIXME, should append ellipsis
+		AP_RID_DIALOG_LIST_BTN_FONT				, AP_STRING_ID_DLG_Lists_ButtonFont,
+		AP_RID_DIALOG_LIST_STATIC_FORMAT		, AP_STRING_ID_DLG_Lists_DelimiterString,
 		AP_RID_DIALOG_LIST_STATIC_LEVEL			, AP_STRING_ID_DLG_Lists_Level,
 		AP_RID_DIALOG_LIST_STATIC_START_AT		, AP_STRING_ID_DLG_Lists_Start,
 		AP_RID_DIALOG_LIST_STATIC_LIST_ALIGN	, AP_STRING_ID_DLG_Lists_Align,
@@ -206,6 +207,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	activate();
 
 	m_pAutoUpdateLists = UT_Timer::static_constructor(autoupdateLists, this);
+	m_bDestroy_says_stopupdating = false;
 	m_pAutoUpdateLists->set(500);	// auto-updater at 1/2 Hz
 
 	return 1;							// 1 == we did not call SetFocus()
@@ -284,6 +286,7 @@ BOOL AP_Win32Dialog_Lists::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	case AP_RID_DIALOG_LIST_EDIT_INDENT_ALIGN:
 	case AP_RID_DIALOG_LIST_EDIT_START_AT:
 	case AP_RID_DIALOG_LIST_EDIT_LEVEL:
+	case AP_RID_DIALOG_LIST_EDIT_DECIMAL:
 		if (wNotifyCode == EN_CHANGE)
 		{
 			_getDisplayedData(wId);
@@ -362,6 +365,10 @@ void AP_Win32Dialog_Lists::destroy(void)
 
 	// Check that our hWnd still is valid. This is not always the case,
 	// such as when shutting down the application.
+	m_bDestroy_says_stopupdating = true;
+	while (m_bAutoUpdate_happening_now == true) ;
+	m_pAutoUpdateLists->stop();
+	setAnswer(AP_Dialog_Lists::a_CLOSE);	
 	if (IsWindow(m_hThisDlg))
 	{
 		_win32Dialog.destroyWindow();
@@ -622,6 +629,13 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 		AP_STRING_ID_DLG_Lists_Upper_Case_List,
 		AP_STRING_ID_DLG_Lists_Lower_Roman_List,
 		AP_STRING_ID_DLG_Lists_Upper_Roman_List
+/*
+#ifdef BIDI_ENABLED
+		,AP_STRING_ID_DLG_Lists_Arabic_List,
+
+		AP_STRING_ID_DLG_Lists_Hebrew_List
+#endif
+*/
 	};
 
 	const XAP_String_Id*	pIDs;
@@ -651,7 +665,7 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 	HDC hDCcombo = GetDC(hComboStyle);
 	HDC hDC = CreateCompatibleDC(hDCcombo);
 	ReleaseDC(hComboStyle, hDCcombo);
-
+	
 	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
 	SelectObject(hDC, hFont);
 
@@ -702,6 +716,15 @@ void AP_Win32Dialog_Lists::_styleChanged()
   	setDirty();
 	setNewListType(_getListTypeFromCombos());
 	setbisCustomized(false);
+/*
+#ifdef BIDI_ENABLED
+
+	// have to do this, since for instance the decimal delimter can change
+
+	fillUncustomizedValues();
+
+#endif
+*/
 	_previewExposed();
 	_setDisplayedData();
 	_enableControls();
@@ -727,8 +750,10 @@ static const UT_sint32 rgCustomIds[] =
 	AP_RID_DIALOG_LIST_EDIT_FORMAT,
 	AP_RID_DIALOG_LIST_STATIC_FONT,
 	AP_RID_DIALOG_LIST_STATIC_LEVEL,
-	AP_RID_DIALOG_LIST_EDIT_LEVEL,
-	AP_RID_DIALOG_LIST_SPIN_LEVEL,
+	AP_RID_DIALOG_LIST_STATIC_DECIMAL,
+	AP_RID_DIALOG_LIST_EDIT_DECIMAL,
+	//AP_RID_DIALOG_LIST_EDIT_LEVEL,
+	//AP_RID_DIALOG_LIST_SPIN_LEVEL,
 	AP_RID_DIALOG_LIST_BTN_FONT
 };
 
@@ -795,8 +820,10 @@ static void updateControlValue(XAP_Win32DialogHelper& helper, UT_sint32 id, LPCS
 //
 void AP_Win32Dialog_Lists::_setDisplayedData()
 {
-	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_START_AT,
-						getiStartValue());
+	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_START_AT,getiStartValue());
+	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_FORMAT	, getDelim());
+	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_LEVEL	, getiLevel());
+	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_DECIMAL, getDecimal());
 
 	updateControlValue(_win32Dialog,AP_RID_DIALOG_LIST_EDIT_LIST_ALIGN,
 						UT_convertToDimensionlessString(getfAlign(), ".2"));
@@ -804,9 +831,6 @@ void AP_Win32Dialog_Lists::_setDisplayedData()
 	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_INDENT_ALIGN,
 						UT_convertToDimensionlessString(getfIndent(), ".2"));
 
-	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_FORMAT	, getDelim());
-	updateControlValue(_win32Dialog, AP_RID_DIALOG_LIST_EDIT_LEVEL	, getiLevel());
-
 	_setListType(getNewListType());
 
 	// Resume should only be enabled if the cursor is at/in a list
@@ -870,6 +894,11 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 		_win32Dialog.getControlText(AP_RID_DIALOG_LIST_EDIT_FORMAT, szTmp, sizeof(szTmp));
 		copyCharToDelim(szTmp);
 	}
+	if (controlId == -1 || controlId == AP_RID_DIALOG_LIST_EDIT_DECIMAL)
+	{
+		_win32Dialog.getControlText(AP_RID_DIALOG_LIST_EDIT_DECIMAL, szTmp, sizeof(szTmp));
+		copyCharToDecimal(szTmp);
+	}
 }
 
 //
@@ -879,7 +908,7 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 {
 	const int iType  = _getTypeComboCurSel();
-	const int iStyle = _getStyleComboCurSel();
+	int iStyle = _getStyleComboCurSel();
 
 	if (iType == 0	/* List Type "None" selected	*/ ||
 		iType < 0	/* no selection at all ???		*/ ||
@@ -893,6 +922,9 @@ List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 		return (List_Type)(iStyle + BULLETED_LIST);
 	}
 
+	if(iStyle >= BULLETED_LIST)
+		iStyle += (OTHER_NUMBERED_LISTS - BULLETED_LIST + 1);
+
 	// Numbered List, but just to make really REALLY sure, we assert it
 	UT_ASSERT(IS_NUMBERED_LIST_TYPE(iStyle));
 
@@ -913,6 +945,8 @@ void AP_Win32Dialog_Lists::_setListType(List_Type type)
 	{
 		iType = 2;
 		iStyle = type;
+		if(iStyle > BULLETED_LIST)
+			iStyle -= (OTHER_NUMBERED_LISTS + 1 - BULLETED_LIST);
 	}
 	else
 	{
@@ -965,11 +999,23 @@ void AP_Win32Dialog_Lists::_selectFont()
 	}
 
 	const XML_Char** props_in = NULL;
+
+	bool bUnderline = false;
+
+	bool bOverline = false;
+
+	bool bStrikeOut = false;
+
+	bool bTopLine = false;
+
+	bool bBottomLine = false;
+
+
 	if (pView->getCharFormat(&props_in))
 	{
 		// stuff font properties into the dialog.
 
-		pDialog->setFontFamily(getFont());
+		pDialog->setFontFamily(UT_getAttribute("font-family", props_in));
 		pDialog->setFontSize(UT_getAttribute("font-size", props_in));
 		pDialog->setFontWeight(UT_getAttribute("font-weight", props_in));
 		pDialog->setFontStyle(UT_getAttribute("font-style", props_in));
@@ -980,15 +1026,16 @@ void AP_Win32Dialog_Lists::_selectFont()
 		// worry about initializing a combo box with a choice
 		// (and because they are all stuck under one CSS attribute).
 
-		bool bUnderline = false;
-		bool bOverline = false;
-		bool bStrikeOut = false;
 		const XML_Char * s = UT_getAttribute("text-decoration", props_in);
 		if (s)
 		{
 			bUnderline = (strstr(s, "underline") != NULL);
 			bOverline = (strstr(s, "overline") != NULL);
 			bStrikeOut = (strstr(s, "line-through") != NULL);
+
+			bTopLine = (strstr(s, "topline") != NULL);
+
+			bBottomLine = (strstr(s, "bottomline") != NULL);
 		}
 		pDialog->setFontDecoration(bUnderline,bOverline,bStrikeOut,false,false);
 
@@ -1005,21 +1052,53 @@ void AP_Win32Dialog_Lists::_selectFont()
 	}
 
 	const XML_Char* pszFont;
+	bool bDirty = false;
+
 
 	if (pDialog->getChangedFontFamily(&pszFont))
+
+/*
+
+		|| pDialog->getChangedFontSize(&pszFont)
+
+		|| pDialog->getChangedFontWeight(&pszFont)
+
+		|| pDialog->getChangedFontStyle(&pszFont)
+
+		|| pDialog->getChangedBGColor(&pszFont)
+
+		|| pDialog->getChangedColor(&pszFont)
+
+		|| pDialog->getChangedUnderline(&bUnderline)
+
+		|| pDialog->getChangedOverline(&bOverline)
+
+		|| pDialog->getChangedStrikeOut(&bStrikeOut)
+
+		|| pDialog->getChangedTopline(&bTopLine)
+
+		|| pDialog->getChangedBottomline(&bBottomLine)
+
+		)
+
+*/
 	{
+
+		setDirty();
 		copyCharToFont(pszFont);
 		_previewExposed();
+
+		_enableControls();
 	}
 }
 
 
-void AP_Win32Dialog_Lists::autoupdateLists(UT_Worker * pTimer)
+void AP_Win32Dialog_Lists::autoupdateLists(UT_Worker * pWorker)
 {
-	UT_ASSERT(pTimer);
+	UT_ASSERT(pWorker);
 	// this is a static callback method and does not have a 'this' pointer.
 	AP_Win32Dialog_Lists* pDialog =
-		reinterpret_cast<AP_Win32Dialog_Lists*>(pTimer->getInstanceData());
+		reinterpret_cast<AP_Win32Dialog_Lists*>(pWorker->getInstanceData());
 	// Handshaking code. Plus only update if something in the document
 	// changed.
 
@@ -1035,14 +1114,21 @@ void AP_Win32Dialog_Lists::autoupdateLists(UT_Worker * pTimer)
 		if (!pView || pView->getTick() != pDialog->getTick())
 		{
 			pDialog->setTick(pView ? pView->getTick() : 0);
+			pDialog->PopulateDialogData();
+			pDialog->_setDisplayedData();
+			pDialog->clearDirty();
+			// This is done improperly by _setDisplayedData()
+			// since it leaves the Apply button enabled.
 			pDialog->_enableControls();
-			pDialog->_previewExposed();
+			// This is done by _setDisplayedData();
+			//pDialog->_previewExposed();
+
 		}
 	}
 }
 
 const XML_Char* AP_Win32Dialog_Lists::_getDingbatsFontName() const
 {
-	return "Wingdings";
+	return "Dingbats";
 }
 
