@@ -35,7 +35,6 @@
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
 #include "pp_Property.h"
-#include "xap_App.h"
 #include "ap_Prefs.h"
 #include "pd_Style.h"
 #include "fd_Field.h"
@@ -83,8 +82,8 @@ protected:
 	void				_handleDataItems(void);
 	void                _outputData(const UT_UCSChar * data, UT_uint32 length);
 
-	void				_convertFontSize(char* szDest, const char* szFontSize);
-	void                _convertColor(char* szDest, const char* pszColor);
+	void				_convertFontSize(UT_String& szDest, const char* szFontSize);
+	void                _convertColor(UT_String& szDest, const char* pszColor);
 
 	void				_closeSection(void);
 	void				_closeBlock(void);
@@ -92,6 +91,7 @@ protected:
 	void				_openBlock(PT_AttrPropIndex api);
 	void				_openSection(PT_AttrPropIndex api);
 	void				_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_uint32 len);
+	void                _writeMarginSize(PT_AttrPropIndex api, char * name);
 
 private:
 	PD_Document *		m_pDocument;
@@ -108,6 +108,9 @@ private:
 	int                 m_iImgCnt;
 };
 
+// KWord version 1 width/height are in mm
+static const UT_Dimension kword_1_unit = DIM_MM;
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -115,10 +118,10 @@ private:
 
 // completely generic code to allow this to be a plugin
 
-ABI_PLUGIN_DECLARE("KWord")
-
 #include "xap_Module.h"
 
+ABI_PLUGIN_DECLARE("KWord")
+
 // we use a reference-counted sniffer
 static IE_Exp_KWord_1_Sniffer * m_sniffer = 0;
 
@@ -169,7 +172,7 @@ ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
 								 UT_uint32 release)
 {
-	return isCurrentAbiVersion(major, minor, release);
+  return 1;
 }
 
 #endif
@@ -309,12 +312,6 @@ bool s_KWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			{
 			case PTO_Image:
 			{
-				//char buf[16];
-				//sprintf(buf, "%d.png", m_iImgCnt++);
-				//m_pie->write("<fo:external-graphic src=\"");
-				//m_pie->write(m_pie->getFileName());
-				//m_pie->write(buf);
-				//m_pie->write("\"/>\n");
 				return true;
 			}
 
@@ -323,6 +320,11 @@ bool s_KWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				return true;
 			}
 
+				// todo: support these
+			case PTO_Hyperlink:
+			case PTO_Bookmark:
+			  return true;
+
 			default:
 			{
 				UT_ASSERT(0);
@@ -435,22 +437,6 @@ bool s_KWord_1_Listener::insertStrux(PL_StruxFmtHandle /*sfh*/,
 /*****************************************************************/
 /*****************************************************************/
 
-static const char *
-preferedUnitString(fp_PageSize::Unit docUnit)
-{
-	if(docUnit == fp_PageSize::cm)
-		return "mm";
-	else if(docUnit == fp_PageSize::mm)
-		return "mm";
-	else if(docUnit == fp_PageSize::inch)
-		return "inch";
-	else
-	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return "";
-	}
-}
-
 static const char *
 justificationToNumber(const char * justification_name)
 {
@@ -478,62 +464,62 @@ abiPageSizeToKoPageFormat (fp_PageSize abi_page_size)
 {
 	// The goofy order of the pagesizes is due to koGlobal.h
 	switch (fp_PageSize::NameToPredefined(abi_page_size.getPredefinedName())) {
-		case fp_PageSize::A3:
+		case fp_PageSize::psA3:
 			return "0";
 			break;
-		case fp_PageSize::A4:
+		case fp_PageSize::psA4:
 			return "1";
 			break;
-		case fp_PageSize::A5:
+		case fp_PageSize::psA5:
 			return "2";
 			break;
-		case fp_PageSize::Letter:
+		case fp_PageSize::psLetter:
 			return "3";
 			break;
-		case fp_PageSize::Legal:
+		case fp_PageSize::psLegal:
 			return "4";
 			break;
 		// FIXME: I don't understand what is meant in KWord by `screen'
 		// sized paper
-		case fp_PageSize::Custom:
+		case fp_PageSize::psCustom:
 			return "6";
 			break;
-		case fp_PageSize::B5:
+		case fp_PageSize::psB5:
 			return "7";
 			break;
 #if 0 // This requires the `lot more page sizes' patch
-		case fp_PageSize::Executive:
+		case fp_PageSize::psExecutive:
 			return "8";
 			break;
 #endif
 #if 0 // KWord DTD 2 sizes
-		case fp_PageSize::A0:
+		case fp_PageSize::psA0:
 			return 9;
-		case fp_PageSize::A1:
+		case fp_PageSize::psA1:
 			return 10;
-		case fp_PageSize::A2:
+		case fp_PageSize::psA2:
 			return 11;
-		case fp_PageSize::A6:
+		case fp_PageSize::psA6:
 			return 12;
-		case fp_PageSize::A7:
+		case fp_PageSize::psA7:
 			return 13;
-		case fp_PageSize::A8:
+		case fp_PageSize::psA8:
 			return 14;
-		case fp_PageSize::A9:
+		case fp_PageSize::psA9:
 			return 15;
-		case fp_PageSize::B0:
+		case fp_PageSize::psB0:
 			return 16;
-		case fp_PageSize::B1:
+		case fp_PageSize::psB1:
 			return 17;
-		case fp_PageSize::B10:
+		case fp_PageSize::psB10:
 			return 18;
-		case fp_PageSize::B2:
+		case fp_PageSize::psB2:
 			return 19;
-		case fp_PageSize::B3:
+		case fp_PageSize::psB3:
 			return 20;
-		case fp_PageSize::B4:
+		case fp_PageSize::psB4:
 			return 21;
-		case fp_PageSize::B6:
+		case fp_PageSize::psB6:
 			return 22;
 #endif
 		default:
@@ -541,6 +527,23 @@ abiPageSizeToKoPageFormat (fp_PageSize abi_page_size)
 			break;
 	}
 }
+
+void s_KWord_1_Listener::_writeMarginSize(PT_AttrPropIndex api, char * name)
+{
+        UT_String buf;
+	const XML_Char * szValue;
+	const PP_AttrProp * pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+	UT_ASSERT((bHaveProp));
+
+	UT_String_sprintf(buf, "page-margin-%s", name);
+	szValue = PP_evalProperty(buf.c_str(), 
+				  NULL, NULL, pAP, m_pDocument, true);
+	UT_String_sprintf(buf," %s=\"%f",name, UT_convertToDimension(szValue, kword_1_unit));
+	m_pie->write(buf);
+	m_pie->write("\"");
+
+}
 		
 void s_KWord_1_Listener::_handlePageSize(PT_AttrPropIndex api)
 {
@@ -556,9 +559,6 @@ void s_KWord_1_Listener::_handlePageSize(PT_AttrPropIndex api)
 
 	m_pie->write("<PAPER");
 
-	// KWord version 1 width/height are in mm
-	const fp_PageSize::Unit kword_1_unit = fp_PageSize::mm;
-	
 	m_pie->write(" format=\"");
 	m_pie->write(abiPageSizeToKoPageFormat(m_pDocument->m_docPageSize));
 	m_pie->write("\"");
@@ -576,41 +576,26 @@ void s_KWord_1_Listener::_handlePageSize(PT_AttrPropIndex api)
 	// FIXME: put something useful here
 	m_pie->write(" columnspacing=\"0\"");
 	
-	char buf[20]; 
+	UT_String buf;
 	m_pie->write(" width=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Width(kword_1_unit));
-	m_pie->write((char *)buf);
+	UT_String_sprintf(buf,"%f",m_pDocument->m_docPageSize.Width(kword_1_unit));
+	m_pie->write(buf);
 	m_pie->write("\"");
 
 	m_pie->write(" height=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Height(kword_1_unit));
-	m_pie->write((char *)buf);
+	UT_String_sprintf(buf,"%f",m_pDocument->m_docPageSize.Height(kword_1_unit));
+	m_pie->write(buf);
 	m_pie->write("\"");
 	
 	m_pie->write(">\n");
 	
 	// PAPERBORDERS
 	m_pie->write("<PAPERBORDERS");
-	
-	m_pie->write(" left=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.MarginLeft(kword_1_unit));
-	m_pie->write((char *)buf);
-	m_pie->write("\"");
 
-	m_pie->write(" right=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.MarginRight(kword_1_unit));
-	m_pie->write((char *)buf);
-	m_pie->write("\"");
-
-	m_pie->write(" top=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.MarginTop(kword_1_unit));
-	m_pie->write((char *)buf);
-	m_pie->write("\"");
-
-	m_pie->write(" bottom=\"");
-	sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.MarginBottom(kword_1_unit));
-	m_pie->write((char *)buf);
-	m_pie->write("\"");
+	_writeMarginSize(api, "left");
+	_writeMarginSize(api, "right");
+	_writeMarginSize(api, "top");
+	_writeMarginSize(api, "bottom");
 
 	m_pie->write("/>\n");
 	
@@ -628,7 +613,7 @@ void s_KWord_1_Listener::_handleAttributes(PT_AttrPropIndex api)
 	m_pie->write("<ATTRIBUTES");
 	m_pie->write(" processing=\"0\"");
 	m_pie->write(" unit=\"");
-	m_pie->write(preferedUnitString(m_pDocument->m_docPageSize.getUnit()));
+	m_pie->write(UT_dimensionName(m_pDocument->m_docPageSize.getDims()));
 	m_pie->write("\"");
 	m_pie->write("/>\n");
 
@@ -644,16 +629,16 @@ void s_KWord_1_Listener::_handleDataItems(void)
 	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,(void**)&szMimeType)); k++)
 	{	  	  
 	  FILE *fp;
-	  char fname [1024]; // FIXME EVIL EVIL bad hardcoded buffer size
+	  UT_String fname;	  
 	  
 	  if (!UT_strcmp(szMimeType, "image/svg-xml"))
-	      sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
+	      UT_String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
 	  if (!UT_strcmp(szMimeType, "text/mathml"))
-	    sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
+	    UT_String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
 	  else // PNG Image
-	    sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
+	    UT_String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
 	  
-	  fp = fopen (fname, "wb+");
+	  fp = fopen (fname.c_str(), "wb+");
 	  
 	  if(!fp)
 	    continue;
@@ -662,7 +647,6 @@ void s_KWord_1_Listener::_handleDataItems(void)
 	  
 	  while (cnt < len)
 	    {
-	      xxx_UT_DEBUGMSG(("DOM: len: %d cnt: %d\n", len, cnt));
 	      cnt += fwrite (pByteBuf->getPointer(cnt), sizeof(UT_Byte), len-cnt, fp);
 	    }
 	  
@@ -1026,12 +1010,15 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 	// closeBlock is called
 	m_sFormats += "<FORMAT id=\"1\""; // id="1" means normal text
 	m_sFormats += " pos=\""; // current cursor position
-	char buf[100]; // FIXME: bad! hard-coded buffer size
-	sprintf(buf,"%ld", (long) pos);
+
+	UT_String buf;
+
+	UT_String_sprintf(buf,"%ld", (long) pos);
 	m_sFormats += buf;
 	m_sFormats += "\"";
 	m_sFormats += " len=\""; // length of span
-	sprintf(buf,"%ld", (long) len);
+
+	UT_String_sprintf(buf,"%ld", (long) len);
 	m_sFormats += buf;
 	m_sFormats += "\"";
 	m_sFormats += ">\n";
@@ -1046,20 +1033,16 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 
 		if (pAP->getProperty("color", szValue))
 		{
-		  char red[4], green[4], blue[4];
+		  UT_String red, green, blue;
 		  UT_RGBColor rgb;
 
 		  // convert from hex/decimal
 
 		  UT_parseColor(szValue, rgb);
 
-		  memset(red, 0, sizeof(red));
-		  memset(green, 0, sizeof(green));
-		  memset(blue, 0, sizeof(blue));
-		  
-		  sprintf(red, "%d", rgb.m_red);
-		  sprintf(green, "%d", rgb.m_grn);
-		  sprintf(blue, "%d", rgb.m_blu);
+		  UT_String_sprintf(red, "%d", rgb.m_red);
+		  UT_String_sprintf(green, "%d", rgb.m_grn);
+		  UT_String_sprintf(blue, "%d", rgb.m_blu);
 
 		  m_sFormats += "<COLOR red=\"";
 		  m_sFormats += red;
@@ -1099,10 +1082,10 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 		// <SIZE value="12"/> size in pt
 		if (pAP->getProperty("font-size", szValue))
 		{
-			char buf[100]; // FIXME: bad! hard-coded buffer size
+		  UT_String buf;
 
 			m_sFormats += "<SIZE value=\"";
-			sprintf(buf,"%d", (int) UT_convertToDimension(szValue, DIM_PT));
+			UT_String_sprintf(buf,"%d", (int) UT_convertToDimension(szValue, DIM_PT));
 			m_sFormats += buf;
 			m_sFormats += "\"/>\n";
 		}		
@@ -1218,15 +1201,15 @@ void s_KWord_1_Listener::_closeSpan(void)
 /*****************************************************************/
 /*****************************************************************/
 
-void s_KWord_1_Listener::_convertColor(char* szDest, const char* pszColor)
+void s_KWord_1_Listener::_convertColor(UT_String& szDest, const char* pszColor)
 {
 	/* FIXME: Convert this to deal with KWord's format for colors. */
-	strcpy(szDest, pszColor);
+	szDest = pszColor;
 }
 
-void s_KWord_1_Listener::_convertFontSize(char* szDest, const char* pszFontSize)
+void s_KWord_1_Listener::_convertFontSize(UT_String& szDest, const char* pszFontSize)
 {
-	strcpy (szDest, pszFontSize);
+	szDest = pszFontSize;
 }
 
 /*****************************************************************/
@@ -1306,10 +1289,7 @@ void s_KWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 					UT_UCSChar c = XAP_EncodingManager::get_instance()->try_UToNative(*pData);
 					if (c==0 || c>255)
 					{
-						char localBuf[20];
-						char * plocal = localBuf;
-						sprintf(localBuf,"&#x%x;",*pData++);
-						sBuf += plocal;
+					  sBuf += UT_String_sprintf("&#x%x;",*pData++);
 					}
 					else
 					{
