@@ -36,8 +36,8 @@
 #include "px_CR_Object.h"
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
-#include "xap_App.h"
 #include "xap_EncodingManager.h"
+#include "fd_Field.h"
 
 #include "ut_string_class.h"
 
@@ -102,7 +102,7 @@ ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
 								 UT_uint32 release)
 {
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
+  return 1;
 }
 
 #endif
@@ -161,6 +161,7 @@ IE_Exp_LaTeX::~IE_Exp_LaTeX()
 /*****************************************************************/
 typedef UT_UCSChar U16;
 static int wvConvertUnicodeToLaTeX(U16 char16,char*& out);
+static bool _convertLettersToSymbols(char c, char *& subst);
 
 #define BT_NORMAL		1
 #define BT_HEADING1		2
@@ -205,8 +206,8 @@ protected:
 	void				_openSpan(PT_AttrPropIndex api);
 	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
 	void				_handleDataItems(void);
-	void				_convertFontSize(char* szDest, const char* pszFontSize);
-	void				_convertColor(char* szDest, const char* pszColor);
+	void				_convertFontSize(UT_String& szDest, const char* pszFontSize);
+	void				_convertColor(UT_String& szDest, const char* pszColor);
 	
 	PD_Document *		m_pDocument;
 	IE_Exp_LaTeX *		m_pie;
@@ -214,7 +215,8 @@ protected:
 	bool				m_bInBlock;
 	bool				m_bInSpan;
 	const PP_AttrProp*	m_pAP_Span;
-	bool             m_bMultiCols;
+	bool                m_bMultiCols;
+	UT_uint16           m_iInSymbol;
 	JustificationTypes  m_eJustification;
 	bool				m_bLineHeight;
 	bool				m_bFirstSection;
@@ -250,6 +252,9 @@ void s_LaTeX_Listener::_closeBlock(void)
 	switch (m_iBlockType)
 	{
 	case BT_NORMAL:
+		if (m_bLineHeight)
+		  m_pie->write("\n\\end{spacing}");
+
 		switch (m_eJustification)
 		{
 		case JUSTIFIED:
@@ -265,9 +270,6 @@ void s_LaTeX_Listener::_closeBlock(void)
 			break;
 		}
 
-		if (m_bLineHeight)
-		  m_pie->write("\n\\end{spacing}");
-
 		m_pie->write("\n\n");
 		break;
 	case BT_HEADING1:
@@ -382,7 +384,7 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 				else if (height > 1.9 && height < 2.1)
 					m_pie->write("1.66}\n");
 				else if (m_bLineHeight) // glup.  TODO: calculate the spacing :)
-				    m_pie->write("1.0} % Sorry.  I know that you don't expected the 1.0... feel free to fix it! :)\n");
+				    m_pie->write("1.0} % Sorry.  I know that you don't expect the 1.0... feel free to fix it! :)\n");
 			}
 		}
 	}
@@ -397,12 +399,13 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 {
 	const PP_AttrProp* pAP = NULL;
+	bool bMustEmitMulticol = false;
+	const XML_Char* pszNbCols = NULL;
 
 	m_bMultiCols = false;
 
 	if (m_pDocument->getAttrProp(api, &pAP) && pAP)
 	{
-		const XML_Char* pszNbCols = NULL;
 		const XML_Char* pszPageMarginLeft = NULL;
 		const XML_Char* pszPageMarginRight = NULL;
 
@@ -413,9 +416,7 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 		if (pszNbCols != NULL && ((0 == UT_strcmp(pszNbCols, "2"))
 								  || (0 == UT_strcmp(pszNbCols, "3"))))
 		{
-			m_pie->write("\\begin{multicols}{");
-			m_pie->write(static_cast<const char *> (pszNbCols));
-			m_pie->write("}\n");
+			bMustEmitMulticol = true;
 			m_bMultiCols = true;
 		}
 		if (pszPageMarginLeft != NULL)
@@ -440,9 +441,16 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 		m_pie->write ("\n\n\\begin{document}\n");
 		m_bFirstSection = false;
 	}
+
+	if (bMustEmitMulticol)
+	{
+		m_pie->write("\\begin{multicols}{");
+		m_pie->write(static_cast<const char *> (pszNbCols));
+		m_pie->write("}\n");
+	}
 }
 
-void s_LaTeX_Listener::_convertColor(char* szDest, const char* pszColor)
+void s_LaTeX_Listener::_convertColor(UT_String& szDest, const char* pszColor)
 {
 	char colors[3][3];
 	for (int i=0;i<3;++i)
@@ -451,57 +459,56 @@ void s_LaTeX_Listener::_convertColor(char* szDest, const char* pszColor)
 		colors[i][2]=0;
 	}
 	setlocale (LC_NUMERIC, "C");
-	//FIXME: should use std_size_string here
-	sprintf (szDest, "%.3f,%.3f,%.3f",
-					strtol (&colors[0][0],NULL,16)/255.,
-					strtol (&colors[1][0],NULL,16)/255.,
-					strtol (&colors[2][0],NULL,16)/255.);
+	UT_String_sprintf (szDest, "%.3f,%.3f,%.3f",
+			   strtol (&colors[0][0],NULL,16)/255.,
+			   strtol (&colors[1][0],NULL,16)/255.,
+			   strtol (&colors[2][0],NULL,16)/255.);
 	setlocale (LC_NUMERIC, "");
 }
 
-void s_LaTeX_Listener::_convertFontSize(char* szDest, const char* pszFontSize)
+void s_LaTeX_Listener::_convertFontSize(UT_String& szDest, const char* pszFontSize)
 {
 	double fSizeInPoints = UT_convertToPoints(pszFontSize);
 
 	if (fSizeInPoints <= 6)
 	{
-		strcpy(szDest, "tiny");
+		szDest = "tiny";
 	}
 	else if (fSizeInPoints <= 8)
 	{
-		strcpy(szDest, "scriptsize");
+		szDest = "scriptsize";
 	}
 	else if (fSizeInPoints <= 10)
 	{
-		strcpy(szDest, "footnotesize");
+		szDest = "footnotesize";
 	}
 	else if (fSizeInPoints <= 11)
 	{
-		strcpy(szDest, "small");
+		szDest = "small";
 	}
 	else if (fSizeInPoints <= 12)
 	{
-		strcpy(szDest, "normalsize");
+		szDest = "normalsize";
 	}
 	else if (fSizeInPoints <= 14)
 	{
-		strcpy(szDest, "large");
+		szDest = "large";
 	}
 	else if (fSizeInPoints <= 17)
 	{
-		strcpy(szDest, "Large");
+		szDest = "Large";
 	}
 	else if (fSizeInPoints <= 20)
 	{
-		strcpy(szDest, "LARGE");
+		szDest = "LARGE";
 	}
 	else if (fSizeInPoints <= 25)
 	{
-		strcpy(szDest, "huge");
+		szDest = "huge";
 	}
 	else
 	{
-		strcpy(szDest, "Huge");
+		szDest = "Huge";
 	}
 }
 
@@ -578,11 +585,11 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		{
 			if (!UT_strcmp("superscript", szValue))
 			{
-				m_pie->write("$^{\\rm{}"); // TODO: Finish it
+				m_pie->write("$^{\\mathrm{");
 			}
 			else if (!UT_strcmp("subscript", szValue))
 			{
-				m_pie->write("$_{\\rm{}"); // TODO: Finish it
+				m_pie->write("$_{\\mathrm{");
 			}
 		}
 		
@@ -590,7 +597,7 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		pAP->getProperty("color", pszColor);
 		if (pszColor)
 		{
-			char szColor[18];
+		  UT_String szColor;
 			_convertColor(szColor,(char*)pszColor);
 			m_pie->write("\\textcolor[rgb]{");
 			m_pie->write(szColor);
@@ -601,7 +608,7 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		pAP->getProperty("bgcolor", pszBgColor);
 		if (pszBgColor)
 		{
-			char szColor[18];
+		  UT_String szColor;
 			_convertColor(szColor,(char*)pszBgColor);
 			m_pie->write("\\colorbox[rgb]{");
 			m_pie->write(szColor);
@@ -610,10 +617,10 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 
 		if (pAP->getProperty("font-size", szValue))
 		{
-			if (strcmp (szValue, DEFAULT_SIZE) != 0)
+			if (!UT_strcmp (DEFAULT_SIZE, szValue))
 			{
 				m_pie->write("{\\");
-				char szSize[16];
+				UT_String szSize;
 				_convertFontSize(szSize, (char*)szValue);
 				m_pie->write(szSize);
 				m_pie->write("{}");
@@ -622,6 +629,9 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		
 		if (pAP->getProperty("font-family", szValue))
 		{
+			if (!UT_strcmp ("Symbol", szValue) || 
+				!UT_strcmp("Standard Symbols", szValue))
+				m_iInSymbol++;
 			UT_DEBUGMSG (("Latex export: TODO: 'font-family' property\n"));
 		}
 
@@ -663,11 +673,11 @@ void s_LaTeX_Listener::_closeSpan(void)
 		{
 			if (!UT_strcmp("superscript", szValue))
 			{
-				m_pie->write("}$"); // TODO
+				m_pie->write("}}$");
 			}
 			else if (!UT_strcmp("subscript", szValue))
 			{
-				m_pie->write("}$"); // TODO
+				m_pie->write("}}$");
 			}
 		}
 
@@ -748,6 +758,13 @@ void s_LaTeX_Listener::_closeSpan(void)
 			m_pie->write("}");
 		}
 
+		if (pAP->getProperty("font-family", szValue))
+		{
+			if (!UT_strcmp ("Symbol", szValue) ||
+				!UT_strcmp("Standard Symbols", szValue))
+				m_iInSymbol--;
+		}
+
 		m_pAP_Span = NULL;
 	}
 
@@ -770,6 +787,19 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 	for (pData = data; (pData < data + length); /**/)
 	{
+		char* subst = "";
+
+		if (m_iInSymbol > 0)
+		{
+			if (_convertLettersToSymbols(*pData, subst))
+			{
+				while (*subst)
+					sBuf += *subst++;
+				pData++;
+				continue;
+			}
+		}
+
 		switch (*pData)
 		{
 		case '\\':
@@ -860,7 +890,6 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			
 			
 		default:
-			char* subst = "";
 			int translated =  wvConvertUnicodeToLaTeX(*pData,subst);
 			if (translated) 
 			{
@@ -884,6 +913,50 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 	m_pie->write(sBuf.c_str(),sBuf.size());
 }
 
+#define SUB(a,who) case a: subst = "\\(\\"who"\\)"; return true;
+#define SUBd(a,who) case a: subst = who; return true;
+static bool _convertLettersToSymbols(char c, char *& subst)
+{
+	switch (c)
+	{
+		// only-if-amssymb
+// 		SUB('\\', "therefore");
+		// todo: $ and ^ don't actually work right.
+
+		SUB('\"', "forall");    SUB('$', "exists");
+		SUB('\'', "ni");        SUB('@', "cong");
+		SUB('^', "perp");       SUB('`', "overline{\\ }");
+		SUB('a', "alpha");      SUBd('A', "A");
+		SUB('b', "beta"); 	    SUBd('B', "B");
+		SUB('c', "chi");  	    SUBd('C', "X");
+		SUB('d', "delta");	    SUB('D', "Delta");
+		SUB('e', "varepsilon"); SUBd('E', "E");
+		SUB('f', "phi");  	    SUB('F', "Phi");
+		SUB('g', "gamma");	    SUB('G', "Gamma");
+		SUB('h', "eta");	    SUBd('H', "H");
+		SUB('i', "iota"); 	    SUBd('I', "I"); 
+		SUB('j', "varphi");     SUB('J', "vartheta");
+		SUB('k', "kappa"); 	    SUBd('K', "K");
+		SUB('l', "lambda");	    SUB('L', "Lambda");
+		SUB('m', "mu");    	    SUBd('M', "M");
+		SUB('n', "nu");    	    SUBd('N', "N");
+		SUBd('o', "o");    	    SUBd('O', "O");
+		SUB('p', "pi");    	    SUB('P', "Pi");
+		SUB('q', "theta"); 	    SUB('Q', "Theta");
+		SUB('r', "rho");   	    SUBd('R', "P");
+		SUB('s', "sigma"); 	    SUB('S', "Sigma");
+		SUB('t', "tau");   	    SUBd('T', "T");
+		SUB('u', "upsilon");    SUBd('U', "Y");
+ 		SUB('v', "varpi");		SUB('V', "varsigma");
+		SUB('w', "omega");      SUB('W', "Omega");
+		SUB('x', "xi");         SUB('X', "Xi");
+		SUB('y', "psi");        SUB('Y', "Psi");
+		SUB('z', "zeta");       SUBd('Z', "Z");
+// TODO all those fun upper-ascii letters
+	default: return false;
+	}
+}
+
 s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 										 IE_Exp_LaTeX * pie)
 {
@@ -893,6 +966,7 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 	m_bInBlock = false;
 	m_bInSpan = false;
 	m_bFirstSection = true;
+	m_iInSymbol = 0;
 
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("%% This LaTeX file was created by AbiWord.                                         \n");
@@ -901,52 +975,14 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("\n");
 
-	if (XAP_App::s_szBuild_ID && XAP_App::s_szBuild_ID[0])
-	{
-		m_pie->write("%%         Build_ID          = ");
-		m_pie->write(XAP_App::s_szBuild_ID);
-		m_pie->write("\n");
-	}
-	if (XAP_App::s_szBuild_Version && XAP_App::s_szBuild_Version[0])
-	{
-		m_pie->write("%%         Build_Version     = ");
-		m_pie->write(XAP_App::s_szBuild_Version);
-		m_pie->write("\n");
-	}
-	if (XAP_App::s_szBuild_Options && XAP_App::s_szBuild_Options[0])
-	{
-		m_pie->write("%%         Build_Options     = ");
-		m_pie->write(XAP_App::s_szBuild_Options);
-		m_pie->write("\n");
-	}
-	if (XAP_App::s_szBuild_Target && XAP_App::s_szBuild_Target[0])
-	{
-		m_pie->write("%%         Build_Target      = ");
-		m_pie->write(XAP_App::s_szBuild_Target);
-		m_pie->write("\n");
-	}
-	if (XAP_App::s_szBuild_CompileTime && XAP_App::s_szBuild_CompileTime[0])
-	{
-		m_pie->write("%%         Build_CompileTime = ");
-		m_pie->write(XAP_App::s_szBuild_CompileTime);
-		m_pie->write("\n");
-	}
-	if (XAP_App::s_szBuild_CompileDate && XAP_App::s_szBuild_CompileDate[0])
-	{
-		m_pie->write("%%         Build_CompileDate = ");
-		m_pie->write(XAP_App::s_szBuild_CompileDate);
-		m_pie->write("\n");
-	}
-	
-	m_pie->write("\n");
-	
 	m_pie->write("\\documentclass[12pt]{article}\n");
 	m_pie->write("\\usepackage[T1]{fontenc}\n");
 	m_pie->write("\\usepackage{calc}\n");
+	m_pie->write("\\usepackage{hyperref}");
 	m_pie->write("\\usepackage{setspace}\n");
-	m_pie->write("\\usepackage{multicol}\t% TODO: I don't need this package if the document is a single column one.\n");
-	m_pie->write("\\usepackage[normalem]{ulem}\t% TODO: Package is only needed if you have underline/strikeout.\n");
-	m_pie->write("\\usepackage{color}\t% TODO: Package is only needed if you have color.\n");
+	m_pie->write("\\usepackage{multicol}\n");
+	m_pie->write("\\usepackage[normalem]{ulem}\n");
+	m_pie->write("\\usepackage{color}\n");
 	{
 	    const char* misc = XAP_EncodingManager::get_instance()->getTexPrologue();
 	    if (misc)
@@ -992,9 +1028,16 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
-#if 0			
-			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
 			PT_AttrPropIndex api = pcr->getIndexAP();
+			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
+
+			const PP_AttrProp * pAP = NULL;
+			bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+			const XML_Char* szValue = NULL;
+
+			fd_Field* field = NULL;
+
 			switch (pcro->getObjectType())
 			{
 			case PTO_Image:
@@ -1002,16 +1045,34 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				return true;
 
 			case PTO_Field:
+
+			  field = pcro->getField();
+			  m_pie->write(field->getValue());
+
 				// we do nothing with computed fields.
 				return true;
 
+			case PTO_Hyperlink:
+			  _closeSpan () ;
+				if(bHaveProp && pAP && pAP->getAttribute("xlink:href", szValue))
+				{
+					m_pie->write("\\href{");
+					m_pie->write(szValue);
+					m_pie->write("}{");
+				}
+				else
+				{
+					m_pie->write("}");
+				}
+				return true;
+
+			case PTO_Bookmark:
+			  return true;
+
 			default:
 				UT_ASSERT(0);
-				return false;
+				return true;
 			}
-#else
-			return true;
-#endif
 		}
 
 	case PX_ChangeRecord::PXT_InsertFmtMark:
