@@ -19,11 +19,14 @@
 
 #include <locale.h>
 
+#include "ut_string_class.h"
 #include "ut_string.h"
 #include "ut_types.h"
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
 #include "ut_debugmsg.h"
+#include "ut_map.h"
+#include "ut_pair.h"
 #include "pt_Types.h"
 #include "ie_exp_XSL-FO.h"
 #include "pd_Document.h"
@@ -33,7 +36,6 @@
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
 #include "pp_Property.h"
-#include "xap_App.h"
 #include "ap_Prefs.h"
 #include "pd_Style.h"
 #include "fd_Field.h"
@@ -43,9 +45,105 @@
 
 #include "ut_string_class.h"
 
+//////////////////////////////////////////////////
+// Some helper functions to prevent the use of static buffers.
+// may we should add these hacks to ut_string_class
+//////////////////////////////////////////////////
+
+UT_String&
+operator<< (UT_String& st, int i)
+{
+  return (st += UT_String_sprintf ("%d", i));
+}
+ 
+UT_UCS2String&
+operator<< (UT_UCS2String& st, int i)
+{
+	UT_UCSChar tmp2[16];
+	UT_UCS_strcpy_char(tmp2, UT_String_sprintf("%d", i).c_str());
+	return (st += tmp2);
+}
+
+UT_String&
+operator<< (UT_String& st, const UT_String& st2)
+{
+	st += st2;
+	return st;
+}
+
+UT_UCS2String&
+operator<< (UT_UCS2String& st, const UT_UCS2String& st2)
+{
+	st += st2;
+	return st;
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
+// A little class to help export lists
+
+class ListHelper
+{
+public:
+	explicit ListHelper(UT_uint32 id = 0)
+		: m_pan(0),
+		  m_iNextNb(0),
+		  m_iInc(0)
+	{
+		setIdList(id);
+	}
+
+	void setIdList(UT_uint32 id)
+	{
+		UT_Map::Iterator it(myLists.find(reinterpret_cast<UT_Map::key_t> (id)));
+
+		if (it.is_valid())
+		{
+			m_pan = static_cast<const fl_AutoNum*> (static_cast<const UT_Pair*> (it.value())->second());
+			UT_ASSERT(m_pan);
+			m_iNextNb = m_pan->getStartValue32();
+
+			if (m_pan->getType() >= BULLETED_LIST)
+				m_iInc = 0;
+			else
+				m_iInc = 1;
+		}
+	}
+
+	UT_String getNextLabel()
+	{
+		UT_ASSERT(m_pan);
+		UT_uint32 next_nb = m_iNextNb;
+		m_iNextNb += m_iInc;
+		// TODO: I'm laking pre and post text
+		return int2label(next_nb);
+	}
+	
+    static void addList(fl_AutoNum* pAutoNum) 
+	{
+		myLists.insert(reinterpret_cast<void*> (pAutoNum->getID()), pAutoNum);
+	}
+
+private:
+	UT_String int2label(UT_uint32 i)
+	{
+		UT_String retval;
+		retval << i;
+		return retval;
+	}
+
+	const fl_AutoNum* m_pan;
+	UT_uint32 m_iNextNb;
+	UT_uint32 m_iInc;
+    static UT_Map myLists;
+};
+
+UT_Map  ListHelper::myLists;
+
+/************************************************************/
+/************************************************************/
+
 class s_XSL_FO_Listener : public PL_Listener
 {
 public:
@@ -72,18 +170,20 @@ public:
 															PL_StruxFmtHandle sfhNew));
 
 	virtual bool		signal(UT_uint32 iSignal);
-protected:
 
+protected:
 	void                _handlePageSize(PT_AttrPropIndex api);
-	void				_handleDataItems(void);
+	void				_handleDataItems();
+	void				_handleLists();
+	void				_handleField(PT_AttrPropIndex api);
 	void                _outputData(const UT_UCSChar * data, UT_uint32 length);
 
-	void				_convertFontSize(char* szDest, const char* szFontSize);
-	void                _convertColor(char* szDest, const char* pszColor);
+	void				_convertFontSize(UT_String& szDest, const char* szFontSize);
+	void                _convertColor(UT_String& szDest, const char* pszColor);
 
-	void				_closeSection(void);
-	void				_closeBlock(void);
-	void				_closeSpan(void);
+	void				_closeSection();
+	void				_closeBlock();
+	void				_closeSpan();
 	void				_openBlock(PT_AttrPropIndex api);
 	void				_openSection(PT_AttrPropIndex api);
 	void				_openSpan(PT_AttrPropIndex api);
@@ -91,12 +191,12 @@ protected:
 private:
 	PD_Document *		m_pDocument;
 	IE_Exp_XSL_FO *	    m_pie;
+	ListHelper			m_List;
 
 	bool				m_bInSection;
 	bool				m_bInBlock;
 	bool				m_bInSpan;
 	bool				m_bFirstWrite;
-
 	int                 m_iImgCnt;
 };
 
@@ -119,13 +219,9 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
 {
 
 	if (!m_sniffer)
-	{
 		m_sniffer = new IE_Exp_XSL_FO_Sniffer ();
-	}
 	else
-	{
 		m_sniffer->ref();
-	}
 
 	mi->name = "XSL-FO Exporter";
 	mi->desc = "Export XSL-FO Documents";
@@ -150,9 +246,7 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
 
 	IE_Exp::unregisterExporter (m_sniffer);
 	if (!m_sniffer->unref())
-	{
 		m_sniffer = 0;
-	}
 
 	return 1;
 }
@@ -161,7 +255,7 @@ ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
 								 UT_uint32 release)
 {
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
+	return 1;
 }
 
 #endif
@@ -170,9 +264,10 @@ int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor,
 /*****************************************************************/
 
 IE_Exp_XSL_FO::IE_Exp_XSL_FO(PD_Document * pDocument)
-	: IE_Exp(pDocument), m_pListener(0)
+	: IE_Exp(pDocument),
+	  m_pListener(0),
+	  m_error(UT_OK)
 {
-	m_error = UT_OK;
 }
 
 IE_Exp_XSL_FO::~IE_Exp_XSL_FO()
@@ -184,7 +279,7 @@ IE_Exp_XSL_FO::~IE_Exp_XSL_FO()
 
 bool IE_Exp_XSL_FO_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".fo"));
+	return (!UT_stricmp(szSuffix, ".fo"));
 }
 
 UT_Error IE_Exp_XSL_FO_Sniffer::constructExporter(PD_Document * pDocument,
@@ -208,7 +303,7 @@ bool IE_Exp_XSL_FO_Sniffer::getDlgLabels(const char ** pszDesc,
 /*****************************************************************/
 /*****************************************************************/
 
-UT_Error IE_Exp_XSL_FO::_writeDocument(void)
+UT_Error IE_Exp_XSL_FO::_writeDocument()
 {
 	m_pListener = new s_XSL_FO_Listener(getDoc(),this);
 	if (!m_pListener)
@@ -227,10 +322,14 @@ UT_Error IE_Exp_XSL_FO::_writeDocument(void)
 
 s_XSL_FO_Listener::s_XSL_FO_Listener(PD_Document * pDocument,
 									 IE_Exp_XSL_FO * pie)
-	: m_pDocument (pDocument), m_pie (pie), 
-	m_bInSection(false), m_bInBlock(false), 
-	m_bInSpan(false), m_bFirstWrite(true),
-	m_iImgCnt(0)
+	: m_pDocument(pDocument),
+	  m_pie (pie),
+	  m_List(),
+	  m_bInSection(false),
+	  m_bInBlock(false),
+	  m_bInSpan(false),
+	  m_bFirstWrite(true),
+	  m_iImgCnt(0)
 {
 	// Be nice to XML apps.  See the notes in _outputData() for more 
 	// details on the charset used in our documents.  By not declaring 
@@ -252,6 +351,8 @@ s_XSL_FO_Listener::s_XSL_FO_Listener(PD_Document * pDocument,
 	m_pie->write("<!-- This document was created by AbiWord -->\n");
 	m_pie->write("<!-- AbiWord is a free, Open Source word processor. -->\n");
 	m_pie->write("<!-- You may obtain more information about AbiWord at www.abisource.com -->\n\n");
+
+	_handleLists();
 }
 
 s_XSL_FO_Listener::~s_XSL_FO_Listener()
@@ -264,6 +365,41 @@ s_XSL_FO_Listener::~s_XSL_FO_Listener()
 	m_pie->write ("</fo:root>\n");
 }
 
+void s_XSL_FO_Listener::_handleLists()
+{
+	fl_AutoNum* pAutoNum;
+
+	for (UT_uint32 k = 0; m_pDocument->enumLists(k, &pAutoNum); ++k)
+	{
+		if (pAutoNum->isEmpty() == true)
+			continue;
+
+		ListHelper::addList(pAutoNum);
+	}
+}
+
+void s_XSL_FO_Listener::_handleField(PT_AttrPropIndex api)
+{
+	const PP_AttrProp* pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api, &pAP);
+	
+	if (bHaveProp && pAP)
+	{
+		const XML_Char* szValue;
+		if (pAP->getAttribute("type", szValue))
+		{
+			if (szValue[0] == 'l' && strcmp((const char*) szValue, "list_label") == 0)
+			{
+				m_pie->write("<fo:list-item-label end-indent=\"label-end()\">\n"
+							 "  <fo:block>\n");
+				m_pie->write(m_List.getNextLabel().c_str());
+				m_pie->write("  </fo:block>\n"
+							 "</fo:list-item-label>\n");
+			}
+		}
+	}
+}
+
 bool s_XSL_FO_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 								 const PX_ChangeRecord * pcr)
 {
@@ -293,26 +429,26 @@ bool s_XSL_FO_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
 			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
-			PT_AttrPropIndex api = pcr->getIndexAP();
 
 			switch (pcro->getObjectType())
 			{
 			case PTO_Image:
 			{
-				char buf[16];
-				sprintf(buf, "%d.png", m_iImgCnt++);
-				m_pie->write("<fo:external-graphic src=\"");
-				m_pie->write(m_pie->getFileName());
-				m_pie->write(buf);
-				m_pie->write("\"/>\n");
+			        m_pie->write(UT_String_sprintf("<fo:external-graphic src=\"%s-%d.png\"/>\n", m_pie->getFileName(), m_iImgCnt++));
 				return true;
 			}
 
 			case PTO_Field:
 			{
+				_handleField(pcr->getIndexAP());
 				return true;
 			}
 
+				// todo: support these
+			case PTO_Hyperlink:
+			case PTO_Bookmark:
+			  return true;
+
 			default:
 			{
 				UT_ASSERT(0);
@@ -376,6 +512,15 @@ bool s_XSL_FO_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	
 	case PTX_SectionHdrFtr:
 	{
+		_closeSpan();
+		_closeBlock();
+		
+//		<fo:static-content flow-name="xsl-region-before">
+//		<fo:block>
+//		</fo:block>
+//		</fo:static-content> 
+
+		
 		// TODO???
 		return true;
 	}
@@ -425,20 +570,20 @@ bool s_XSL_FO_Listener::insertStrux(PL_StruxFmtHandle /*sfh*/,
 /*****************************************************************/
 /*****************************************************************/
 
-static const char *
-docUnitToString(fp_PageSize::Unit docUnit)
+static UT_String
+purgeSpaces(const char* st)
 {
-	if(docUnit == fp_PageSize::cm)
-		return "cm";
-	else if(docUnit == fp_PageSize::mm)
-		return "mm";
-	else if(docUnit == fp_PageSize::inch)
-		return "in";
-	else
+	UT_String retval;
+
+	while (*st != '\0')
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return "";
+		if (*st != ' ')
+			retval += *st++;
+		else
+			++st;
 	}
+
+	return retval;
 }
 
 void s_XSL_FO_Listener::_handlePageSize(PT_AttrPropIndex api)
@@ -486,21 +631,12 @@ void s_XSL_FO_Listener::_handlePageSize(PT_AttrPropIndex api)
 		m_pie->write(szValue);
 		m_pie->write("\"");
 		
-		fp_PageSize::Unit  docUnit = m_pDocument->m_docPageSize.getUnit(); 
+		UT_Dimension docUnit = m_pDocument->m_docPageSize.getDims(); 
 		char buf[20];
 
-		m_pie->write( " page-width=\"");
-		sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Width(docUnit));
-		m_pie->write((char *)buf);
-		m_pie->write(docUnitToString(docUnit));
-		m_pie->write("\"");
+		m_pie->write( UT_String_sprintf(" page-width=\"%f%s\"", m_pDocument->m_docPageSize.Width(docUnit), UT_dimensionName(docUnit)) );
 
-		m_pie->write(" page-height=\"");
-		sprintf((char *) buf,"%f",m_pDocument->m_docPageSize.Height(docUnit));
-		m_pie->write((char *)buf);
-		m_pie->write(docUnitToString(docUnit));
-		m_pie->write("\"");
-		
+		m_pie->write(UT_String_sprintf(" page-height=\"%f%s\"", m_pDocument->m_docPageSize.Height(docUnit), UT_dimensionName(docUnit)) );
 	}
 	// page-width, page-height
 
@@ -517,7 +653,7 @@ void s_XSL_FO_Listener::_handlePageSize(PT_AttrPropIndex api)
 	return;
 }
 
-void s_XSL_FO_Listener::_handleDataItems(void)
+void s_XSL_FO_Listener::_handleDataItems()
 {
 	const char * szName;
    	const char * szMimeType;
@@ -526,16 +662,16 @@ void s_XSL_FO_Listener::_handleDataItems(void)
 	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,(void**)&szMimeType)); k++)
 	{	  	  
 	  FILE *fp;
-	  char fname [1024]; // FIXME EVIL EVIL bad hardcoded buffer size
+	  UT_String fname;
 	  
 	  if (!UT_strcmp(szMimeType, "image/svg-xml"))
-	      sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
+	    UT_String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
 	  if (!UT_strcmp(szMimeType, "text/mathml"))
-	    sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
+	    UT_String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
 	  else // PNG Image
-	    sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
+	    UT_String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
 	  
-	  fp = fopen (fname, "wb+");
+	  fp = fopen (fname.c_str(), "wb+");
 	  
 	  if(!fp)
 	    continue;
@@ -564,190 +700,102 @@ void s_XSL_FO_Listener::_openSection(PT_AttrPropIndex api)
 	m_bInSection = true;
 
 	m_pie->write("<fo:page-sequence master-name=\"first\">\n");
-	m_pie->write("<fo:flow flow-name=\"section\">\n");
+	m_pie->write("<fo:flow flow-name=\"xsl-region-body\">\n");
 }
 
-#define USED() do {m_pie->write(" "); if(!used) used = true;} while (0)
+#define PROPERTY(x) \
+	if (pAP->getProperty(x, szValue)) \
+		content_st << " "x"=\"" << (const char*) szValue << "\"";
 
 void s_XSL_FO_Listener::_openBlock(PT_AttrPropIndex api)
 {
 	if (!m_bInSection)
-	{
 		return;
-	}
 
-	const PP_AttrProp * pAP = NULL;
-	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+	UT_String start_st;
+	UT_String content_st;
+	const PP_AttrProp* pAP = 0;
+	bool bHaveProp = m_pDocument->getAttrProp(api, &pAP);
 
 	m_bInBlock = true;
-	m_pie->write("<fo:block");
-
-	// keep track of whether we have we written anything
-	bool used = false;
 
 	// query and output properties
 	// todo - validate these and make sure they all make sense
+	const XML_Char* szValue;
+
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
-
 		if (pAP->getProperty("bgcolor", szValue))
 		{
-			USED();
-			m_pie->write("background-color=\"#");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " background-color=\"";
+
+			if (*szValue >= '0' && *szValue <= '9')
+				content_st += '#';
+
+			content_st << (const char*) szValue << "\"";
 		}
 
 		if (pAP->getProperty("color", szValue))
 		{
-			USED();
-			m_pie->write("color=\"#");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " color=\"";
+
+			if (*szValue >= '0' && *szValue <= '9')
+				content_st += '#';
+
+			content_st << (const char*) szValue << "\"";
 		}
 
 		if (pAP->getProperty("lang", szValue))
-		{
-			USED();
-			m_pie->write("language=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-		
+			content_st << " language=\"" << (const char*) szValue << "\"";
+
 		if (pAP->getProperty("font-size", szValue))
-		{
-			USED();
-			m_pie->write("font-size=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}		
+			content_st << " font-size=\"" << purgeSpaces((const char *)szValue).c_str() << "\"";
 
-		if (pAP->getProperty("font-family", szValue))
-		{
-			USED();
-			m_pie->write("font-family=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-weight", szValue))
-		{
-			USED();
-			m_pie->write("font-weight=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-style", szValue))
-		{
-			USED();
-			m_pie->write("font-style=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-stretch", szValue))
-		{
-			USED();
-			m_pie->write("font-stretch=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("keep-together", szValue))
-		{
-			USED();
-			m_pie->write("keep-together=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("keep-with-next", szValue))
-		{
-			USED();
-			m_pie->write("keep-with-next=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("line-height", szValue))
-		{
-			USED();
-			m_pie->write("line-height=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("margin-bottom", szValue))
-		{
-			USED();
-			m_pie->write("margin-bottom=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("margin-top", szValue))
-		{
-			USED();
-			m_pie->write("margin-top=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("margin-left", szValue))
-		{
-			USED();
-			m_pie->write("margin-left=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("margin-right", szValue))
-		{
-			USED();
-			m_pie->write("margin-right=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("text-align", szValue))
-		{
-			USED();
-			m_pie->write("text-align=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("widows", szValue))
-		{
-			USED();
-			m_pie->write("widows=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
+		PROPERTY("font-family");
+		PROPERTY("font-weight");
+		PROPERTY("font-style");
+		PROPERTY("font-stretch");
+		PROPERTY("keep-together");
+		PROPERTY("keep-with-next");
+		PROPERTY("line-height");
+		PROPERTY("margin-bottom");
+		PROPERTY("margin-top");
+		PROPERTY("margin-left");
+		PROPERTY("margin-right");
+		PROPERTY("text-align");
+		PROPERTY("widows");
 	}
 
-	m_pie->write(">\n");
+	if (pAP && pAP->getAttribute("listid", szValue))
+	{
+		start_st = "<fo:list-block";
+		UT_uint32 id = (UT_uint32) atoi((const char*)szValue);
+		m_List.setIdList(id);
+	}
+
+	if (!content_st.empty())
+	{
+		if (start_st.empty())
+			start_st = "<fo:block";
+
+		start_st += content_st;
+		start_st += '>';
+		m_pie->write(start_st.c_str());
+	}
 }
 
 void s_XSL_FO_Listener::_openSpan(PT_AttrPropIndex api)
 {
 	if (!m_bInBlock)
-	{
 		return;
-	}
 
+	const PP_AttrProp* pAP = 0;
+	bool bHaveProp = m_pDocument->getAttrProp(api, &pAP);
+	UT_String start_st("<fo:inline");
+	UT_String content_st;
+	
 	m_bInSpan = true;
 
-	const PP_AttrProp * pAP = NULL;
-	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-
-	// keep track of whether we've written out anything
-	bool used = false;
-
-	m_pie->write("<fo:inline");
-
 	// query and output properties
 	if (bHaveProp && pAP)
 	{
@@ -755,151 +803,103 @@ void s_XSL_FO_Listener::_openSpan(PT_AttrPropIndex api)
 
 		if (pAP->getProperty("bgcolor", szValue))
 		{
-			USED();
-			m_pie->write("background-color=\"#");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " background-color=\"";
+
+			if (*szValue >= '0' && *szValue <= '9')
+				content_st += "#";
+
+			content_st += (const char *)szValue;
+			content_st += "\"";
 		}
 
 		if (pAP->getProperty("color", szValue))
 		{
-			USED();
-			m_pie->write("color=\"#");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " color=\"";
+
+			if (*szValue >= '0' && *szValue <= '9')
+				content_st += "#";
+
+			content_st += (const char *)szValue;
+			content_st += "\"";
 		}
 
 		if (pAP->getProperty("lang", szValue))
 		{
-			USED();
-			m_pie->write("language=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " language=\"";
+			content_st += (const char *)szValue;
+			content_st += "\"";
 		}
 		
 		if (pAP->getProperty("font-size", szValue))
 		{
-			USED();
-			m_pie->write("font-size=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
+			content_st += " font-size=\"";
+			content_st += purgeSpaces((const char *)szValue).c_str();
+			content_st += "\"";
 		}		
 
-		if (pAP->getProperty("font-family", szValue))
-		{
-			USED();
-			m_pie->write("font-family=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-weight", szValue))
-		{
-			USED();
-			m_pie->write("font-weight=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-style", szValue))
-		{
-			USED();
-			m_pie->write("font-style=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("font-stretch", szValue))
-		{
-			USED();
-			m_pie->write("font-stretch=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("keep-together", szValue))
-		{
-			USED();
-			m_pie->write("keep-together=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("keep-with-next", szValue))
-		{
-			USED();
-			m_pie->write("keep-with-next=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
-		if (pAP->getProperty("text-decoration", szValue))
-		{
-			USED();
-			m_pie->write("text-decoration=\"");
-			m_pie->write((const char *)szValue);
-			m_pie->write("\"");
-		}
-
+		PROPERTY("font-family");
+		PROPERTY("font-weight");
+		PROPERTY("font-style");
+		PROPERTY("font-stretch");
+		PROPERTY("keep-together");
+		PROPERTY("keep-with-next");
+		PROPERTY("text-decoration");
 	}
 
-	m_pie->write(">");
+	if (!content_st.empty())
+	{
+		start_st += content_st;
+		start_st += '>';
+		m_pie->write(start_st.c_str());
+	}
 }
 
-#undef USED
+#undef PROPERTY
 
-void s_XSL_FO_Listener::_closeBlock(void)
+void s_XSL_FO_Listener::_closeBlock()
 {
 	if (!m_bInBlock)
-	{
 		return;
-	}
 
 	m_bInBlock = false;
 	m_pie->write("\n</fo:block>\n");
 }
 
-void s_XSL_FO_Listener::_closeSection(void)
+void s_XSL_FO_Listener::_closeSection()
 {
 	if (!m_bInSection)
-	{
 		return;
-	}
 	
 	m_bInSection = false;
-
 	m_pie->write("</fo:flow>\n");
 	m_pie->write("</fo:page-sequence>\n");
 }
 
-void s_XSL_FO_Listener::_closeSpan(void)
+void s_XSL_FO_Listener::_closeSpan()
 {
 	if (!m_bInSpan)
-	{
 		return;
-	}
 
 	m_bInSpan = false;
-
 	m_pie->write("</fo:inline>");
 }
 
 /*****************************************************************/
 /*****************************************************************/
 
-void s_XSL_FO_Listener::_convertColor(char* szDest, const char* pszColor)
+void s_XSL_FO_Listener::_convertColor(UT_String& szDest, const char* pszColor)
 {
 	/*
 	 * TODO we might want to be a little more careful about this.
 	 * The proper XSL-FO color is #rrggbb, which is basically the same
 	 * as what we use this.
 	 */
-	strcpy(szDest, pszColor);
+	szDest = pszColor;
 }
 
-void s_XSL_FO_Listener::_convertFontSize(char* szDest, const char* pszFontSize)
+void s_XSL_FO_Listener::_convertFontSize(UT_String& szDest, const char* pszFontSize)
 {
-	strcpy (szDest, pszFontSize);
+	szDest = pszFontSize;
 }
 
 /*****************************************************************/
@@ -979,10 +979,7 @@ void s_XSL_FO_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 					UT_UCSChar c = XAP_EncodingManager::get_instance()->try_UToNative(*pData);
 					if (c==0 || c>255)
 					{
-						char localBuf[20];
-						char * plocal = localBuf;
-						sprintf(localBuf,"&#x%x;",*pData++);
-						sBuf += plocal;
+					  sBuf += UT_String_sprintf("&#x%x;",*pData++);
 					}
 					else
 					{
@@ -1001,3 +998,15 @@ void s_XSL_FO_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 	m_pie->write(sBuf.c_str(), sBuf.size());
 }
+
+
+
+
+
+
+
+
+
+
+
+
