@@ -36,12 +36,17 @@
 #include "fl_AutoNum.h"
 #include "fl_BlockLayout.h"
 
+#ifdef BIDI_ENABLED
+#include "fribidi.h"
+#endif
+
 class IE_Imp_RTF;
 class RTF_msword97_list;
 
 // Font table entry
-struct RTFFontTableItem
+struct ABI_EXPORT RTFFontTableItem
 {
+ public:
 	enum FontFamilyEnum { ffNone, ffRoman, ffSwiss, ffModern, ffScript, ffDecorative, ffTechnical, ffBiDirectional};
 	enum FontPitch { fpDefault, fpFixed, fpVariable};
 
@@ -60,7 +65,7 @@ struct RTFFontTableItem
 };
 
 // Set true if Character properties have been changed in list structure.
-class RTFProps_CharProps
+class ABI_EXPORT RTFProps_CharProps
 {
 public:
 	RTFProps_CharProps(void);
@@ -86,9 +91,9 @@ public:
 	UT_sint32  m_styleNumber ; //index into the style table
 	UT_uint32  m_listTag; // tag for lists to hang off
 	const char * m_szLang;
-};                  
+};
 
-class RTFProps_bCharProps
+class ABI_EXPORT RTFProps_bCharProps
 {
 public:
 	RTFProps_bCharProps(void);
@@ -115,7 +120,7 @@ public:
 	bool bm_listTag; // tag for lists to hanfg off
 };
 
-struct _rtfListTable
+struct ABI_EXPORT _rtfListTable
 {      
 	UT_uint32 start_value;
 	UT_uint32 level;
@@ -144,9 +149,8 @@ struct _rtfListTable
 };
 
 
-
 // Paragraph properties
-struct RTFProps_ParaProps
+struct ABI_EXPORT RTFProps_ParaProps
 {
 	enum ParaJustification { pjLeft, pjCentre, pjRight, pjFull};
 
@@ -179,10 +183,13 @@ struct RTFProps_ParaProps
 	UT_uint32       m_iOverideLevel;     // 0's index to the level
 	_rtfListTable   m_rtfListTable; 
 	UT_sint32  m_styleNumber ; //index into the style table
+#ifdef BIDI_ENABLED
+	FriBidiCharType m_dom_dir;
+#endif
 };                  
 
 // These are set true if changed in list definitions.
-class RTFProps_bParaProps
+class ABI_EXPORT RTFProps_bParaProps
 {
 public:
     RTFProps_bParaProps(void);
@@ -202,7 +209,10 @@ public:
 	bool        bm_curTabType;        // Current Tab type
 	bool        bm_curTabLeader;       // Current Tab Leader
 	bool        bm_rtfListTable; 
-};                  
+#ifdef BIDI_ENABLED
+	bool bm_dom_dir;
+#endif
+};
 
 
 //typedef struct sect_prop
@@ -216,7 +226,7 @@ public:
 
 
 // Lists Level class
-class RTF_msword97_level
+class ABI_EXPORT RTF_msword97_level
 {
 public:
     RTF_msword97_level(	RTF_msword97_list * pmsword97List, UT_uint32 level);
@@ -231,35 +241,40 @@ public:
 								 const char ** szAlign, 
 								 const char ** szIndent, 
 								 const char ** szListStyle);
-	UT_sint32 levelStartAt;
-	UT_uint32 AbiLevelID;
-	UT_uint32 RTFListType;
-	UT_String levelNumber;
-	UT_String levelText;
-    RTFProps_ParaProps * pParaProps;
-	RTFProps_CharProps *  pCharProps;
-    RTFProps_bParaProps * pbParaProps;
-	RTFProps_bCharProps *  pbCharProps;
+	bool ParseLevelText(const UT_String & szLevelText,const UT_String & szLevelNumbers, UT_uint32 iLevel);
+	UT_sint32 m_levelStartAt;
+	UT_uint32 m_AbiLevelID;
+	static UT_uint32 m_sLastAssignedLevelID;
+	static UT_uint32 m_sPreviousLevel;
+	UT_uint32 m_RTFListType;
+	UT_String m_listDelim;
+	char      m_cLevelFollow;
+	bool m_bStartNewList;
+	bool m_bRestart;
+    RTFProps_ParaProps * m_pParaProps;
+	RTFProps_CharProps *  m_pCharProps;
+    RTFProps_bParaProps * m_pbParaProps;
+	RTFProps_bCharProps *  m_pbCharProps;
 private:
-	UT_uint32 localLevel;
+	UT_uint32 m_localLevel;
 	RTF_msword97_list * m_pMSWord97_list ;
 };
 
 // List Header Class
-class RTF_msword97_list
+class ABI_EXPORT RTF_msword97_list
 {
 public:
 	RTF_msword97_list(	IE_Imp_RTF * pie_rtf);
     ~RTF_msword97_list();
-	UT_uint32 RTF_listID;
-	UT_uint32 RTF_listTemplateID;
-	RTF_msword97_level * RTF_level[9];
+	UT_uint32 m_RTF_listID;
+	UT_uint32 m_RTF_listTemplateID;
+	RTF_msword97_level * m_RTF_level[9];
 private:
 	IE_Imp_RTF * m_pie_rtf;
 };
 
 // List Header Overide
-class RTF_msword97_listOveride
+class ABI_EXPORT RTF_msword97_listOveride
 {
 public:
 	RTF_msword97_listOveride(	IE_Imp_RTF * pie_rtf);
@@ -275,12 +290,12 @@ public:
 								 const char ** szIndent, 
 								 const char ** szListStyle,
 								 UT_uint32 iLevel);
-	UT_uint32 RTF_listID;
-	UT_uint32 OverideCount;
-	RTFProps_ParaProps * pParaProps;
-	RTFProps_CharProps * pCharProps;
-	RTFProps_bParaProps * pbParaProps;
-	RTFProps_bCharProps * pbCharProps;
+	UT_uint32 m_RTF_listID;
+	UT_uint32 m_OverideCount;
+	RTFProps_ParaProps * m_pParaProps;
+	RTFProps_CharProps * m_pCharProps;
+	RTFProps_bParaProps * m_pbParaProps;
+	RTFProps_bCharProps * m_pbCharProps;
 	bool setList(void);
 	bool isTab(UT_uint32 iLevel);
 	UT_Vector * getTabStopVect(UT_uint32 iLevel);
@@ -322,18 +337,43 @@ private:
 	RTF_msword97_list* m_pList;
 };
 
-// Section properties
-struct RTFProps_SectionProps
+struct ABI_EXPORT RTFProps_ImageProps
 {
-	enum SectionBreak {sbkNone, sbkColumn, sbkEven, sbkOdd, sbkPage};
-	enum PageNumber {pgDecimal, pgURoman, pgLRoman, pgULtr, pgLLtr};
+	enum IPSizeType { ipstNone, ipstGoal, ipstScale };
+	
+	RTFProps_ImageProps ();
+	IPSizeType sizeType;
+	UT_uint32 wGoal;
+	UT_uint32 hGoal;
+	UT_uint16 scaleX;
+	UT_uint16 scaleY;
+	UT_uint32 width;
+	UT_uint32 height;
+};
+
+// Section properties
+struct ABI_EXPORT RTFProps_SectionProps
+{
+	enum ESectionBreak {sbkNone, sbkColumn, sbkEven, sbkOdd, sbkPage};
+	enum EPageNumber {pgDecimal, pgURoman, pgLRoman, pgULtr, pgLLtr};
 
 	RTFProps_SectionProps();
 	RTFProps_SectionProps& operator=(const RTFProps_SectionProps&);
 
 	UT_uint32		m_numCols;
-	SectionBreak	m_breakType;
-	PageNumber		m_pageNumFormat;
+	bool m_bColumnLine;
+	ESectionBreak	m_breakType;
+	EPageNumber		m_pageNumFormat;
+	UT_sint32       m_leftMargTwips;     
+	UT_sint32       m_rightMargTwips;     
+	UT_sint32       m_topMargTwips;     
+	UT_sint32       m_bottomMargTwips;     
+	UT_sint32       m_headerYTwips;     
+	UT_sint32       m_footerYTwips;     
+	UT_sint32       m_colSpaceTwips;     
+#ifdef BIDI_ENABLED
+	FriBidiCharType m_dir;
+#endif
 };
 
 
@@ -343,9 +383,17 @@ struct RTFProps_SectionProps
   before any section data begins.
   \todo add right and left headers and footer. Not yet supported by AbiWord
  */
-struct RTFHdrFtr
+struct ABI_EXPORT RTFHdrFtr
 {
-	enum HdrFtrType {hftNone, hftHeader, hftFooter };
+	enum HdrFtrType {hftNone,
+					 hftHeader, 
+					 hftHeaderEven, 
+					 hftHeaderFirst, 
+					 hftHeaderLast,  
+					 hftFooter, 
+					 hftFooterEven, 
+					 hftFooterFirst, 
+					 hftFooterLast };
 
 	RTFHdrFtr () : m_type(hftNone), m_id(0), m_buf(1024) {}
 	
@@ -355,7 +403,7 @@ struct RTFHdrFtr
 };	
 
 // RTFStateStore
-struct RTFStateStore
+struct ABI_EXPORT RTFStateStore
 {
 	RTFStateStore();
 	
@@ -394,7 +442,7 @@ typedef struct doc_prop
 
 // The importer/reader for Rich Text Format files
 
-class IE_Imp_RTF_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_RTF_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 
@@ -402,9 +450,9 @@ public:
 	IE_Imp_RTF_Sniffer() {}
 	virtual ~IE_Imp_RTF_Sniffer() {}
 
-	virtual bool recognizeContents (const char * szBuf, 
+	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
 									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -413,7 +461,7 @@ public:
 
 };
 
-class IE_Imp_RTF : public IE_Imp
+class ABI_EXPORT IE_Imp_RTF : public IE_Imp
 {
 public:
 	IE_Imp_RTF(PD_Document * pDocument);
@@ -443,6 +491,11 @@ private:
 		picGIF
 	};
 
+	typedef enum {
+		RBT_START = 0,
+		RBT_END
+	} RTFBookmarkType;
+
 	bool AddChar(UT_UCSChar ch);
 	bool FlushStoredChars(bool forceInsertPara = false);
 	bool StartNewPara();
@@ -463,12 +516,15 @@ private:
 	bool HandlePicture();
 	bool HandleObject();
 	bool HandleField();
+	bool HandleHyperlink();
 	bool HandleStyleDefinition(void);
 	bool HandleHeaderFooter(RTFHdrFtr::HdrFtrType hftype, UT_uint32 & headerID);
 	bool SkipCurrentGroup(bool bConsumeLastBrace = false);
 	bool StuffCurrentGroup(UT_ByteBuf & buf);
-	bool CanHandlePictFormat(PictFormat format);
-	bool LoadPictData(PictFormat format, char * image_name);
+	bool LoadPictData(PictFormat format, const char * image_name, 
+					  struct RTFProps_ImageProps & imgProps);
+	bool InsertImage (const UT_ByteBuf * buf, const char * image_name,
+					  const struct RTFProps_ImageProps & imgProps);
 	
 	RTFFontTableItem* GetNthTableFont(UT_uint32 fontNum);
 	UT_uint32 GetNthTableColour(UT_uint32 colNum);
@@ -491,6 +547,7 @@ private:
 
 	// Character property handlers
 	bool ResetCharacterAttributes();
+	bool buildCharacterProps(UT_String & propBuffer);
 	bool ApplyCharacterAttributes();
 	bool HandleBoolCharacterProp(bool state, bool* pProp);
 	bool HandleDeleted(bool state);
@@ -506,6 +563,7 @@ private:
 	bool HandleSubscript(bool state);
 	bool HandleSubscriptPosition(UT_uint32 pos);
 	bool HandleFontSize(long sizeInHalfPoints);
+	bool HandleBookmark (RTFBookmarkType type);
 	bool HandleListTag(long id);
 
 	// Generic handlers
@@ -539,16 +597,17 @@ private:
 	    RTF_TOKEN_ERROR = -1
 	} RTFTokenType;
 	RTFTokenType NextToken (unsigned char *pKeyword, long* pParam, 
-							bool* pParamUsed, UT_uint32 len);
-	
+							bool* pParamUsed, UT_uint32 len, bool bIgnoreWhiteSpace=false);
+
+
 // import member vars
 private:
 	UT_GrowBuf m_gbBlock;
+	char *m_szFileDirName;
 
 	int m_groupCount;
 	bool m_newParaFlagged;
 	bool m_newSectionFlagged;
-
 	int m_cbBin;
 
 	// headers and footers
@@ -558,6 +617,12 @@ private:
 	// TODO: handle \titlepg and \facingpg cases.
 	UT_uint32       m_currentHdrID;     // these are numbers.
 	UT_uint32       m_currentFtrID;
+	UT_uint32       m_currentHdrEvenID;     // these are numbers.
+	UT_uint32       m_currentFtrEvenID;
+	UT_uint32       m_currentHdrFirstID;     // these are numbers.
+	UT_uint32       m_currentFtrFirstID;
+	UT_uint32       m_currentHdrLastID;     // these are numbers.
+	UT_uint32       m_currentFtrLastID;
 
 
 	UT_Stack m_stateStack;
@@ -602,7 +667,8 @@ private:
 	UT_Vector           m_vecWord97ListOveride;
 	void _appendHdrFtr ();
 	bool _appendField (const XML_Char *xmlField);
-	XML_Char *_parseFldinstBlock (UT_ByteBuf & buf, XML_Char *xmlField);
+	XML_Char *_parseFldinstBlock (UT_ByteBuf & buf, XML_Char *xmlField, bool & isXML);
+	bool                m_bAppendAnyway;
 };
 
 #endif /* IE_IMP_RTF_H */
