@@ -85,9 +85,17 @@ wvOutputTextChar (U16 eachchar, U8 chartype, wvParseStruct * ps, CHP * achp)
     if ((v <= WORD7) && (!ps->fib.fFarEast))
       {
 	  FFN currentfont;
-	  currentfont = ps->fonts.ffn[achp->ftc];
-	  /* Return 0 if no match */
-	  lid = wvnLocaleToLIDConverter (currentfont.chs);
+	
+	  if (ps->fonts.ffn == NULL)
+          {
+	      lid = 0;
+	  }
+	  else
+          {
+	  	currentfont = ps->fonts.ffn[achp->ftc];
+	  	/* Return 0 if no match */
+	  	lid = wvnLocaleToLIDConverter (currentfont.chs);
+	  }
       }
     if (!lid)
 	lid = achp->lidDefault;
@@ -222,7 +230,7 @@ U16 wvLangToLIDConverter ( const char * lang )
   else if(!strcmp(lang, "it-IT"))
     return 0x0410;
   else if(!strcmp(lang, "iw-IL"))
-    return 0x0410;
+    return 0x040d;
   else if(!strcmp(lang, "ka-GE"))
     return 0x0437;
   else if(!strcmp(lang, "ja-JP"))
@@ -249,6 +257,8 @@ U16 wvLangToLIDConverter ( const char * lang )
     return 0x041f;
   else if(!strcmp(lang, "vi-VN"))
     return 0x042a;
+  else if(!strcmp(lang, "lt-LT"))
+    return 0x0427;
   else if(!strcmp(lang, "-none-"))
     return 0x0400;
   else
@@ -468,10 +478,12 @@ wvLIDToLangConverter (U16 lid)
     case 0x0412: /* Korean */
       return "ko-KR";
 
-#if 0
-    case 0x0426: /* Latvian */
     case 0x0827: /* Lithuanian (Classic) */
     case 0x0427: /* Lithuanian */
+      return "lt-LT";
+
+#if 0
+    case 0x0426: /* Latvian */
     case 0x042f: /* Macedonian */
     case 0x083e: /* Malay (Brunei Darussalam) */
     case 0x043e: /* Malay (Malaysian) */
@@ -580,11 +592,12 @@ wvLIDToCodePageConverter (U16 lid)
       switch (lid)
 		{
 #if 0
-      	case 0x0c04:		/*Chinese (Hong Kong SAR, PRC) */
       	case 0x1404:		/*Chinese (Macau SAR) */
 #endif
+      	case 0x0c04:		/*Chinese (Hong Kong SAR, PRC) */
+	  	CPNAME_OR_FALLBACK ("CP950", "BIG5-HKSCS");
       	case 0x0804:		/*Chinese (PRC) */
-	  	CPNAME_OR_FALLBACK ("CP936", "GB2312");
+	  	CPNAME_OR_FALLBACK ("CP936", "GBK");
 #if 0
       	case 0x1004:		/*Chinese (Singapore) */
 #endif
@@ -1065,20 +1078,18 @@ wvIsEmptyPara (PAP * apap, expand_data * data, int inc)
      */
     if (apap->fInTable == 1)
       {
+#if 0
 	  wvTrace (
 		   ("This Para is in cell %d %d\n", data->whichrow,
 		    data->whichcell));
 	  if (*data->vmerges)
 	    {
 		/* only ignore a vertically merged cell if the setting in the config file have been set that way */
-		if (data && data->sd
+		if (data
+		    && data->sd
 		    && data->sd->elements[TT_TABLEOVERRIDES].str
 		    && data->sd->elements[TT_TABLEOVERRIDES].str[5])
 		  {
-		      wvTrace (
-			       ("%d\n",
-				(*data->vmerges)[data->whichrow][data->
-								 whichcell]));
 		      if ((*data->vmerges)[data->whichrow][data->whichcell] ==
 			  0)
 
@@ -1090,6 +1101,9 @@ wvIsEmptyPara (PAP * apap, expand_data * data, int inc)
 			}
 		  }
 	    }
+#else
+	  return 0;
+#endif
       }
     return (0);
 }
@@ -1350,6 +1364,7 @@ wvConvertUnicodeToLaTeX (U16 char16)
       case 37:
 	  printf ("\\%%");
 	  return (1);
+      case 10:
       case 11:
 	  printf ("\\\\\n");
 	  return (1);
@@ -1363,6 +1378,8 @@ wvConvertUnicodeToLaTeX (U16 char16)
 	  /* case 45: minus/hyphen, pass through */
 
       case 12:
+	  printf("\\newpage\n");
+	  return (1);
       case 13:
       case 14:
       case 7:
@@ -1370,7 +1387,6 @@ wvConvertUnicodeToLaTeX (U16 char16)
       case 9:
 	  printf ("\\hfill{}");	/* tab -- horrible cludge */
 	  return (1);
-
       case 0xf020:
 	  printf (" ");		/* Mac specialty ? MV 10.10.2000 */
 	  return (1);
@@ -2396,7 +2412,10 @@ wvConvertUnicodeToLaTeX (U16 char16)
       }
     /* Debugging aid: */
     if (char16 >= 0x80)
+      {
 	printf ("[%x]", char16);
+	return (1);
+      }
     return (0);
 }
 
